local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO = E:LoadModules('Config')

--[[
	New font option prototype
	------------------------------------
	
	['enable'] = true,
	['position'] = 'CENTER',
	['xOffset'] = 0,
	['yOffset'] = 0,
	['fontShadowColor'] = {0, 0, 0, 1},
	['xFontShadowOffset'] = 1,
	['yFontShadowOffset'] = 1,
	['horizontalAlign'] = 'CENTER',
	['verticalAlign'] = 'MIDDLE',
	['fontColor'] = {1, 1, 1, 1},
	['width'] = 0,
	['fontFlags'] = 'None',
	['fontType'] = 'PT Sans Narrow',
	['fontHeight'] = 12,
	['drawLayer'] = 'ARTWORK',
	['textFormat'] = '',
--]]

local DefaultFont = 'PT Sans Narrow'
local DefaultStatusbarTexture = "CUI V2"

function CO:SetDefaults()	
	-- Table to reference some of the most-repeating option values from
	CO.Defaults = {
		['auras'] = {
			['filterType'] = -1,
			['alpha'] = 1,
			['minDuration'] = 0,
			['time'] = {
				['enable'] = true,
				['position'] = 'CENTER',
				['xOffset'] = 0,
				['yOffset'] = 0,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'CENTER',
				['verticalAlign'] = 'MIDDLE',
				['fontColor'] = {1, 0.96, 0.41, 1},
				['width'] = 30,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 12,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
			['count'] = {
				['enable'] = true,
				['position'] = 'BOTTOMRIGHT',
				['xOffset'] = 0,
				['yOffset'] = 0,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'RIGHT',
				['verticalAlign'] = 'MIDDLE',
				['fontColor'] = {1,0.96,0.41, 1},
				['width'] = 0,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 11,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
		},
		['unitframe'] = {
			['health'] = {
				['staticColor'] = {0.75, 0.75, 0.75, 1},
				['absorbs'] = {
					['damage'] = {
						['enable'] = true,
						['textureSizeMultiplier'] = 7,
						['useStripes'] = false,
						['borderColor'] = {0, 0.5, 0.5, 1},
						['textureColor'] = {0, 0.5, 0.5, 1},
						['onHealthbar'] = true,
						['alpha'] = 0.6,
					},
					['healing'] = {
						['enable'] = true,
						['textureSizeMultiplier'] = 7,
						['useStripes'] = false,
						['borderColor'] = {0.811, 0.176, 0.176, 1},
						['textureColor'] = {0.811, 0.176, 0.176, 1},
						['onHealthbar'] = false,
						['alpha'] = 0.65,
					},
				},
			},
			['altPower'] = {
				['enable'] = true,
				['width'] = 200,
				['height'] = 10,
				['position'] = 'BOTTOM',
				['offsetX'] = 0,
				['offsetY'] = 0,
				['autoWidth'] = true,
				['autoHeight'] = false,
				['barInverseFill'] = false,
				['barOrientation'] = 'HORIZONTAL',
				['barSmooth'] = true,
				['overrideBarTexture'] = false,
				['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
				['barBorderSize'] = 1,
				['barBorderColor'] = {0, 0, 0, 1},
			},
			['targetIcon'] = {
				['enable'] = true,
				['position'] = 'TOP',
				['offsetX'] = 0,
				['offsetY'] = 5,
				['size'] = 24,
			},
			['leaderIcon'] = {
				['enable'] = true,
				['position'] = 'TOPLEFT',
				['offsetX'] = 8,
				['offsetY'] = 5,
				['size'] = 18,
			},
			['roleIcon'] = {
				['enable'] = true,
				['position'] = 'TOPRIGHT',
				['offsetX'] = -8,
				['offsetY'] = 5,
				['size'] = 18,
			},
			['readyCheckIndicator'] = {
				['enable'] = true,
				['position'] = 'CENTER',
				['offsetX'] = 0,
				['offsetY'] = 0,
				['size'] = 20,
			},
			['resIndicator'] = {
				['enable'] = true,
				['position'] = 'TOP',
				['offsetX'] = 0,
				['offsetY'] = 0,
				['size'] = 30,
			},
			['summonIndicator'] = {
				['enable'] = true,
				['position'] = 'CENTER',
				['offsetX'] = 0,
				['offsetY'] = 20,
				['size'] = 30,
			},
			['healPrediction'] = {
				['enable'] = true,
			},
			['threat'] = {
				['enable'] = true,
				['position'] = 'RIGHT',
				['xOffset'] = 0,
				['yOffset'] = 0,
				['width'] = 5,
				['height'] = 65,
				['autoWidth'] = false,
				['autoHeight'] = true,
				['barInverseFill'] = false,
				['barOrientation'] = 'VERTICAL',
				['barSmooth'] = true,
				['overrideBarTexture'] = false,
				['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
				['barBorderSize'] = 1,
				['barBorderColor'] = {0, 0, 0, 1},
			},
			['portrait'] = {
				['style'] = "3D",
				['overlay'] = true,
				['position'] = "LEFT",
				['width'] = 32,
				['height'] = 32,
				['offsetX'] = 0,
				['offsetY'] = 0,
			},
			['power'] = {
				['enable'] = true,
				['autoWidth'] = true,
				['autoHeight'] = false,
			},
			['rangeIndicator'] = true,
		},
		['actionbar'] = {
			['cooldownFormat'] = 'Actionbars',
			['hotkey'] = {
				['enable'] = true,
				['position'] = 'TOPRIGHT',
				['xOffset'] = -2,
				['yOffset'] = -4,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'CENTER',
				['verticalAlign'] = 'MIDDLE',
				['fontColor'] = {1, 1, 1, 1},
				['width'] = 0,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 13,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
			['cooldown'] = {
				['enable'] = true,
				['position'] = 'CENTER',
				['xOffset'] = 0,
				['yOffset'] = 0,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'CENTER',
				['verticalAlign'] = 'MIDDLE',
				['fontColor'] = {1, 1, 1, 1},
				['width'] = 0,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 20,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
			['count'] = {
				['enable'] = true,
				['position'] = 'BOTTOMRIGHT',
				['xOffset'] = -2,
				['yOffset'] = 2,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'CENTER',
				['verticalAlign'] = 'MIDDLE',
				['fontColor'] = {1, 1, 1, 1},
				['width'] = 0,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 12,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
			['macro'] = {
				['enable'] = true,
				['position'] = 'BOTTOM',
				['xOffset'] = 0,
				['yOffset'] = 0,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'CENTER',
				['verticalAlign'] = 'MIDDLE',
				['fontColor'] = {1, 1, 1, 1},
				['width'] = 42,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 12,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
			
			['flash'] = {
				['enable'] = true,
				['blendMode'] = "ADD",
				['drawLayer'] = "OVERLAY",
				['drawLayerSubLevel'] = 5,
				['showAboveCooldown'] = false,
				['fadeInTime'] = 0.025,
				['fadeOutTime'] = 0.25,
				['rgba'] = {useClassColor = true, {1,1,1, 1}},
				['forceTexture'] = nil,
			},
		},
		['nameplates'] = {
			['name'] = {
				['enable'] = true,
				['position'] = 'TOP',
				['xOffset'] = 0,
				['yOffset'] = 15,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'CENTER',
				['verticalAlign'] = 'BOTTOM',
				['fontColor'] = {1, 1, 1, 1},
				['width'] = 0,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 11,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
			['health'] = {
				['enable'] = true,
				['position'] = 'RIGHT',
				['xOffset'] = 55,
				['yOffset'] = 0,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'LEFT',
				['verticalAlign'] = 'MIDDLE',
				['fontColor'] = {0.63, 0.93, 1, 1},
				['width'] = 50,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 11,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
			['level'] = {
				['enable'] = true,
				['position'] = 'LEFT',
				['xOffset'] = -55,
				['yOffset'] = 0,
				['fontShadowColor'] = {0, 0, 0, 1},
				['xFontShadowOffset'] = 1,
				['yFontShadowOffset'] = 1,
				['horizontalAlign'] = 'RIGHT',
				['verticalAlign'] = 'MIDDLE',
				['fontColor'] = {0.63, 0.93, 1, 1},
				['width'] = 50,
				['fontFlags'] = 'OUTLINE',
				['fontType'] = DefaultFont,
				['fontHeight'] = 11,
				['drawLayer'] = 'ARTWORK',
				['textFormat'] = '',
			},
		},
	}
	
	E.MoverDefaults = {
		['*'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'CENTER',
			['point'] = 'CENTER',
			['yOffset'] = 0,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_MinimapHolderMover'] = {
			['xOffset'] = 0,
			['yOffset'] = 0,
			['point'] = 'TOPRIGHT',
			['relativePoint'] = 'TOPRIGHT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['UIWidgetTopCenterContainerFrameMover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'TOP',
			['point'] = 'TOP',
			['yOffset'] = -100,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['UIWidgetBelowMinimapContainerFrameMover'] = {
			["xOffset"] = -383.578125,
			["yOffset"] = -250.6661987304688,
			["point"] = "TOPRIGHT",
			["relativePoint"] = "RIGHT",
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_TotemBarMover'] = {
			['xOffset'] = 160,
			['yOffset'] = 75,
			['point'] = 'LEFT',
			['relativePoint'] = 'LEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['VehicleSeatIndicatorMover'] = {
			['xOffset'] = -255.999954223633,
			['yOffset'] = -222.666610717773,
			['point'] = 'TOPRIGHT',
			['relativePoint'] = 'TOPRIGHT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['RaidRoleFrameMover'] = {
			['xOffset'] = 0,
			['yOffset'] = -30,
			['point'] = 'TOP',
			['relativePoint'] = 'TOP',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['RaidControlFrameMover'] = {
			['xOffset'] = 350,
			['yOffset'] = 0,
			['point'] = 'TOPLEFT',
			['relativePoint'] = 'TOPLEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['arenaHolderMover'] = {
			['xOffset'] = 285.555694580078,
			['relativePoint'] = 'TOPLEFT',
			['point'] = 'TOPLEFT',
			['yOffset'] = -131.666549682617,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['bossHolderMover'] = {
			['xOffset'] = 418.889190673828,
			['yOffset'] = -179.444259643555,
			['point'] = 'TOP',
			['relativePoint'] = 'TOP',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['partyHolderMover'] = {
			['xOffset'] = 69.4441604614258,
			['yOffset'] = -130.555435180664,
			['point'] = 'TOPLEFT',
			['relativePoint'] = 'TOPLEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['AlertFrameHolderMover'] = {
			['xOffset'] = -0.553298354148865,
			['yOffset'] = 234.667053222656,
			['point'] = 'BOTTOM',
			['relativePoint'] = 'BOTTOM',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['BNToastFrameMover'] = {
			['xOffset'] = -1.66770040988922,
			['relativePoint'] = 'TOPRIGHT',
			['point'] = 'TOPRIGHT',
			['yOffset'] = -254.999938964844,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_focusMover'] = {
			['xOffset'] = -315,
			['relativePoint'] = 'BOTTOMRIGHT',
			['point'] = 'BOTTOMRIGHT',
			['yOffset'] = 140,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['MicroMenuMover'] = {
			['xOffset'] = -2,
			['yOffset'] = 2,
			['point'] = 'TOPLEFT',
			['relativePoint'] = 'TOPLEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['ZoneActionButtonHolderMover'] = {
			['xOffset'] = 475,
			['relativePoint'] = 'LEFT',
			['point'] = 'LEFT',
			['yOffset'] = 75,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['ExtraAbilityFrameHolderMover'] = {
			['xOffset'] = 494,
			['yOffset'] = -429,
			['relativePoint'] = 'TOPLEFT',
			['point'] = 'TOPLEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['ExtraActionBarFrameHolderMover'] = {
			['xOffset'] = -270,
			['yOffset'] = 75,
			['relativePoint'] = 'CENTER',
			['point'] = 'CENTER',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['PlayerPowerBarAltMover'] = {
			['xOffset'] = 0,
			['yOffset'] = 170,
			['point'] = 'BOTTOM',
			['relativePoint'] = 'BOTTOM',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['raidHolderMover'] = {
			['xOffset'] = 15,
			['yOffset'] = 292,
			['point'] = 'BOTTOMLEFT',
			['relativePoint'] = 'BOTTOMLEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['raid40HolderMover'] = {
			['xOffset'] = 15,
			['yOffset'] = -168,
			['relativePoint'] = 'LEFT',
			['point'] = 'LEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		["maintankHolderMover"] = {
			['xOffset'] = 418,
			['yOffset'] = -360,
			['relativePoint'] = 'BOTTOMLEFT',
			['point'] = 'BOTTOMLEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['TooltipAnchorMover'] = {
			['xOffset'] = -100,
			['yOffset'] = 230,
			['relativePoint'] = 'BOTTOMRIGHT',
			['point'] = 'BOTTOMRIGHT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_AlternatePowerMover'] = {
			['xOffset'] = 0,
			['yOffset'] = 5,
			['relativePoint'] = 'TOP',
			['point'] = 'TOP',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_ActionBar1'},
		},
		['ChatParentMover'] = {
			['xOffset'] = 37,
			['yOffset'] = 73,
			['point'] = 'BOTTOMLEFT',
			['relativePoint'] = 'BOTTOMLEFT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_targettargetMover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'BOTTOMRIGHT',
			['point'] = 'BOTTOMRIGHT',
			['yOffset'] = 0,
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Target'},
		},
		['CUI_petMover'] = {
			['xOffset'] = 0,
			['yOffset'] = 0,
			['relativePoint'] = 'BOTTOMLEFT',
			['point'] = 'BOTTOMLEFT',		
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Player'},
		},
		['CUI_targetMover'] = {
			['xOffset'] = 414.999847412109,
			['relativePoint'] = 'BOTTOM',
			['point'] = 'BOTTOM',
			['yOffset'] = 231.111526489258,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_focustargetMover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'BOTTOMRIGHT',
			['point'] = 'BOTTOMRIGHT',
			['yOffset'] = -8,
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Focus'},
		},
		['CUI_playerMover'] = {
			['xOffset'] = -415,
			['relativePoint'] = 'BOTTOM',
			['point'] = 'BOTTOM',
			['yOffset'] = 231.111389160156,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		
		['CUIPlayerBuffsMover'] = {
			['xOffset'] = -220,
			['yOffset'] = -10,
			['point'] = 'TOPRIGHT',
			['relativePoint'] = 'TOPRIGHT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUIPlayerDebuffsMover'] = {
			['xOffset'] = -220,
			['yOffset'] = -105,
			['point'] = 'TOPRIGHT',
			['relativePoint'] = 'TOPRIGHT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['AuraBarContainerplayerMover'] = {
			['xOffset'] = 0,
			['yOffset'] = 40,
			['relativePoint'] = 'TOP',
			['point'] = 'TOP',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Player'},
		},
		['AuraBarContainertargetMover'] = {
			['xOffset'] = 0,
			['yOffset'] = 40,
			['relativePoint'] = 'TOP',
			['point'] = 'TOP',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Target'},
		},
		['ObjectiveTrackerFrameMover'] = {
			['xOffset'] = -95.2779083251953,
			['yOffset'] = 120.475082397461,
			['point'] = 'RIGHT',
			['relativePoint'] = 'RIGHT',
		},
		
		['CUI_targettargetCastbar1Mover'] = {
			['xOffset'] = -2,
			['yOffset'] = 0,
			['point'] = 'LEFT',
			['relativePoint'] = 'LEFT',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Targettarget'},
		},
		['CUI_focustargetCastbar1Mover'] = {
			['xOffset'] = -2,
			['yOffset'] = 0,
			['point'] = 'LEFT',
			['relativePoint'] = 'LEFT',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Focustarget'},
		},
		['CUI_petCastbar1Mover'] = {
			['xOffset'] = 2,
			['yOffset'] = 0,
			['point'] = 'RIGHT',
			['relativePoint'] = 'RIGHT',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Pet'},
		},
		['CUI_playerCastbar1Mover'] = {
			['xOffset'] = 2,
			['yOffset'] = 0,
			['point'] = 'RIGHT',
			['relativePoint'] = 'RIGHT',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Player'},
		},
		['CUI_targetCastbar1Mover'] = {
			['xOffset'] = -2,
			['yOffset'] = 0,
			['point'] = 'LEFT',
			['relativePoint'] = 'LEFT',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Target'},
		},
		
		['CUI_focusCastbar1Mover'] = {
			['xOffset'] = -2,
			['yOffset'] = 0,
			['point'] = 'LEFT',
			['relativePoint'] = 'LEFT',
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Focus'},
		},
		
		['CUI_PetActionbarMover'] = {
			['xOffset'] = 1.77710914611816,
			['relativePoint'] = 'TOPLEFT',
			['point'] = 'TOPLEFT',
			['yOffset'] = -36.055721282959,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar1Mover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'BOTTOM',
			['point'] = 'BOTTOM',
			['yOffset'] = 105,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar4Mover'] = {
			['xOffset'] = -10,
			['relativePoint'] = 'RIGHT',
			['point'] = 'RIGHT',
			['yOffset'] = 0,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar8Mover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'CENTER',
			['point'] = 'CENTER',
			['yOffset'] = 0,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_StanceBarMover'] = {
			['point'] = 'TOPLEFT',
			['relativePoint'] = 'TOPLEFT',
			['xOffset'] = 35,
			['yOffset'] = -37,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar6Mover'] = {
			['xOffset'] = -50,
			['yOffset'] = 0,
			['point'] = 'RIGHT',
			['relativePoint'] = 'RIGHT',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar3Mover'] = {
			['xOffset'] = 315,
			['yOffset'] = 25,
			['point'] = 'BOTTOM',
			['relativePoint'] = 'BOTTOM',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar5Mover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'BOTTOM',
			['point'] = 'BOTTOM',
			['yOffset'] = 25,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar2Mover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'BOTTOM',
			['point'] = 'BOTTOM',
			['yOffset'] = 65,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar7Mover'] = {
			['xOffset'] = -315,
			['relativePoint'] = 'BOTTOM',
			['point'] = 'BOTTOM',
			['yOffset'] = 25,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar9Mover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'CENTER',
			['point'] = 'CENTER',
			['yOffset'] = 40,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_ActionBar10Mover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'CENTER',
			['point'] = 'CENTER',
			['yOffset'] = 80,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_BagBarHolderMover'] = {
			['xOffset'] = 0,
			['relativePoint'] = 'BOTTOMRIGHT',
			['point'] = 'BOTTOMRIGHT',
			['yOffset'] = 50,
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['CUI_LeaveVehicleButtonMover'] = {
			['xOffset'] = 280,
			['yOffset'] = -100,
			['point'] = 'TOP',
			['relativePoint'] = 'TOP',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		['MirrorTimerHolderMover'] = {
			['xOffset'] = 0,
			['yOffset'] = -77,
			['point'] = 'TOP',
			['relativePoint'] = 'TOP',
			['enableAttach'] = false,
			['attachTo'] = {''},
		},
		["TalkingHeadFrameMover"] = {
			["xOffset"] = 0,
			["yOffset"] = -170,
			["point"] = "TOP",
			["relativePoint"] = "TOP",
			['attachTo'] = {''},
		},
		["ArcheologyDigsiteProgressBarMover"] = {
			["xOffset"] = 0,
			["yOffset"] = 200,
			["point"] = "BOTTOM",
			["relativePoint"] = "BOTTOM",
			['attachTo'] = {''},
		},
		["DurabilityFrameMover"] = {
			["xOffset"] = -18,
			["yOffset"] = 273,
			["point"] = "BOTTOMRIGHT",
			["relativePoint"] = "BOTTOMRIGHT",
			['attachTo'] = {''},
		},
		["CUI_BattlerezChargesMover"] = {
			["xOffset"] = -15,
			["yOffset"] = 0,
			["point"] = "LEFT",
			["relativePoint"] = "LEFT",
			['enableAttach'] = true,
			['attachTo'] = {'CUI_UF_Player'},
		},
	}
	
	E.ConfigDefaults = {
		char = CO:GetCharacterDefaults(),
		global = CO:GetGlobalDefaults(),
		profile = {
			['code'] = {
				['testFunc'] = '',
			},
			['LUAErrors'] = true,
			['media'] = {
				['worldNameFont'] = 'Michroma',
				['worldDamageFont'] = 'EncodeSans Regular',
				['worldDefaultFont'] = 'EncodeSans Regular',
				
				['overrideGeneralFont'] = true,
				['generalFont'] = 'PT Sans Narrow',
				['generalFontSize'] = 12,
			},
			['castbar'] = {
				['fontType'] = 'Oswald',
			},
			['utility'] = {
				['pullTimer'] = 10,
				['autoSellNoBoE'] = true,
				['autoSellNoUncollectedTransmog'] = true,
				['autoSellOldGems'] = false,
				['autoSellOldGemsBelowIlvl'] = 50,
				['autoSellBelowIlvlEnable'] = false,
				['autoSellBelowIlvlEnableAtMaxlevel'] = true,
				['autoSellBelowIlvl'] = 60,
				['autoSellGreys'] = false,
				['autoSellGreysReport'] = true,
			},
			['blizzard'] = {
				['talkingHead'] = {
					['scale'] = 0.8,
				},
				['dungeonReadyDialog'] = {
					['enable'] = true,
					['hideBigWigs'] = true,
				},
				['chatBubbles'] = {
					-- ['enable'] = true, -- CHAR SETTING
					['borderSize'] = 1,
					['borderColor'] = {0.75, 0.75, 0.75,1},
					['backgroundColor'] = {0.1, 0.1, 0.1, 1},
					['name'] = {
						['enable'] = true,
						['position'] = 'TOPLEFT',
						['xOffset'] = -10,
						['yOffset'] = 5,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'LEFT',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 0.96, 0.41, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 12,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
					['text'] = {
						['enable'] = true,
						['position'] = 'CENTER',
						['xOffset'] = 0,
						['yOffset'] = 0,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'CENTER',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 0.96, 0.41, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 12,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
				},
			},
			['chat'] = {
				['fontType'] = 'PT Sans Narrow',
				['showRoles'] = true,
			},
			['layout'] = {			
				['fps'] = {
					['enable'] = true,
					['position'] = 'BOTTOMLEFT',
					['xOffset'] = 25,
					['yOffset'] = 8,
					['fontShadowColor'] = {0, 0, 0, 1},
					['xFontShadowOffset'] = 1,
					['yFontShadowOffset'] = 1,
					['horizontalAlign'] = 'CENTER',
					['verticalAlign'] = 'MIDDLE',
					['fontColor'] = {1, 1, 1, 1},
					['width'] = 0,
					['fontFlags'] = 'OUTLINE',
					['fontType'] = 'PT Sans Narrow',
					['fontHeight'] = 12,
					['drawLayer'] = 'ARTWORK',
					['textFormat'] = '',
				},
				['ping'] = {
					['enable'] = true,
					['position'] = 'BOTTOMRIGHT',
					['xOffset'] = -25,
					['yOffset'] = 8,
					['fontShadowColor'] = {0, 0, 0, 1},
					['xFontShadowOffset'] = 1,
					['yFontShadowOffset'] = 1,
					['horizontalAlign'] = 'CENTER',
					['fontColor'] = {1, 1, 1, 1},
					['width'] = 0,
					['fontFlags'] = 'OUTLINE',
					['fontType'] = 'PT Sans Narrow',
					['fontHeight'] = 12,
					['drawLayer'] = 'ARTWORK',
					['textFormat'] = '',
				},
				['zone'] = {
					['enable'] = true,
					['position'] = 'TOP',
					['xOffset'] = 0,
					['yOffset'] = -8,
					['fontShadowColor'] = {0, 0, 0, 1},
					['xFontShadowOffset'] = 1,
					['yFontShadowOffset'] = 1,
					['horizontalAlign'] = 'CENTER',
					['verticalAlign'] = 'TOP',
					['fontColor'] = {1, 1, 1, 1},
					['width'] = 300,
					['height'] = 20,
					['fontFlags'] = 'OUTLINE',
					['fontType'] = 'PT Sans Narrow',
					['fontHeight'] = 15,
					['drawLayer'] = 'ARTWORK',
					['textFormat'] = '',
				},
				['coordx'] = {
					['enable'] = true,
					['position'] = 'TOP',
					['xOffset'] = -300,
					['yOffset'] = -10,
					['fontShadowColor'] = {0, 0, 0, 1},
					['xFontShadowOffset'] = 1,
					['yFontShadowOffset'] = 1,
					['horizontalAlign'] = 'CENTER',
					['verticalAlign'] = 'MIDDLE',
					['fontColor'] = {1, 1, 1, 1},
					['width'] = 0,
					['fontFlags'] = 'OUTLINE',
					['fontType'] = DefaultFont,
					['fontHeight'] = 12,
					['drawLayer'] = 'ARTWORK',
					['textFormat'] = '',
				},
				['coordy'] = {
					['enable'] = true,
					['position'] = 'TOP',
					['xOffset'] = 300,
					['yOffset'] = -10,
					['fontShadowColor'] = {0, 0, 0, 1},
					['xFontShadowOffset'] = 1,
					['yFontShadowOffset'] = 1,
					['horizontalAlign'] = 'CENTER',
					['verticalAlign'] = 'MIDDLE',
					['fontColor'] = {1, 1, 1, 1},
					['width'] = 0,
					['fontFlags'] = 'OUTLINE',
					['fontType'] = DefaultFont,
					['fontHeight'] = 12,
					['drawLayer'] = 'ARTWORK',
					['textFormat'] = '',
				},
				['barExperience'] = {
					['enable'] = true,
					['position'] = 'BOTTOM',
					['offsetX'] = 0,
					['offsetY'] = 0,
					['width'] = 746,
					['height'] = 15,
					['reverseFill'] = false,
					['fillOrientation'] = 'HORIZONTAL',
					['borderSize'] = 2,
					['borderColor'] = {0, 0, 0, 1},
					['backgroundColor'] = {0.08, 0.08, 0.08, 0.5},
					['style'] = 'normal',
					['font'] = {
						['enable'] = true,
						['position'] = 'CENTER',
						['xOffset'] = 0,
						['yOffset'] = 0,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'CENTER',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 1, 1, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 11,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
				},
				['barAzerite'] = {
					['enable'] = true,
					['position'] = 'BOTTOM',
					['offsetX'] = 0,
					['offsetY'] = 14,
					['width'] = 746,
					['height'] = 8,
					['reverseFill'] = false,
					['fillOrientation'] = 'HORIZONTAL',
					['borderSize'] = 2,
					['borderColor'] = {0, 0, 0, 1},
					['backgroundColor'] = {0.08, 0.08, 0.08, 0.75},
					['underlayColor'] = {0.08, 0.08, 0.08, 0},
					['style'] = 'normal',
					['font'] = {
						['enable'] = true,
						['position'] = 'CENTER',
						['xOffset'] = 0,
						['yOffset'] = 0,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'CENTER',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 1, 1, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 11,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
				},
				['barHonor'] = {
					['enable'] = true,
					['position'] = 'BOTTOMLEFT',
					['offsetX'] = 0,
					['offsetY'] = 14,
					['width'] = 256,
					['height'] = 36,
					['reverseFill'] = false,
					['fillOrientation'] = 'HORIZONTAL',
					['borderSize'] = 1,
					['borderColor'] = {0, 0, 0, 1},
					['backgroundColor'] = {0.08, 0.08, 0.08, 0.5},
					['overlayColor'] = {1, 0.439, 0.478, 1},
					['style'] = 'integrated',
					['font'] = {
						['enable'] = false,
						['position'] = 'CENTER',
						['xOffset'] = 0,
						['yOffset'] = 0,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'CENTER',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 1, 1, 1},
						['width'] = 0,
						['fontFlags'] = 'None',
						['fontType'] = DefaultFont,
						['fontHeight'] = 12,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
				},
				['barReputation'] = {
					['enable'] = true,
					['position'] = 'BOTTOMRIGHT',
					['offsetX'] = 0,
					['offsetY'] = 14,
					['width'] = 256,
					['height'] = 36,
					['reverseFill'] = false,
					['fillOrientation'] = 'HORIZONTAL',
					['borderSize'] = 1,
					['borderColor'] = {0, 0, 0, 1},
					['backgroundColor'] = {0.08, 0.08, 0.08, 0.5},
					['style'] = 'integratedReversed',
					['font'] = {
						['enable'] = false,
						['position'] = 'CENTER',
						['xOffset'] = 0,
						['yOffset'] = 0,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'CENTER',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 1, 1, 1},
						['width'] = 0,
						['fontFlags'] = 'None',
						['fontType'] = DefaultFont,
						['fontHeight'] = 12,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
				},
			},
			['system'] = {			
				['enableTop'] = true,
				['enableBottom'] = true,
				['enableBottomLeft'] = true,
				['enableBottomRight'] = true,
				['layoutUpdateFrequency'] = 0.15,
				['coordsUpdateFrequency'] = 0.15,
			},
			['dataframes'] = {
				['raidroledata'] = {
					['enable'] = true,
					['scale'] = 1,
					['backgroundColor'] = {0.1, 0.1, 0.1, 0},
					['borderColor'] = {0, 0, 0, 0},
					['clickThrough'] = false,
				},
				['raidControl'] = {
					['enable'] = true,
					['scale'] = 1,
					['pullOnEnter'] = false,
				},
				['battlerezCharges'] = {
					['enable'] = true,
					['size'] = 64,
					['time'] = {
						["position"] = "CENTER",
						["fontHeight"] = 16,
						["enable"] = true,
						["fontShadowColor"] = {0,0,0,1},
						["xFontShadowOffset"] = 1,
						["yFontShadowOffset"] = 1,
						["yOffset"] = -12,
						["fontType"] = "PT Sans Narrow",
						["xOffset"] = 0,
						["fontFlags"] = "OUTLINE",
						["width"] = 0,
						["horizontalAlign"] = "LEFT",
						["fontColor"] = {1, 0.84, 0, 1},
					},
					['charges'] = {
						["position"] = "CENTER",
						["fontHeight"] = 32,
						["enable"] = true,
						["fontShadowColor"] = {0,0,0,1},
						["xFontShadowOffset"] = 1,
						["yFontShadowOffset"] = 1,
						["yOffset"] = 11,
						["fontType"] = "PT Sans Narrow",
						["xOffset"] = 0,
						["fontFlags"] = "OUTLINE",
						["width"] = 0,
						["horizontalAlign"] = "LEFT",
						["fontColor"] = {0, 1, 1, 1},
					},
				},
			},
			['colors'] = {
				['classes'] = {
					['WARRIOR'] = E.ClassColors[1],
					['PALADIN'] = E.ClassColors[2],
					['HUNTER'] = E.ClassColors[3],
					['ROGUE'] = E.ClassColors[4],
					['PRIEST'] = E.ClassColors[5],
					['DEATHKNIGHT'] = E.ClassColors[6],
					['SHAMAN'] = E.ClassColors[7],
					['MAGE'] = E.ClassColors[8],
					['WARLOCK'] = E.ClassColors[9],
					['MONK'] = E.ClassColors[10],
					['DRUID'] = E.ClassColors[11],
					['DEMONHUNTER'] = E.ClassColors[12],
				},
				['powers'] = {
					['MANA'] = {0.00,0.44,0.87},
					['RAGE'] = {1,0,0},
					['FOCUS'] = {1,0.5,0.25},
					['ENERGY'] = {1,1,0},
					['COMBO_POINTS'] = {1,0.96,0.41},
					['RUNES'] = {0.5,0.5,0.5},
					['RUNE_READY'] = {0.77, 0.12, 0.23},
					['RUNE_NOT_READY'] = {0.15, 0.15, 0.15},
					['RUNIC_POWER'] = {0,0.82,1},
					['SOUL_SHARDS'] = {0.5,0.32,0.55},
					['LUNAR_POWER'] = {0.3,0.52,0.9},
					['HOLY_POWER'] = {0.95,0.9,0.6},
					['FUEL'] = {0.8,0.6,0},
					['MAELSTROM'] = {0,0.5,1},
					['CHI'] = {0.71,1,00.92},
					['INSANITY'] = {0.4,0,0.8},
					['ARCANE_CHARGES'] = {0.41, 0.80, 0.94},
					['FURY'] = {0.788,0.259,0.992},
					['PAIN'] = {1,0.61,0},
					['STAGGER'] = {
							['light'] = {0, 0.7, 0},
							['medium'] = {0.7, 0.7, 0},
							['heavy'] = {0.7, 0, 0},
						},
					['MAGE_ICICLES'] = {0.41, 0.80, 0.94},
					['ALTERNATE_POWER'] = {0.8,0.6,0},
				},
				['readycheck'] = {
					['ready'] = {0, 125, 0},
					['notready'] = {125, 0, 0},
					['waiting'] = {125, 80, 0},
				},
				['reactions'] = {
					['friendly'] = E.ReactionColors['friendly'],
					['neutral'] = E.ReactionColors['neutral'],
					['unfriendly'] = E.ReactionColors['unfriendly'],
					['hostile'] = E.ReactionColors['hostile'],
				},
				['castbar'] = {
					['success'] = {0, 0.85, 0, 0.95},
					['failed'] = {0.85, 0, 0, 0.95},
					['interruptible'] = {0.85, 0.85, 0.85},
					['notInterruptible'] = {1, 0.772, 0.380},
				},
				['layoutBars'] = {
					['barExperienceNormal'] = {useClassColor = true, {0.631, 0.215, 0.858,1}},
					['barExperienceRested'] = {0, 0, 0.85, 0.95},
					['barAzerite'] = {useClassColor = false, {0.878, 0.780, 0.513,1}},
					['barReputation'] = {useClassColor = false, {0.7, 0.7, 0.7,1}},
				},
				['zones'] = {
					['arena'] = {1.0, 0.1, 0.1},
					['friendly'] = {0.1, 1.0, 0.1},
					['contested'] = {1.0, 0.7, 0},
					['hostile'] = {1.0, 0.1, 0.1},
					['sanctuary'] = {0.41, 0.8, 0.94},
					['combat'] = {1.0, 0.1, 0.1},
					['default'] = {1.0, 0.9294, 0.7607},
				},
				['auras'] = {},
			},
			['numberFormats'] = {
				['Aurabars Player'] = {['isProtected'] = true, ['formatType'] = 'ActionbarCooldown', {['Threshold'] = 0, ['Decimals'] = 2, ['ColorRGB'] = {1, 0, 0}}, {['Threshold'] = 10, ['Decimals'] = 1, ['ColorRGB'] = {0, 1, 0}}, {['Threshold'] = 20, ['Decimals'] = 0, ['ColorRGB'] = {0, 0.3, 1}}},
				['Aurabars Target'] = {['isProtected'] = true, ['formatType'] = 'ActionbarCooldown', {['Threshold'] = 0, ['Decimals'] = 2, ['ColorRGB'] = {1, 0, 0}}, {['Threshold'] = 10, ['Decimals'] = 1, ['ColorRGB'] = {0, 1, 0}}, {['Threshold'] = 20, ['Decimals'] = 0, ['ColorRGB'] = {0, 0.3, 1}}},
				['Actionbars'] = {['isProtected'] = true, ['formatType'] = 'ActionbarCooldown', {['Threshold'] = 0, ['Decimals'] = 1, ['ColorRGB'] = {0.85, 0.1, 0.1}}, {['Threshold'] = 10, ['Decimals'] = 1, ['ColorRGB'] = {1, 1, 0}}, {['Threshold'] = 300, ['Decimals'] = 0, ['ColorRGB'] = {1, 1, 0}}},
			},
			['auras'] = {
				['generalAurabars'] = {
					['useMasque'] = false,
				},
				['units'] = {
					['player'] = {
						['aurabars'] = {
							['enable'] = true,
							['width'] = 278,
							['height'] = 18,
							['gapY'] = 2,
							['iconSize'] = 18,
							['barNum'] = 10,
							['autoColorBarBorder'] = false,
							['autoColorIconBorder'] = true,
							['maxThreshold'] = 300,
							['barBorderColor'] = {['r'] = 0, ['g'] = 0, ['b'] = 0, ['a'] = 1},
							['iconBorderColor'] = {['r'] = 0.1, ['g'] = 0.1, ['b'] = 0.1, ['a'] = 1},
							['cooldownIdentifier'] = "Aurabars Player",
							['filterType'] = -1,
							
							['name'] = {
								['enable'] = true,
								['position'] = 'LEFT',
								['xOffset'] = 10,
								['yOffset'] = 0,
								['fontShadowColor'] = {0, 0, 0, 1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 1, 1, 1},
								['width'] = 0,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['drawLayer'] = 'ARTWORK',
								['textFormat'] = '',
							},
							['time'] = {
								['enable'] = true,
								['position'] = 'RIGHT',
								['xOffset'] = -10,
								['yOffset'] = 0,
								['fontShadowColor'] = {0, 0, 0, 1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['horizontalAlign'] = 'RIGHT',
								['fontColor'] = {1, 1, 1, 1},
								['width'] = 0,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['drawLayer'] = 'ARTWORK',
								['textFormat'] = '',
							},
							
							['backgroundColor'] = {0.15, 0.15, 0.15, 0.85},
						},
					},
					['target'] = {
						['aurabars'] = {
							['enable'] = true,
							['width'] = 278,
							['height'] = 18,
							['gapY'] = 2,
							['iconSize'] = 18,
							['barNum'] = 10,
							['autoColorBarBorder'] = false,
							['autoColorIconBorder'] = true,
							['maxThreshold'] = 300,
							['barBorderColor'] = {['r'] = 0, ['g'] = 0, ['b'] = 0, ['a'] = 1},
							['iconBorderColor'] = {['r'] = 0.1, ['g'] = 0.1, ['b'] = 0.1, ['a'] = 1},
							['cooldownIdentifier'] = "Aurabars Target",
							['filterType'] = -1,
							
							['name'] = {
								['enable'] = true,
								['position'] = 'LEFT',
								['xOffset'] = 10,
								['yOffset'] = 0,
								['fontShadowColor'] = {0, 0, 0, 1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 1, 1, 1},
								['width'] = 0,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['drawLayer'] = 'ARTWORK',
								['textFormat'] = '',
							},
							['time'] = {
								['enable'] = true,
								['position'] = 'RIGHT',
								['xOffset'] = -10,
								['yOffset'] = 0,
								['fontShadowColor'] = {0, 0, 0, 1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['horizontalAlign'] = 'RIGHT',
								['fontColor'] = {1, 1, 1, 1},
								['width'] = 0,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['drawLayer'] = 'ARTWORK',
								['textFormat'] = '',
							},
							
							['backgroundColor'] = {0.15, 0.15, 0.15, 0.85},
						},
					},
				},
			},
			['CVars'] = {
				['nameplateShowSelf'] = false,
				['nameplateShowAll'] = true,
				['test_cameraTargetFocusInteractEnable'] = false,
				['test_cameraTargetFocusEnemyEnable'] = false,
				['test_cameraTargetFocusEnemyStrengthPitch'] = 0.05,
				['cameraYawMoveSpeed'] = 0,
				['test_cameraTargetFocusEnemyStrengthYaw'] = 0.15,
				['test_cameraDynamicPitch'] = false,
				['test_cameraOverShoulder'] = 0,
				['test_cameraHeadMovementStrength'] = 0,
				['cameraPitchMoveSpeed'] = 0,
				['test_cameraDynamicPitchBaseFovPad'] = 0.25,
				['test_cameraDynamicPitchBaseFovPadFlying'] = 0.5,
			},
			['engine'] = {
				['hideActioncamNotification'] = true,
			},
			['minimap'] = {
				['clock'] = {
					['position'] = "TOP",
					['xOffset'] = 0,
					['yOffset'] = -2,
					['width'] = 135,
					['height'] = 16,
					['backgroundColor'] = {useClassColor = false, {0,0,0, 0.6}},
					['borderColor'] = {useClassColor = true, {0,0,0, 1}},
					['text'] = {
						['enable'] = true,
						['position'] = 'CENTER',
						['xOffset'] = 0,
						['yOffset'] = 0,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 1, 1, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 11,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
				},
				['scale'] = 1.5,
				['zoneText'] = {
					['enable'] = false,
				},
				['worldMapButton'] = {
					['enable'] = false,
				},
				['mailIcon'] = {
					['enable'] = false,
				},
				['customMailIcon'] = {
					['enable'] = true,
					['position'] = "BOTTOMRIGHT",
					['xOffset'] = -6,
					['yOffset'] = 0,
					['scale'] = 1.2,
					['rgba'] = {useClassColor = false, {1,1,1, 1}},
				},
				['borderColor'] = {useClassColor = true, {1,1,1, 1}},
				['dbIconRgb'] = {useClassColor = true, {1,1,1, 1}},				
			},
			['worldmap'] = {
				['coords'] = {
					['enable'] = true,
					['position'] = 'BOTTOM',
					['xOffset'] = 50,
					['yOffset'] = 0,
					['fontShadowColor'] = {0, 0, 0, 1},
					['xFontShadowOffset'] = 1,
					['yFontShadowOffset'] = 1,
					['horizontalAlign'] = 'LEFT',
					['fontColor'] = {1, 1, 1, 1},
					['width'] = 220,
					['fontFlags'] = 'None',
					['fontType'] = DefaultFont,
					['fontHeight'] = 12,
					['drawLayer'] = 'ARTWORK',
					['textFormat'] = '',
				},
			},
			['bags'] = {
				['enable'] = true,
				['buttonsPerRow'] = 1,
				['buttonSizeMultiplier'] = 0.7,
				['buttonGap'] = 0,
			},
			['movers'] = E:TableDeepCopy(E.MoverDefaults),
			['tooltip'] = {
				['enable'] = true,
				['showItemCounts'] = true,
				['header'] = {
					['fontType'] = 'PT Sans Narrow',
					['fontSize'] = 14,
					['fontFlags'] = '',
				},
				['body'] = {
					['fontType'] = 'PT Sans Narrow',
					['fontSize'] = 12,
					['fontFlags'] = '',
				},
				['background'] = {
					['border'] = '',
					['backgroundFile'] = 'Interface/Tooltips/UI-Tooltip-Background',
					['rgba'] = {0.1, 0.1, 0.1, 0.95}
				},
				['statusbar'] = {
					['fontType'] = DefaultFont,
					['fontSize'] = 12,
					['fontFlags'] = '',
				},
			},
			['actionbar'] = {
					['global'] = {
						['borderTextureColor'] = {['r'] = 0.35, ['g'] = 0.35, ['b'] = 0.35, ['a'] = 1},
						['normalTextureColor'] = {['r'] = 0.55, ['g'] = 0.55, ['b'] = 0.55, ['a'] = 1},
						['highlightTextureColor'] = {['r'] = 0.9, ['g'] = 0.9, ['b'] = 0.9, ['a'] = 1},
						['pushedTextureColor'] = {['r'] = 0.9, ['g'] = 0.9, ['b'] = 0.9, ['a'] = 1},
						['normalTextureBlendMode'] = 'BLEND',
						['borderTextureBlendMode'] = 'BLEND',
						['highlightTextureBlendMode'] = 'ADD',
						['pushedTextureBlendMode'] = 'ADD',
						['useMasque'] = false,
					},
					['useMasque'] = false,
					['**'] = CO.Defaults.actionbar,
					['micromenu'] = {
						['buttonSizeMultiplier'] = 0.85,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['borderSize'] = 2,
						['borderColor'] = {useClassColor = false, {0.5, 0.5, 0.5, 1}},
						['enable'] = true,
					},
					['zonebar'] = {
						['buttonSizeMultiplier'] = 1,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['xOffset'] = 0,
						['yOffset'] = -350,
						['point'] = 'BOTTOM',
						['relativePoint'] = 'BOTTOM',
						['enable'] = true,
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
						['hotkey'] = {
							['position'] = 'TOPRIGHT',
							['xOffset'] = -5,
							['yOffset'] = -7,
							['horizontalAlign'] = 'RIGHT',
							['fontHeight'] = 16,
						},
					},
					['extrabar'] = {
						['buttonSizeMultiplier'] = 1.25,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['xOffset'] = 0,
						['yOffset'] = -350,
						['point'] = 'BOTTOM',
						['relativePoint'] = 'BOTTOM',
						['enable'] = true,
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
						['hotkey'] = {
							['position'] = 'TOPRIGHT',
							['xOffset'] = -5,
							['yOffset'] = -7,
							['horizontalAlign'] = 'RIGHT',
							['fontHeight'] = 16,
						},
					},
					['totembar'] = {
						['enable'] = true,
						['buttonSizeMultiplier'] = 1,
						['buttonsPerRow'] = 4,
						['buttonNum'] = 4,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['borderColor'] = {useClassColor = false, {0, 0, 0, 1}},
						['duration'] = {
							['enable'] = true,
							['position'] = 'CENTER',
							['xOffset'] = 0,
							['yOffset'] = 0,
							['fontShadowColor'] = {0, 0, 0, 1},
							['xFontShadowOffset'] = 1,
							['yFontShadowOffset'] = 1,
							['horizontalAlign'] = 'CENTER',
							['verticalAlign'] = 'MIDDLE',
							['fontColor'] = {1, 1, 1, 1},
							['width'] = 0,
							['fontFlags'] = 'None',
							['fontType'] = DefaultFont,
							['fontHeight'] = 12,
							['drawLayer'] = 'ARTWORK',
							['textFormat'] = '',
						},
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['stancebar'] = {
						['buttonSizeMultiplier'] = 0.75,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['xOffset'] = 10,
						['yOffset'] = -50,
						['point'] = 'TOPLEFT',
						['relativePoint'] = 'TOPLEFT',
						['enable'] = true,
						['visibilityCondition'] = '[petbattle] 0;1',
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
						['flash'] = {
							['enable'] = true,
							['blendMode'] = "ADD",
							['drawLayer'] = "OVERLAY",
							['drawLayerSubLevel'] = 5,
							['showAboveCooldown'] = false,
							['fadeInTime'] = 0.025,
							['fadeOutTime'] = 0.25,
							['rgba'] = {useClassColor = true, {1,1,1, 1}},
							['forceTexture'] = nil,
						},
					},
					['petbar'] = {
						['buttonSizeMultiplier'] = 0.8,
						['buttonsPerRow'] = -1,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['enable'] = true,
						['xOffset'] = 0,
						['yOffset'] = 105,
						['point'] = 'BOTTOM',
						['relativePoint'] = 'BOTTOM',
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0; [overridebar] 0;[pet] 1;0',
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'RIGHT',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
						['flash'] = {
							['enable'] = true,
							['blendMode'] = "ADD",
							['drawLayer'] = "OVERLAY",
							['drawLayerSubLevel'] = 5,
							['showAboveCooldown'] = false,
							['fadeInTime'] = 0.025,
							['fadeOutTime'] = 0.25,
							['rgba'] = {useClassColor = true, {1,1,1, 1}},
							['forceTexture'] = nil,
						},
					},
					['bar1'] = {
						['buttonSizeMultiplier'] = 0.9,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 7,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['enable'] = true,
						['xOffset'] = 0,
						['yOffset'] = 105,
						['point'] = 'BOTTOM',
						['relativePoint'] = 'BOTTOM',
						['visibilityCondition'] = '[petbattle] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar2'] = {
						['buttonSizeMultiplier'] = 0.9,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['enable'] = true,
						['xOffset'] = -50,
						['yOffset'] = 0,
						['point'] = 'RIGHT',
						['relativePoint'] = 'RIGHT',
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar3'] = {
						['buttonSizeMultiplier'] = 0.85,
						['buttonsPerRow'] = 3,
						['buttonNum'] = 6,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['enable'] = true,
						['xOffset'] = 0,
						['yOffset'] = 20,
						['point'] = 'BOTTOM',
						['relativePoint'] = 'BOTTOM',
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar4'] = {
						['buttonSizeMultiplier'] = 0.9,
						['buttonsPerRow'] = -1,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['enable'] = true,
						['xOffset'] = -10,
						['yOffset'] = 0,
						['point'] = 'RIGHT',
						['relativePoint'] = 'RIGHT',
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'LEFT',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar5'] = {
						['buttonSizeMultiplier'] = 0.9,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['enable'] = true,
						['xOffset'] = 0,
						['yOffset'] = 60,
						['point'] = 'BOTTOM',
						['relativePoint'] = 'BOTTOM',
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar6'] = {
						['buttonSizeMultiplier'] = 0.9,
						['buttonsPerRow'] = 1,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['enable'] = true,
						['xOffset'] = 315,
						['yOffset'] = 20,
						['point'] = 'BOTTOM',
						['relativePoint'] = 'BOTTOM',
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'LEFT',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar7'] = {
						['buttonSizeMultiplier'] = 0.85,
						['buttonsPerRow'] = 3,
						['buttonNum'] = 6,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['xOffset'] = 0,
						['yOffset'] = 0,
						['point'] = 'CENTER',
						['relativePoint'] = 'CENTER',
						['enable'] = true,
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar8'] = {
						['buttonSizeMultiplier'] = 0.9,
						['buttonsPerRow'] = 1,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['enable'] = false,
						['xOffset'] = -315,
						['yOffset'] = 20,
						['point'] = 'BOTTOM',
						['relativePoint'] = 'BOTTOM',
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar9'] = {
						['buttonSizeMultiplier'] = 0.9,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['xOffset'] = 0,
						['yOffset'] = 0,
						['point'] = 'CENTER',
						['relativePoint'] = 'CENTER',
						['enable'] = false,
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
					['bar10'] = {
						['buttonSizeMultiplier'] = 0.9,
						['buttonsPerRow'] = 12,
						['buttonNum'] = 12,
						['buttonGap'] = E:LoadModule('Actionbars').ACTIONBUTTON_GAP,
						['xOffset'] = 0,
						['yOffset'] = 0,
						['point'] = 'CENTER',
						['relativePoint'] = 'CENTER',
						['enable'] = false,
						['visibilityCondition'] = '[petbattle] 0; [vehicleui] 0;[overridebar] 0;1',
						['showGrid'] = true,
						['clickOnDown'] = false,
						['showTooltip'] = true,
						['flyoutDirection'] = 'UP',
						['showOnMouseOver'] = false,
						['alphaActive'] = 1,
						['alphaInactive'] = 0,
						['fadeInSpeed'] = 0.2,
						['fadeOutSpeed'] = 0.2,
						['fadeInCombat'] = 'none',
						['artFill'] = {
							['enable'] = true,
							['borderSize'] = 1,
							['borderColor'] = {0,0,0,1},
							['backgroundColor'] = {0.1,0.1,0.1,1},
							['paddingX'] = 2,
							['paddingY'] = 2,
						},
					},
			},
			['nameplates'] = {
				['barWidth'] = 150,
				['barHeight'] = 18,
				['clickableWidth'] = 160,
				['clickableHeight'] = 70,
				['barTexture'] = DefaultStatusbarTexture,
				['questIcon'] = {
					['enable'] = true,
					['position'] = 'LEFT',
					['xOffset'] = -25,
					['yOffset'] = 0,
					['scale'] = 1,
				},
				['name'] = CO.Defaults.nameplates.name,
				['health'] = CO.Defaults.nameplates.health,
				['level'] = CO.Defaults.nameplates.level,
			},
			['unitframe'] = {
				['dummyShowIndex'] = true,
				['aurasDefaultBorderColor'] = {useClassColor = true, {0, 0, 0, 1}},
				['desaturateOtherDebuffs'] = true,
				['unitBuffs'] = {
					['useMasque'] = false,
				},
				['unitDebuffs'] = {
					['useMasque'] = false,
				},
				['buffs'] = {
					['enable'] = true,		
					['size'] = 32,
					['sortMethod'] = 'TIME',
					['sortDirection'] = '-',
					['maxWraps'] = 2,
					['maxPerRow'] = 9,
					['growthDirectionX'] = 'LEFT',
					['growthDirectionY'] = 'DOWN',
					['gapX'] = 4,
					['gapY'] = 10,
					['borderUseClassColor'] = {useClassColor = true, {0, 0, 0, 1}},
					['time'] = {
						['enable'] = true,
						['position'] = 'CENTER',
						['xOffset'] = 0,
						['yOffset'] = 0,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'CENTER',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {0.921, 0.843, 0.105, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 15,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
					['count'] = {
						['enable'] = true,
						['position'] = 'BOTTOMRIGHT',
						['xOffset'] = -2,
						['yOffset'] = 2,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'RIGHT',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 1, 1, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 12,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
				},
				['debuffs'] = {
					['enable'] = true,		
					['size'] = 36,
					['sortMethod'] = 'TIME',
					['sortDirection'] = '-',
					['maxWraps'] = 2,
					['maxPerRow'] = 8,
					['growthDirectionX'] = 'LEFT',
					['growthDirectionY'] = 'DOWN',
					['gapX'] = 4,
					['gapY'] = 10,
					['time'] = {
						['enable'] = true,
						['position'] = 'CENTER',
						['xOffset'] = 0,
						['yOffset'] = 0,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'CENTER',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {0.921, 0.843, 0.105, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 15,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
					['count'] = {
						['enable'] = true,
						['position'] = 'BOTTOMRIGHT',
						['xOffset'] = -2,
						['yOffset'] = 2,
						['fontShadowColor'] = {0, 0, 0, 1},
						['xFontShadowOffset'] = 1,
						['yFontShadowOffset'] = 1,
						['horizontalAlign'] = 'RIGHT',
						['verticalAlign'] = 'MIDDLE',
						['fontColor'] = {1, 1, 1, 1},
						['width'] = 0,
						['fontFlags'] = 'OUTLINE',
						['fontType'] = DefaultFont,
						['fontHeight'] = 12,
						['drawLayer'] = 'ARTWORK',
						['textFormat'] = '',
					},
				},
				['units'] = {
					['**'] = {
						['buffs'] = CO.Defaults.auras,
						['debuffs'] = CO.Defaults.auras,
						['altPower'] = CO.Defaults.unitframe.altPower,
						['rangeIndicator'] = CO.Defaults.unitframe.rangeIndicator,
						['targetIcon'] = CO.Defaults.unitframe.targetIcon,
						['leaderIcon'] = CO.Defaults.unitframe.leaderIcon,
						['roleIcon'] = CO.Defaults.unitframe.roleIcon,
						['readyCheckIndicator'] = CO.Defaults.unitframe.readyCheckIndicator,
						['resIndicator'] = CO.Defaults.unitframe.resIndicator,
						['summonIndicator'] = CO.Defaults.unitframe.summonIndicator,
						['health'] = CO.Defaults.unitframe.health,
						['healPrediction'] = CO.Defaults.unitframe.healPrediction,
						['threat'] = CO.Defaults.unitframe.threat,
						['power'] = CO.Defaults.unitframe.power,
						['portrait'] = CO.Defaults.unitframe.portrait,
					},
					['all'] = {
						['nameFont'] = {
							['type'] = DefaultFont,
							['height'] = 15,
							['shadow'] = {}
						},
						['health'] = {
							['colorByValue'] = true,
						},
						['highlight'] = {
							['enable'] = true,
							['color'] = {1, 1, 0, 0.1},
							['blendMode'] = "ADD",
							['fadeTime'] = 0.05,
						},
						['targetHighlight'] = {
							['enable'] = true,
							['fadeTime'] = 0.1,
							['borderSize'] = 1,
							['color'] = {1, 0.82, 0.21, 1},
						},
						['castbar'] = {
							['flashSize'] = 7,
							['flashFadeInTime'] = 0.18,
							['flashFadeOutTime'] = 0.25,
						},
						['targetSounds'] = {
							['enable'] = true,
							['situations'] = {
								['npc'] = SOUNDKIT.IG_CHARACTER_NPC_SELECT,
								['aggro'] = SOUNDKIT.IG_CREATURE_AGGRO_SELECT,
								['neutral'] = SOUNDKIT.IG_CREATURE_NEUTRAL_SELECT,
								['lost'] = SOUNDKIT.INTERFACE_SOUND_LOST_TARGET_UNIT,
							},
						},
						['outOfRangeAlpha'] = 0.5,
						['borderWidth'] = 50,
						['borderHeight'] = 1,
						['barTexture'] = DefaultStatusbarTexture,
						['barTexturePath'] = 'Interface/AddOns/CUI/Textures/minimalistic/minimalistic',
					},
					['player'] = {
						['UFInfo'] = {
							['unit'] = 'player',
							['displayName'] = 'Player',
							['displayOrder'] = 1,
							['cluster'] = {
								['size'] = 1,
								['clusterName'] = '',
							},
						},
						['enable'] = true,
						['buffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Buffs',
							['filterType'] = 3,
						},
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['point'] = 'TOP',
								['fontHeight'] = 11,
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['fontFlags'] = 'OUTLINE',
								['xOffset'] = 10,
								['xFontShadowOffset'] = -1,
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 11,
								['position'] = 'RIGHT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 14,
								['fontType'] = DefaultFont,
								['xFontShadowOffset'] = -1,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 175,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'TOPLEFT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -25,
								['yOffset'] = 10,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['alternatePower'] = {
							['width'] = 286,
							['height'] = 10,
							['gap'] = 2,
							['reverseFill'] = false,
							['fillOrientation'] = 'HORIZONTAL',
							['backgroundColor'] = {0.15, 0.15 ,0.15 , 1},
							['borderColor'] = {0, 0, 0, 0.9},
							['borderSize'] = 1,
							['barTexture'] = DefaultStatusbarTexture,
							['artFill'] = {
								['enable'] = true,
								['borderSize'] = 1,
								['borderColor'] = {0,0,0,1},
								['backgroundColor'] = {0.85,0.82,0.52,0.52},
								['paddingX'] = 0,
								['paddingY'] = 0,
							},
							['data'] = {
								['BREWMASTER_StaggerMax'] = 60,
								['DEATHKNIGHT_InverseCooldown'] = false,
								['DEATHKNIGHT_ColorBySpec'] = true,
							},
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 290,
							['barHeight'] = 10,
							['barXOffset'] = 0,
							['barYOffset'] = 2,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = true,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'RIGHT',
							['barOffsetX'] = 0,
							['barOffsetY'] = 0,
							['iconPosition'] = 'RIGHT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 235,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 295,
							['height'] = 70,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
						['combatIndicator'] = {
							['enableGlow'] = true,
							['enableIcon'] = true,
							['glowSize'] = 14,
							['glowColor'] = {0.9, 0, 0, 1},
							['glowFadeIn'] = 0.5,
							['glowFadeOut'] = 1,
							['iconPosition'] = 'CENTER',
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 20,
							['iconSize'] = 25,
							['iconFadeIn'] = 0.5,
							['iconFadeOut'] = 1,
						},
						['restingIndicator'] = {
							['enable'] = true,
							['position'] = 'TOPLEFT',
							['offsetX'] = -10,
							['offsetY'] = 15,
							['size'] = 20,							
						},
					},
					['target'] = {
						['UFInfo'] = {
							['unit'] = 'target',
							['displayName'] = 'Target',
							['displayOrder'] = 3,
							['cluster'] = {
								['size'] = 1,
								['clusterName'] = '',
							},
						},
						['enable'] = true,
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['point'] = 'TOP',
								['fontHeight'] = 11,
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 11,
								['position'] = 'RIGHT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 14,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 175,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'TOPLEFT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -25,
								['yOffset'] = 10,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Buffs'
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 290,
							['barHeight'] = 10,
							['barXOffset'] = 0,
							['barYOffset'] = 2,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'RIGHT',
							['barOffsetX'] = 0,
							['barOffsetY'] = 0,
							['iconPosition'] = 'LEFT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 235,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 295,
							['height'] = 70,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['targettarget'] = {
						['UFInfo'] = {
							['unit'] = 'targettarget',
							['displayName'] = 'Target of Target',
							['displayOrder'] = 4,
							['cluster'] = {
								['size'] = 1,
								['clusterName'] = '',
							},
						},
						['enable'] = true,
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'BOTTOM',
								['point'] = 'TOP',
								['fontHeight'] = 11,
								['enable'] = false,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 100,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 11,
								['position'] = 'RIGHT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 11,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 125,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 11,
								['position'] = 'LEFT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -25,
								['yOffset'] = 0,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = true,
							['position'] = 'RIGHT',
							['size'] = 30,
							['numPerRow'] = 2,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Debuffs'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'RIGHT',
							['size'] = 30,
							['numPerRow'] = 2,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 130,
							['barHeight'] = 8,
							['barXOffset'] = 0,
							['barYOffset'] = -8,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'RIGHT',
							['barOffsetX'] = 0,
							['barOffsetY'] = 0,
							['iconPosition'] = 'LEFT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 235,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 130,
							['height'] = 30,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['focus'] = {
						['UFInfo'] = {
							['unit'] = 'focus',
							['displayName'] = 'Focus',
							['displayOrder'] = 5,
							['cluster'] = {
								['size'] = 1,
								['clusterName'] = '',
							},
						},
						['enable'] = true,
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['fontHeight'] = 11,
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 11,
								['position'] = 'RIGHT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 14,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 85,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 11,
								['position'] = 'LEFT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -20,
								['yOffset'] = 0,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Buffs'
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 180,
							['barHeight'] = 8,
							['barXOffset'] = 0,
							['barYOffset'] = -8,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'RIGHT',
							['barOffsetX'] = 0,
							['barOffsetY'] = 0,
							['iconPosition'] = 'LEFT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 200,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 180,
							['height'] = 40,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['focustarget'] = {
						['UFInfo'] = {
							['unit'] = 'focustarget',
							['displayName'] = 'Focus Target',
							['displayOrder'] = 6,
							['cluster'] = {
								['size'] = 1,
								['clusterName'] = '',
							},
						},
						['enable'] = true,
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['fontHeight'] = 12,
								['enable'] = false,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['position'] = 'RIGHT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 14,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 64,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'LEFT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['xOffset'] = -25,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Buffs'
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 130,
							['barHeight'] = 8,
							['barXOffset'] = 0,
							['barYOffset'] = -8,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'RIGHT',
							['barOffsetX'] = 0,
							['barOffsetY'] = 0,
							['iconPosition'] = 'LEFT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 200,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 130,
							['height'] = 30,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['pet'] = {
						['UFInfo'] = {
							['unit'] = 'pet',
							['displayName'] = 'Pet',
							['displayOrder'] = 2,
							['cluster'] = {
								['size'] = 1,
								['clusterName'] = '',
							},
						},
						['enable'] = true,
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['fontHeight'] = 12,
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['position'] = 'RIGHT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 18,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 60,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'LEFT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -24,
								['yOffset'] = 0,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Buffs'
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 175,
							['barHeight'] = 8,
							['barXOffset'] = 0,
							['barYOffset'] = 2,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'RIGHT',
							['barOffsetX'] = 0,
							['barOffsetY'] = 0,
							['iconPosition'] = 'RIGHT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 235,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 180,
							['height'] = 50,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['arena'] = {
						['UFInfo'] = {
							['unit'] = 'arena',
							['displayName'] = 'Arena',
							['displayOrder'] = 7,
							['cluster'] = {
								['visibilityCondition'] = '[@arena1, exists] 1; 0',
								['size'] = 5,
								['perRow'] = 1,
								['count'] = 5,
								['gapX'] = 0,
								['gapY'] = 25,
								['inverseStartX'] = false,
								['inverseStartY'] = false,
							},
						},
						['targetWrapWidth'] = 180,
						['enable'] = true,
						['visibilityCondition'] = '[@arena1, exists] 1; 0',
						['sort'] = {
							['startFrom'] = 'TOPLEFT',
							['gapX'] = 0,
							['gapY'] = 25,
							['directionHorizontal'] = '+',
							['directionVertical'] = '-',
						},
						['headers'] = {
							['growthDirection'] = 'DOWN_RIGHT',
							['perRow'] = 1,
							['groupGapX'] = 2,
							['groupGapY'] = 2,
							['gapX'] = 1,
							['gapY'] = 25,
							['anchorPoint'] = "TOPLEFT",
							['relativeAnchorPoint'] = "TOPRIGHT",
							['xOffset'] = 0,
							['yOffset'] = 0,
							['attr_XOffset'] = 0,
							['attr_YOffset'] = 0,
							['attr_Point'] = "TOP",
							['attr_ColumnAnchorPoint'] = "LEFT",
							['attr_MaxColumns'] = 1,
							['attr_UnitsPerColumn'] = 5,
							['attr_ColumnSpacing'] = 4,
							['attr_GroupBy'] = "GROUP",
							['attr_SortDir'] = "ASC",
							['attr_SortMethod'] = "INDEX",
							['attr_ShowPlayer'] = true,
						},
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['point'] = 'TOP',
								['fontHeight'] = 12,
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['position'] = 'RIGHT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 18,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 175,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'LEFT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -25,
								['yOffset'] = 0,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = true,
							['position'] = 'RIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Debuffs'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'RIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 5,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 180,
							['barHeight'] = 8,
							['barXOffset'] = 0,
							['barYOffset'] = 0,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['fastUpdate'] = true,
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'TOPLEFT',
							['barOffsetX'] = 0,
							['barOffsetY'] = 0,
							['iconPosition'] = 'LEFT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 178,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 140,
									['height'] = 10,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 180,
							['height'] = 45,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['fastUpdate'] = true,
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['party'] = {
						['UFInfo'] = {
							['unit'] = 'party',
							['displayName'] = 'Party',
							['displayOrder'] = 8,
							['cluster'] = {
								['visibilityCondition'] = '[group:raid] 0; [group:party] 1; 0',
							},
						},
						['targetWrapWidth'] = 180,
						['enable'] = true,
						['visibilityCondition'] = '[group:raid] 0; [group:party] 1; 0',
						['sort'] = {
							['startFrom'] = 'TOPLEFT',
							['gapX'] = 0,
							['gapY'] = 25,
							['directionHorizontal'] = '+',
							['directionVertical'] = '-',
						},
						['headers'] = {
							['growthDirection'] = 'DOWN_RIGHT',
							['perRow'] = 1,
							['groupGapX'] = 2,
							['groupGapY'] = 2,
							['gapX'] = 1,
							['gapY'] = 25,
							['anchorPoint'] = "TOPLEFT",
							['relativeAnchorPoint'] = "TOPRIGHT",
							['xOffset'] = 0,
							['yOffset'] = 0,
							['attr_XOffset'] = 0,
							['attr_YOffset'] = 0,
							['attr_Point'] = "TOP",
							['attr_ColumnAnchorPoint'] = "LEFT",
							['attr_MaxColumns'] = 1,
							['attr_UnitsPerColumn'] = 5,
							['attr_ColumnSpacing'] = 4,
							['attr_GroupBy'] = "GROUP",
							['attr_SortDir'] = "ASC",
							['attr_SortMethod'] = "INDEX",
							['attr_ShowPlayer'] = true,
						},
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['point'] = 'TOP',
								['fontHeight'] = 12,
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['position'] = 'RIGHT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 18,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 175,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'LEFT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -25,
								['yOffset'] = 0,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = true,
							['position'] = 'RIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Debuffs'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'RIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 5,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 180,
							['barHeight'] = 8,
							['barXOffset'] = 0,
							['barYOffset'] = 0,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['fastUpdate'] = true,
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'TOPLEFT',
							['barOffsetX'] = 0,
							['barOffsetY'] = 0,
							['iconPosition'] = 'LEFT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 178,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 140,
									['height'] = 10,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 180,
							['height'] = 45,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['fastUpdate'] = true,
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['raid'] = {
						['UFInfo'] = {
							['unit'] = 'raid',
							['displayName'] = 'Raid',
							['displayOrder'] = 9,
							['cluster'] = {
								['visibilityCondition'] = '[@raid1,noexists][@raid21,exists] 0;1',
							},
						},
						['targetWrapWidth'] = 460,
						['enable'] = true,
						['visibilityCondition'] = '[@raid1,noexists][@raid21,exists] 0;1',
						['sort'] = {
							['gapX'] = 1,
							['gapY'] = 1,
							['directionHorizontal'] = '-',
							['directionVertical'] = '+',
						},
						['headers'] = {
							['growthDirection'] = 'DOWN_RIGHT',
							['perRow'] = 1,
							['groupGapX'] = 2,
							['groupGapY'] = 2,
							['gapX'] = 1,
							['gapY'] = 1,
							['anchorPoint'] = "TOPLEFT",
							['relativeAnchorPoint'] = "TOPRIGHT",
							['xOffset'] = 0,
							['yOffset'] = 0,
							['attr_XOffset'] = 0,
							['attr_YOffset'] = 0,
							['attr_Point'] = "TOP",
							['attr_ColumnAnchorPoint'] = "LEFT",
							['attr_MaxColumns'] = 1,
							['attr_UnitsPerColumn'] = 5,
							['attr_ColumnSpacing'] = 4,
							['attr_GroupBy'] = "GROUP",
							['attr_SortDir'] = "ASC",
							['attr_SortMethod'] = "INDEX",
							['attr_ShowPlayer'] = true,
						},
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['point'] = 'LEFT',
								['fontHeight'] = 12,
								['enable'] = false,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['position'] = 'RIGHT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 11,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 50,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'LEFT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 25,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = false,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 2,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 2,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['portrait'] = {
							['enable'] = false,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 80,
							['barHeight'] = 9,
							['barXOffset'] = 0,
							['barYOffset'] = 2,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 85,
							['height'] = 45,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['raid40'] = {
						['UFInfo'] = {
							['unit'] = 'raid',
							['displayName'] = 'Raid 40',
							['displayOrder'] = 10,
							['cluster'] = {
								['visibilityCondition'] = '[@raid21, exists] 1; 0',
							},
						},
						['targetWrapWidth'] = 420,
						['enable'] = true,
						['visibilityCondition'] = '[@raid21, exists] 1; 0',
						['sort'] = {
							['startFrom'] = 'TOPRIGHT',
							['gapX'] = 1,
							['gapY'] = 1,
							['directionHorizontal'] = '-',
							['directionVertical'] = '+',
						},
						['headers'] = {
							['growthDirection'] = 'LEFT_DOWN',
							['perRow'] = 1,
							['groupGapX'] = 2,
							['groupGapY'] = 5,
							['gapX'] = 1,
							['gapY'] = 1,
							['anchorPoint'] = "TOPLEFT",
							['relativeAnchorPoint'] = "TOPRIGHT",
							['xOffset'] = 0,
							['yOffset'] = 0,
							['attr_XOffset'] = 0,
							['attr_YOffset'] = 0,
							['attr_Point'] = "TOP",
							['attr_ColumnAnchorPoint'] = "LEFT",
							['attr_MaxColumns'] = 1,
							['attr_UnitsPerColumn'] = 5,
							['attr_ColumnSpacing'] = 4,
							['attr_GroupBy'] = "GROUP",
							['attr_SortDir'] = "ASC",
							['attr_SortMethod'] = "INDEX",
							['attr_ShowPlayer'] = true,
						},
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['point'] = 'LEFT',
								['fontHeight'] = 12,
								['enable'] = false,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['position'] = 'RIGHT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 11,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 50,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'LEFT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 25,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = false,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 2,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 2,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['portrait'] = {
							['enable'] = false,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 75,
							['barHeight'] = 7,
							['barXOffset'] = 0,
							['barYOffset'] = 2,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 80,
							['height'] = 35,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['boss'] = {
						['UFInfo'] = {
							['unit'] = 'boss',
							['displayName'] = 'Boss',
							['displayOrder'] = 11,
							['cluster'] = {
								['size'] = 5,
								['visibilityCondition'] = '1',
								['perRow'] = 1,
								['count'] = 5,
								['gapX'] = 0,
								['gapY'] = 30,
								['inverseStartX'] = false,
								['inverseStartY'] = false,
							},
						},
						['targetWrapWidth'] = 190,
						['enable'] = true,
						['visibilityCondition'] = '1',
						['sort'] = {
							['startFrom'] = 'TOPLEFT',
							['gapX'] = 0,
							['gapY'] = 15,
							['directionHorizontal'] = '+',
							['directionVertical'] = '-',
						},
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['point'] = 'TOP',
								['fontHeight'] = 12,
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['position'] = 'RIGHT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 13,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 175,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'LEFT',
								['enable'] = true,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -25,
								['yOffset'] = 0,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'TOPRIGHT',
							['size'] = 30,
							['numPerRow'] = 9,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 0,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Buffs'
						},
						['portrait'] = {
							['enable'] = true,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 190,
							['barHeight'] = 10,
							['barXOffset'] = 0,
							['barYOffset'] = -10,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = true,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'TOPLEFT',
							['barOffsetX'] = 25,
							['barOffsetY'] = 0,
							['iconPosition'] = 'LEFT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 165,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 190,
							['height'] = 40,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
					['maintank'] = {
						['UFInfo'] = {
							['unit'] = '',
							['displayName'] = 'Maintank',
							['displayOrder'] = 9,
							['cluster'] = {
								['visibilityCondition'] = '[group:raid] 1; [group:party] 0; 0',
							},
						},
						['targetWrapWidth'] = 460,
						['enable'] = true,
						['sort'] = {
							['gapX'] = 1,
							['gapY'] = 1,
							['directionHorizontal'] = '-',
							['directionVertical'] = '+',
						},
						['headers'] = {
							['growthDirection'] = 'DOWN_RIGHT',
							['perRow'] = 1,
							['groupGapX'] = 2,
							['groupGapY'] = 2,
							['gapX'] = 1,
							['gapY'] = 1,
							['anchorPoint'] = "TOPLEFT",
							['relativeAnchorPoint'] = "TOPRIGHT",
							['xOffset'] = 0,
							['yOffset'] = 0,
							['attr_XOffset'] = 0,
							['attr_YOffset'] = 0,
							['attr_Point'] = "TOP",
							['attr_ColumnAnchorPoint'] = "LEFT",
							['attr_MaxColumns'] = 1,
							['attr_UnitsPerColumn'] = 5,
							['attr_ColumnSpacing'] = 4,
							['attr_GroupBy'] = "GROUP",
							['attr_SortDir'] = "ASC",
							['attr_SortMethod'] = "INDEX",
							['attr_ShowPlayer'] = true,
						},
						['fonts'] = {
							['health'] = {
								['textFormat'] = '[health-formatted][newline][health-pct]',
								['position'] = 'LEFT',
								['point'] = 'LEFT',
								['fontHeight'] = 12,
								['enable'] = false,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['yOffset'] = 0,
								['fontType'] = DefaultFont,
								['xOffset'] = 10,
								['fontFlags'] = 'OUTLINE',
								['width'] = 0,
								['horizontalAlign'] = 'LEFT',
								['fontColor'] = {1, 0.84, 0, 1},
							},
							['power'] = {
								['textFormat'] = '[power][newline][power-pct]',
								['fontType'] = DefaultFont,
								['fontHeight'] = 12,
								['position'] = 'RIGHT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = -10,
								['yOffset'] = 0,
								['width'] = 0,
							},
							['name'] = {
								['textFormat'] = '[name]',
								['position'] = 'CENTER',
								['enable'] = true,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 0,
								['fontHeight'] = 11,
								['fontType'] = DefaultFont,
								['yFontShadowOffset'] = -1,
								['fontFlags'] = 'OUTLINE',
								['width'] = 50,
							},
							['level'] = {
								['textFormat'] = '[level]',
								['fontHeight'] = 12,
								['position'] = 'LEFT',
								['enable'] = false,
								['fontFlags'] = 'OUTLINE',
								['fontType'] = DefaultFont,
								['fontShadowColor'] = {0,0,0,1},
								['xFontShadowOffset'] = 1,
								['yFontShadowOffset'] = 1,
								['xOffset'] = 0,
								['yOffset'] = 25,
								['doNotShowOnMaxLevel'] = true,
								['width'] = 0,
								['fontColor'] = {1, 0.84, 0, 1},
							},
						},
						['buffs'] = {
							['enable'] = false,
							['position'] = 'RIGHT',
							['size'] = 25,
							['numPerRow'] = 3,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 3,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Debuffs'
						},
						['debuffs'] = {
							['enable'] = true,
							['position'] = 'RIGHT',
							['size'] = 25,
							['numPerRow'] = 3,
							['maxWraps'] = 1,
							['gapX'] = 1,
							['gapY'] = 1,
							['offsetX'] = 2,
							['offsetY'] = 0,
							['sortBy'] = 'TIME',
							['sortDirection'] = '-',
							['growthDirectionX'] = 'LEFT',
							['growthDirectionY'] = 'UP',
							['attachTo'] = 'Frame'
						},
						['portrait'] = {
							['enable'] = false,
							['alpha'] = 0.25,
							['zoom'] = 1,
							['camDistanceScale'] = 1.4,
							['rotation'] = -0.2,
						},
						['power'] = {
							['barWidth'] = 80,
							['barHeight'] = 9,
							['barXOffset'] = 0,
							['barYOffset'] = 2,
							['barPosition'] = 'BOTTOM',
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
						},
						['castbar'] = {
							['enable'] = false,
							['enableIcon'] = true,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barBorderSize'] = 1,
							['barBorderColor'] = {0, 0, 0, 1},
							['barPosition'] = 'RIGHT',
							['barOffsetX'] = 25,
							['barOffsetY'] = -10,
							['iconPosition'] = 'LEFT',
							['iconSize'] = 23,
							['iconOffsetX'] = 0,
							['iconOffsetY'] = 0,
							['width'] = 180,
							['height'] = 25,
							['sparkWidth'] = 32,
							['sparkHeight'] = 55,
							['attachToUnitframe'] = true,
							['fonts'] = {
								['name'] = {
									['fontHeight'] = 12,
									['position'] = 'LEFT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = 10,
									['yOffset'] = 0,
									['width'] = 140,
									['height'] = 10,
									['fontColor'] = {1, 0.84, 0, 1},
								},
								['time'] = {
									['fontHeight'] = 12,
									['position'] = 'RIGHT',
									['enable'] = true,
									['fontFlags'] = 'OUTLINE',
									['fontType'] = DefaultFont,
									['fontShadowColor'] = {0,0,0,1},
									['xFontShadowOffset'] = 1,
									['yFontShadowOffset'] = 1,
									['xOffset'] = -10,
									['yOffset'] = 0,
									['width'] = 0,
									['fontColor'] = {1, 0.84, 0, 1},
								},
							},
						},
						['health'] = {
							['barBackgroundColor'] = {0.1, 0.1, 0.1, 0.9},
							['barBorderColor'] = {0, 0, 0, 1},
							['width'] = 85,
							['height'] = 45,
							['barInverseFill'] = false,
							['barOrientation'] = 'HORIZONTAL',
							['barSmooth'] = true,
							['absorbTextureSizeMultiplier'] = 7,
							['enableAbsorb'] = true,
							['absorbUseStripes'] = false,
							['absorbBorderColor'] = {0, 0.5, 0.5, 1},
							['absorbTextureColor'] = {0, 0.5, 0.5, 1},
						},
					},
				},
					
			},
		}
	}
end