local E, L = unpack(select(2, ...)) -- Engine, Locale
local AB, CO, TT = E:LoadModules('Actionbars', 'Config', 'Tooltip')

local _


local pairs						= pairs
local format					= string.format
local CreateFrame				= CreateFrame
local SetOverrideBindingClick	= SetOverrideBindingClick
local ClearOverrideBindings		= ClearOverrideBindings
local C_PetBattles_IsInBattle	= C_PetBattles.IsInBattle
local SetBinding				= SetBinding
local GetBindingKey				= GetBindingKey
local hooksecurefunc			= hooksecurefunc
local InCombatLockdown			= InCombatLockdown


local LAB10 = LibStub('LibActionButton-1.0-CUI')
local LibKeyBound = LibStub('LibKeyBound-1.0-CUI')

AB.ActionBars				= {}
AB.ActionButtons 			= {}

AB.ACTIONBUTTON_SIZE 						= 40 -- X and Y size
AB.ACTIONBUTTON_GAP 						= 5 -- X gap
AB.ACTIONBAR_NUM							= 10 -- Everything above bar 7 is already reserved by shapeshift buttons. Use with caution
AB.ACTIONBAR_NUM_BUTTONS					= 12

AB.ACTIONBUTTON_TEXTURE_BACKDROP			= [[Interface\AddOns\CUI\Textures\buttons\ActionButton1Backdrop]]
AB.ACTIONBUTTON_TEXTURE_HIGHLIGHT			= [[Interface\AddOns\CUI\Textures\buttons\ActionButton1Highlight]]
AB.ACTIONBUTTON_TEXTURE_PUSHED				= [[Interface\AddOns\CUI\Textures\buttons\ActionButton1Pushed]]
AB.ACTIONBUTTON_TEXTURE_BORDER				= [[Interface\AddOns\CUI\Textures\buttons\ActionButton1Border]]


E:RegisterEvents(AB, 'PET_BATTLE_OPENING_DONE', 'PET_BATTLE_CLOSE', 'PLAYER_SPECIALIZATION_CHANGED', 'UPDATE_BINDINGS', 'CVAR_UPDATE','TRADE_SKILL_CLOSE', 'ACTIONBAR_UPDATE_USABLE', 'PLAYER_MOUNT_DISPLAY_CHANGED', 'UPDATE_BONUS_ACTIONBAR', 'UPDATE_VEHICLE_ACTIONBAR', 'UPDATE_OVERRIDE_ACTIONBAR', 'ACTIONBAR_PAGE_CHANGED', 'UPDATE_MACROS', 'ADDON_LOADED', 'PLAYER_TARGET_CHANGED', 'PLAYER_ENTERING_WORLD', 'ACTIONBAR_SLOT_CHANGED', 'UPDATE_SHAPESHIFT_FORM', 'ACTIONBAR_UPDATE_COOLDOWN', 'SPELL_UPDATE_COOLDOWN', 'LOSS_OF_CONTROL_ADDED', 'LOSS_OF_CONTROL_UPDATE')

AB.Bindings = {
	[1] = {
		['binding'] = 'ACTIONBUTTON',
		['page'] = 1,
	},
	[2] = {
		['binding'] = 'MULTIACTIONBAR2BUTTON',
		['page'] = 5,
	},
	[3] = {
		['binding'] = 'MULTIACTIONBAR1BUTTON',
		['page'] = 6,
	},
	[4] = {
		['binding'] = 'MULTIACTIONBAR4BUTTON',
		['page'] = 4,
	},
	[5] = {
		['binding'] = 'MULTIACTIONBAR3BUTTON',
		['page'] = 3,
	},
	[6] = {
		['binding'] = 'EXTRABAR6BUTTON',
		['page'] = 2,
	},
	[7] = {
		['binding'] = 'EXTRABAR7BUTTON',
		['page'] = 8,
	},
	[8] = {
		['binding'] = 'EXTRABAR8BUTTON',
		['page'] = 7,
	},
	[9] = {
		['binding'] = 'EXTRABAR9BUTTON',
		['page'] = 9,
	},
	[10] = {
		['binding'] = 'EXTRABAR10BUTTON',
		['page'] = 10,
	},
}

function AB:UpdateArtFill()
	self.db = CO.db.profile.actionbar
	self.dbFill = nil
	
	for k, v in pairs(self.ActionBars) do
		if v.ConfigKey then
			self.dbFill = self.db[v.ConfigKey].artFill
			if self.dbFill.enable then
				if not v.artFill then
					self:CreateArtFill(v)
				end
				
				-----------------------
				v.artFill:ClearAllPoints()
				v.artFill:SetPoint('TOPLEFT', v, 'TOPLEFT', self.dbFill.paddingX * (-1), self.dbFill.paddingY)
				v.artFill:SetPoint('BOTTOMRIGHT', v, 'BOTTOMRIGHT', self.dbFill.paddingX, self.dbFill.paddingY * (-1))
				v.artFill.Border.SetBorderSize(self.dbFill.borderSize)
				v.artFill.Border:SetBackdropBorderColor(self.dbFill.borderColor[1], self.dbFill.borderColor[2], self.dbFill.borderColor[3], self.dbFill.borderColor[4] or 1)
				v.artFill.Background:SetColorTexture(self.dbFill.backgroundColor[1], self.dbFill.backgroundColor[2], self.dbFill.backgroundColor[3], self.dbFill.backgroundColor[4] or 1)
				
				v.artFill:SetFrameStrata('BACKGROUND')
				v.artFill:SetFrameLevel(1)
				
				-----------------------
				v.artFill:Show()
			else
				if v.artFill then v.artFill:Hide() end
			end
		end
	end
end

function AB:CreateArtFill(frame)
	frame.artFill = CreateFrame('Frame', nil, frame)
	
	frame.artFill:SetFrameStrata('BACKGROUND')
	frame.artFill:SetFrameLevel(0)
	
	frame.artFill.Border = E:CreateBorder(frame.artFill, nil, 1)
	frame.artFill.Background = E:CreateBackground(frame.artFill)
end

function AB:UpdateMasque()
	if CO.db.char.actionbar.useMasque and self.Masque then
		if not self.MasqueGroup then
			self.MasqueGroup = self.Masque:Group('CUI', L['Actionbars'])
		end
		self.MasqueGroup:Enable()
	elseif not CO.db.char.actionbar.useMasque and self.MasqueGroup then	
		self.MasqueGroup:Disable()
	end
	
	for k, v in pairs(self.ActionButtons) do
		self:ActionButton_AddMasque(v)
	end
end

function AB:LoadConfig()
	for i=1,12 do self:UpdateActionbar('bar' .. i) end
	self:UpdateActionbar('stancebar')
	self:UpdateActionbar('petbar')
	self:UpdateZoneActionButton()
	self:UpdateExtraActionButton()
	self:UpdateExtraAbilty()
	self:UpdateMicroMenu()
	
	self:UpdateMasque()
	
	self:UpdateActionButtonStyle()
	self:UpdateArtFill()
	self:UpdateAllFlashes() -- Cosmetic button flashes
end

-- /dump CUI:LoadModule('Actionbars'):UpdateActionButtonStyle()
function AB:UpdateActionButtonStyle(Button)
	local Update, db
	local HighlightTexture, NormalTexture, PushedTexture, CheckedTexture, ButtonName, _
	local ClassColor = E:GetUnitClassColor('player')
	
	db = CO.db.profile.actionbar['global']
	
	Update = function(Button)
		
		HighlightTexture = Button:GetHighlightTexture()
		NormalTexture = Button:GetNormalTexture()
		PushedTexture = Button:GetPushedTexture()
		CheckedTexture = Button:GetCheckedTexture()
		ButtonName = E:GetFullFrameName(Button)
		
		Button.Border:Show()
		NormalTexture:Show()
		
		if Button.__MSQ_Normal then
			Button.__MSQ_Normal:Hide()
		end
	
		-- Button border is included within the icons themselves (WHY, BLIZZARD?!)
		Button.icon:SetTexCoord(0.06,0.94,0.06,0.94)
		Button.icon:ClearAllPoints()
		local Offset = 2
		Button.icon:SetPoint("TOPLEFT", Button, "TOPLEFT", Offset, -Offset)
		Button.icon:SetPoint("BOTTOMRIGHT", Button, "BOTTOMRIGHT", -Offset, Offset)
		
		Button.OverrideNormalTexture1 = AB.ACTIONBUTTON_TEXTURE_BACKDROP
		Button.OverrideNormalTexture2 = AB.ACTIONBUTTON_TEXTURE_BACKDROP
		--Button:SetNormalTexture(AB.ACTIONBUTTON_TEXTURE_BACKDROP)
		Button:SetHighlightTexture(AB.ACTIONBUTTON_TEXTURE_HIGHLIGHT)
		Button:SetPushedTexture(AB.ACTIONBUTTON_TEXTURE_PUSHED)
		Button:SetCheckedTexture(AB.ACTIONBUTTON_TEXTURE_PUSHED)
		Button.Border:SetTexture(AB.ACTIONBUTTON_TEXTURE_BORDER)
		Button.Border:SetTexCoord(0,1,0,1)
		HighlightTexture:SetTexCoord(0,1,0,1)
		PushedTexture:SetTexCoord(0,1,0,1)
		CheckedTexture:SetTexCoord(0,1,0,1)
		
		if E:DoesStringPartExist(ButtonName, 'Stance') then
			NormalTexture:SetTexCoord(1,2,1,2) -- Alternate Tex Coord for smaller buttons
		else
			NormalTexture:SetTexCoord(0,1,0,1)
		end
		
		Button.icon:SetDrawLayer('ARTWORK', 0)
		
		PushedTexture:SetDrawLayer('BACKGROUND', 0)
		Button.Border:ClearAllPoints()
		Button.Border:SetPoint('CENTER', Button, 'CENTER', 0, 0)
		Button.Border:SetDrawLayer('ARTWORK', 1)
		
		NormalTexture:ClearAllPoints()
		NormalTexture:SetPoint('CENTER', Button, 'CENTER', 0, 0)
		NormalTexture:SetDrawLayer('BACKGROUND', 0)
		
		PushedTexture:SetDrawLayer('OVERLAY', 0)
		
		local Padding = 6
		
		Button.Border:SetSize(Button:GetWidth()+Padding,Button:GetHeight()+Padding)
		NormalTexture:SetSize(Button:GetWidth(),Button:GetHeight())
		HighlightTexture:SetSize(Button:GetWidth(),Button:GetHeight())
		PushedTexture:SetSize(Button:GetWidth(),Button:GetHeight())
		CheckedTexture:SetSize(Button:GetWidth(),Button:GetHeight())
		
		NormalTexture:SetBlendMode(db['normalTextureBlendMode']) -- DISABLE for fully opaque background. ADD for transparent but a little too light background
		Button.Border:SetBlendMode(db['borderTextureBlendMode'])
		HighlightTexture:SetBlendMode(db['highlightTextureBlendMode'])
		PushedTexture:SetBlendMode(db['pushedTextureBlendMode'])
		CheckedTexture:SetBlendMode(db['pushedTextureBlendMode'])
		
		NormalTexture:SetVertexColor(db['normalTextureColor'].r, db['normalTextureColor'].g, db['normalTextureColor'].b, db['normalTextureColor'].a)
		Button.Border:SetVertexColor(db['borderTextureColor'].r, db['borderTextureColor'].g, db['borderTextureColor'].b, db['borderTextureColor'].a)
		HighlightTexture:SetVertexColor(db['highlightTextureColor'].r, db['highlightTextureColor'].g, db['highlightTextureColor'].b, db['highlightTextureColor'].a)
		PushedTexture:SetVertexColor(db['pushedTextureColor'].r, db['pushedTextureColor'].g, db['pushedTextureColor'].b, db['pushedTextureColor'].a)
		CheckedTexture:SetVertexColor(db['pushedTextureColor'].r, db['pushedTextureColor'].g, db['pushedTextureColor'].b, db['pushedTextureColor'].a)
		
		Button.BorderColor = db['borderTextureColor']
		--Button.Border:SetVertexColor(ClassColor[1], ClassColor[2], ClassColor[3], 0.75)
		--NormalTexture:SetVertexColor(ClassColor[1], ClassColor[2], ClassColor[3], 0.75)
		--HighlightTexture:SetVertexColor(ClassColor[1], ClassColor[2], ClassColor[3], 1)
		
		--NormalTexture:RemoveMaskTexture()
		
	end
	
	if not (CO.db.char.actionbar.useMasque == true and self.Masque) then
		if Button then
				Update(Button)
		else
			for _, Button in pairs(AB.ActionButtons) do
				Update(Button)
			end
		end
	end
end

function AB:ClearButtonHighlight()
	for _, Button in pairs(self.Buttons) do
		
	end
end

function AB:UpdateButtonHighlight(SpellID)
	-- Called through UpdateOnBarHighlightMarksBySpell
	
	for _, Button in pairs(self.Buttons) do
		
	end
	-- Cleared by ClearOnBarHighlightMarks
end

function AB:ReassignBindings()
	if InCombatLockdown() then return end
	
	for _, Bar in pairs(self.ActionBars) do
		self:UpdateConfig(Bar)
	end
	
	self:UpdateZoneActionButtonBinding()
end

function AB:UpdateBarFade(self)
	if not self:GetAttribute("IsShown") then return end
	if self.isActive or self.isHovered then
		E:UIFrameFadeIn(self, self.fadeInSpeed, self:GetAlpha(), self.alphaActive)
	else
		E:UIFrameFadeOut(self, self.fadeOutSpeed, self:GetAlpha(), self.alphaInactive)
	end
end

function AB:UpdateCombatFaderState(self, inCombat)		
	if self.CombatFade == 'fadeOut' or self.CombatFade == 'fadeIn' then
		if inCombat then
			if self.CombatFade == 'fadeOut' then
				self.isActive = false
			elseif self.CombatFade == 'fadeIn' then
				self.isActive = true
			end
		else
			if self.CombatFade == 'fadeOut' then
				self.isActive = true
			elseif self.CombatFade == 'fadeIn' then
				self.isActive = false
			end
		end
	elseif self.CombatFade == 'custom' then
		self.isActive = inCombat
	elseif self.showOnMouseOver == true then
		self.isActive = false
	else
		self.isActive = true
	end
	
	AB:UpdateBarFade(self)
end

-- Mouseover functionality
function AB:UpdateBarFadeHoverState(self, state)
	if not self.showOnMouseOver then 
		self.isHovered = nil
	elseif state == 'enter' then
		self.isHovered = true
	elseif state == 'leave' then
		self.isHovered = nil
	end
	
	AB:UpdateBarFade(self)
end

function AB:Bar_VisibilityOnAttributeChanged(name, value)
	if name ~= 'state-fade' or not self.useCustomFadeCondition then return end
	value = tonumber(value)
	
	if not value or value == 0 then
		AB:UpdateCombatFaderState(self, false)
	else
		AB:UpdateCombatFaderState(self, true)
	end
end

function AB:BarMOver_OnEnter()
	if not self:GetAttribute('IsShown') then return end
	
	AB:UpdateBarFadeHoverState(self, 'enter')
end
function AB:BarMOver_OnLeave()
	if not self:GetAttribute('IsShown') then return end
	
	AB:UpdateBarFadeHoverState(self, 'leave')
end

function AB:BarMOverButton_OnEnter()
	if not self.Parent:GetAttribute('IsShown') then return end
	
	AB:UpdateBarFadeHoverState(self.Parent, 'enter')
end
function AB:BarMOverButton_OnLeave()
	if not self.Parent:GetAttribute('IsShown') then return end
	
	AB:UpdateBarFadeHoverState(self.Parent, 'leave')
end

function AB:InitBarFader()
	for k, Bar in pairs(self.ActionBars) do
		if not self.CombatFader then
			self.CombatFader = CreateFrame('Frame')
		end
		
		self.CombatFader:RegisterEvent('PLAYER_REGEN_ENABLED')
		self.CombatFader:RegisterEvent('PLAYER_REGEN_DISABLED')
		
		self.CombatFader:SetScript('OnEvent', function(self, event, ...)
			for k, v in pairs(AB.ActionBars) do
				if v:GetAttribute('IsShown') and not v.useCustomFadeCondition then
					AB:UpdateCombatFaderState(v, event == 'PLAYER_REGEN_DISABLED')
				end
			end
			
		end)
	end
end

local FlashDefaults = {
	enable = false,
	blendMode = "ADD",
	drawLayer = "OVERLAY",
	drawLayerSubLevel = 5,
	fadeInTime = 0.025,
	fadeOutTime = 0.25,
	rgba = {1,1,1, 1},
	forceTexture = nil, -- Path or nil
}
local function merge(target, source, default)
	for k,v in pairs(default) do
		if type(v) ~= "table" then
			if source and source[k] ~= nil then
				target[k] = source[k]
			else
				target[k] = v
			end
		else
			if type(target[k]) ~= "table" then target[k] = {} else wipe(target[k]) end
			merge(target[k], type(source) == "table" and source[k], v)
		end
	end
	return target
end
local function DoFlash(self)
	-- This portion is called when the action actually would be used
	if self.flashConfig.enable and (self:GetAttribute("type") == "action" or self.IsStanceButton or self.IsPetButton) then

		-- Replacing UIFrameFlash functions, as those SOMEHOW cause taint on Blizzard frames ???
		--if self.flashOverlay.alphaAnimation then
			-- Cancel current flash to preserve speed of button inputs
				-- if self.flashOverlay.alphaAnimation:IsPlaying() then
					-- self.flashOverlay.alphaAnimation:Restart()
				-- end
				-- self.flashOverlay.alphaAnimation:Play()
				-- self.flashOverlay:Show()
		--end
		
		if E:UIFrameIsFlashing(self.flashOverlay) then
			E:UIFrameFlashStop(self.flashOverlay)
		end
		
		local Total = self.flashConfig.fadeInTime + self.flashConfig.fadeOutTime
		E:UIFrameFlash(self.flashOverlay, self.flashConfig.fadeInTime, self.flashConfig.fadeOutTime, Total, false, 0, 0)
	end
end

local function Flash_OnFinished(self)
	self.Owner.flashOverlay:Hide()
end
function AB:LoadFlashAnimation()
	if not self.flashOverlay.alphaAnimation then
		local Group = self.flashOverlay:CreateAnimationGroup()
		Group.Owner = self
		Group:SetScript('OnFinished', Flash_OnFinished)
		
		local FadeIn = Group:CreateAnimation("Alpha")
		local FadeOut = Group:CreateAnimation("Alpha")
		
		FadeIn:SetFromAlpha(0)
		FadeIn:SetToAlpha(1)
		FadeIn:SetOrder(1)
		
		FadeOut:SetFromAlpha(1)
		FadeOut:SetToAlpha(0)
		FadeOut:SetOrder(2)
		
		self.flashOverlay.alphaAnimation = Group
		self.flashOverlay.FadeIn = FadeIn
		self.flashOverlay.FadeOut = FadeOut
	end
	
	self.flashOverlay.FadeIn:SetDuration(self.flashConfig.fadeInTime)
	self.flashOverlay.FadeOut:SetDuration(self.flashConfig.fadeOutTime)
end

function AB:SetupFlash()
	if not self.flashConfig then
		self.flashConfig = {}
		merge(self.flashConfig, FlashDefaults, FlashDefaults)
	end
	
	local config = self.flashConfig
	if not config.enable then return end
	
	-- if not self.flashOverlayParent then
		-- self.flashOverlayParent = CreateFrame('Frame', nil, self)
		-- self.flashOverlayParent:SetAllPoints(self)
	-- end
	
	local flash = self.flashOverlay or self:CreateTexture("FLASHOVERLAY", "OVERLAY")
	if not config.forceTexture then
		flash:SetColorTexture(config.rgba[1], config.rgba[2], config.rgba[3], config.rgba[4])
	else
		flash:SetTexture(config.forceTexture)
		flash:SetVertexColor(unpack(config.forceTextureRGBA))
	end
	
	flash:SetBlendMode(config.blendMode)
	flash:SetDrawLayer(config.drawLayer, config.drawLayerSubLevel)
	flash:SetAlpha(0)
	
	local ParentFrame = config.showAboveCooldown and self.Overlay or self
	flash:ClearAllPoints()
	flash:SetAllPoints(ParentFrame)
	flash:SetParent(ParentFrame)
	
	if not self.flashOverlay then
		self.flashOverlay = flash
		self:HookScript("OnClick", DoFlash)
		--AB.LoadFlashAnimation(self)
	end
end

local function UpdateButtonFlash(self, Config)
	if not Config then return end
	if not self.flashConfig then
		self.flashConfig = {}
	end
	
	self.flashConfig.enable 			= Config.enable
	self.flashConfig.showAboveCooldown 	= Config.showAboveCooldown
	self.flashConfig.blendMode 			= Config.blendMode
	self.flashConfig.drawLayer 			= Config.drawLayer
	self.flashConfig.drawLayerSubLevel 	= Config.drawLayerSubLevel
	self.flashConfig.fadeInTime 		= Config.fadeInTime
	self.flashConfig.fadeOutTime 		= Config.fadeOutTime
	self.flashConfig.rgba 				= E:ParseDBColor(Config.rgba, "player")
	self.flashConfig.forceTexture 		= Config.forceTexture
	
	if self.flashOverlay then
		--AB.LoadFlashAnimation(self)
	end
	
	AB.SetupFlash(self)
end

function AB:UpdateAllFlashes()
	local Config
	for _, Bar in pairs(self.ActionBars) do
		if Bar.buttons then
			Config = CO.db.profile.actionbar[Bar.ConfigKey].flash
			
			for _, Button in pairs(Bar.buttons) do
				UpdateButtonFlash(Button, Config)
			end
		end
	end
end

--[[
	Expected settings:
	
	- Enable / Disable
	- A scale multiplier (This also has to move the hotkey and macro texts)
	- Setting the actual gap
	- Initial anchor move handler
	- Buttons per row
	- Total Buttons
	
	Optional: 
	
	- Movable individual buttons
]]--
function AB:UpdateActionbar(bar)
	-- Fix for config, as we only pass the bar number from there
	if tonumber(bar) then
		bar = 'bar' .. bar
	end
	local Bar = self.ActionBars[bar]
	
	if not Bar then return end
	
	local Config = CO.db.profile.actionbar[Bar.ConfigKey]
	if not Config then return end
	
	if not Config.enable then
		Bar.ForceMoverEnabled = false
	else
		Bar.ForceMoverEnabled = nil
	end
	-- Hide when disabled. Show when enabled and visibility condition is met
	if not Config.enable then
		Bar:Hide();
		return
	else
		if SecureCmdOptionParse(Config.visibilityCondition) == '1' then
			Bar:Show()
		end
	end
	
	Bar:SetIgnoreParentAlpha(true)
	
	Bar.cooldownFormat = Config.cooldownFormat
	
	UnregisterStateDriver(Bar, 'visible')
	UnregisterStateDriver(Bar, 'page')
	
	RegisterStateDriver(Bar, 'visible', Config.visibilityCondition)
	
	Bar.CombatFade = Config.fadeInCombat
	if Bar.CanBeFaded then
		Bar.showOnMouseOver = Config.showOnMouseOver
		Bar.alphaActive = Config.alphaActive
		Bar.alphaInactive = Config.alphaInactive
		Bar.fadeInSpeed = Config.fadeInSpeed
		Bar.fadeOutSpeed = Config.fadeOutSpeed
		Bar.useCustomFadeCondition = true
		Bar.fadeCondition = Bar.CombatFade == 'custom' and Config.fadeCondition or '[combat] 1; 0'
		
		if Bar:GetAttribute('IsShown') then
			if Bar.useCustomFadeCondition then
				AB.Bar_VisibilityOnAttributeChanged(Bar, 'state-fade', SecureCmdOptionParse(Bar.fadeCondition))
			else
				AB:UpdateCombatFaderState(Bar, InCombatLockdown())
			end
		end
		
		-- Custom Fader Conditional
		if Bar.useCustomFadeCondition then
			RegisterStateDriver(Bar, 'fade', Bar.fadeCondition)
			if not Bar.AttrIsHooked then
				Bar:HookScript('OnAttributeChanged', self.Bar_VisibilityOnAttributeChanged)
				Bar.AttrIsHooked = true
			end
		end
	end
	
	if Bar.BarIndex then
		if Bar.BarIndex > 1 then
			RegisterStateDriver(Bar, 'page', AB.Bindings[Bar.BarIndex].page)
		else
			RegisterStateDriver(Bar, 'page', '[possessbar] 12; [overridebar] 14; [shapeshift] 13; [form, noform] 0; [bar:3] 3; [bar:4] 4; [bar:5] 5; [bar:6] 6; [bonusbar:1, nostealth] 7; [bonusbar:1,stealth] 7; [bonusbar:3] 9; [bonusbar:4] 10; 1')
		end
		
		Bar:SetAttribute('page', AB.Bindings[Bar.BarIndex].page)
		
		AB:UpdateConfig(Bar)
	end
	
	
	for k ,child in ipairs(Bar.buttons) do			
		child.IgnoreSort = nil
		
		if Bar.ConfigKey ~= 'stancebar' and Bar.ConfigKey ~= 'petbar' then
			if k > Config.buttonNum then
				child:Hide(); child:SetAttribute('enable', false)
				child.IgnoreSort = true
			else
				child:Show()
				child:SetAttribute('enable', true)
			end
		end
		
		if bar == 'stancebar' and child.Border then
			child.Border:SetSize(AB.ACTIONBUTTON_SIZE + 5, AB.ACTIONBUTTON_SIZE + 5)
		end
		
		if bar == 'petbar' and child.Flash then
			
			-- That frame is such a Murloc
			child.Flash:SetScale(0.55)
		end
		
		child:SetSize(AB.ACTIONBUTTON_SIZE, AB.ACTIONBUTTON_SIZE)
		child:SetScale(Config.buttonSizeMultiplier)
	end
	
	if bar == 'stancebar' then
		AB:UpdateActiveStanceButtons()
		
		return
	end
	local NewWidth, NewHeight = E:SortFrames(Bar.buttons, Bar, nil, nil, Config.buttonSizeMultiplier, Config.buttonsPerRow, nil, nil, Config.buttonGap, Config.buttonGap, true, false)
	Bar:SetSize(NewWidth, NewHeight)
	
	E:LoadMoverPositions(Bar)
	E:UpdateMoverDimensions(Bar)
end

function AB:CreateActionBars()

	local barName, actionBar, buttonName, actionButton
	

	for b=1, self.ACTIONBAR_NUM do
		
		barName = 'CUI_ActionBar' .. b
		
		actionBar = CreateFrame('Frame', barName, E.Parent, 'SecureHandlerStateTemplate')
		actionBar:SetFrameRef('MainMenuBarArtFrame', MainMenuBarArtFrame)
		actionBar:SetSize((self.ACTIONBUTTON_SIZE + self.ACTIONBUTTON_GAP)*(12) - 24, self.ACTIONBUTTON_SIZE)
		
		actionBar.ConfigKey = 'bar' .. b
		actionBar.BarIndex = b
		
	-- Init Button Container
		actionBar.buttons = {}
	-- Set bindings base to bar
		actionBar.bindButtons = self.Bindings[b].binding
	-- Register actionbar
		self.ActionBars["bar" .. b] = actionBar

	-- Set visibility driver (This is user controlled through the config dialog)
	-- StateDriver is registered in the config
		E:SetVisibilityHandler(actionBar)
		actionBar:SetAttribute('_onstate-page', [[
			self:SetAttribute('state', newstate)
			control:ChildUpdate('state', newstate)
		]])
		
		E:CreateMover(actionBar, L['actionbarFrame'] .. ' ' .. b, nil, nil, nil, nil, "actionbars")
		
		actionBar:SetScript('OnEnter', AB.BarMOver_OnEnter)
		actionBar:SetScript('OnLeave', AB.BarMOver_OnLeave)

		actionBar.CanBeFaded = true
	
		-- Create AB.ACTIONBAR_NUM_BUTTONS (12) buttons per bar
		for i=1, self.ACTIONBAR_NUM_BUTTONS do
			
		-- Button Name
			buttonName = format('CUI_ActionBar%sButton%s', b, i)
			
		-- Create new Button object through LibAB
			actionButton = LAB10:CreateButton(i, buttonName, actionBar)
		-- Register created button
			self.ActionButtons[buttonName] = actionButton
			
		-- Cache parent because we will have to reference to it pretty often
			actionButton.Parent = actionBar
			actionButton.cooldown.Parent = actionBar
		
			for k = 1,14 do
				actionButton:SetState(k, 'action', (k - 1) * 12 + i)
			end
			actionButton:SetState(0, 'action', i)
			actionButton:SetAttribute('buttonlock', GetCVarBool('lockActionBars'))
			
			actionButton.index = i
			
			--actionButton.Overlay = CreateFrame('Frame', nil, actionButton.cooldown)
			--actionButton.Overlay:SetAllPoints(actionButton.cooldown)
			actionButton.Overlay = CreateFrame('Frame', nil, actionButton)
			actionButton.Overlay:SetAllPoints(actionButton)
			self:CreateCooldownText(actionButton)
			hooksecurefunc(actionButton.cooldown, 'SetCooldown', self.OnSetCooldown)
			actionButton.cooldown:SetHideCountdownNumbers(true)
			
			actionButton.cooldown:Hide()
			
			-- N.E.V.E.R use ActionButton_GetOverlayGlow, as it WILL Taint something on Blizzards end and then throw errors
			--actionButton.Glow = ActionButton_GetOverlayGlow(actionButton)
			
			-- Register Fonts with corresponding database path to automate updates
			actionButton.HotKey.__MSQ_Hooked = true -- Trick Masque into thinking it already has control over the text position
			E:RegisterPathFont(actionButton.HotKey, 'db.profile.actionbar.bar' .. b .. '.hotkey')
			E:RegisterPathFont(actionButton.cooldown.cooldownText, 'db.profile.actionbar.bar' .. b .. '.cooldown')
			E:RegisterPathFont(actionButton.Count, 'db.profile.actionbar.bar' .. b .. '.count')
			E:RegisterPathFont(actionButton.Name, 'db.profile.actionbar.bar' .. b .. '.macro')
			
		-- Add LibAB methods
			actionButton.GetHotkey 		= self.GetHotkey
			actionButton.SetKey 		= self.ActionButton_SetKey
			actionButton.ClearBindings 	= self.ActionButton_ClearBindings
			actionButton.GetBindings 	= self.ActionButton_GetBindings
			
		-- Workaround for tooltips of macros, pets and toys
			actionButton:HookScript('OnEnter', self.ActionButton_OnEnter)
			actionButton:HookScript('OnLeave', self.ActionButton_OnLeave)
			
			actionButton:HookScript('OnEnter', AB.BarMOverButton_OnEnter)
			actionButton:HookScript('OnLeave', AB.BarMOverButton_OnLeave)
			
			
			actionBar.buttons[i] = actionButton
		end
		
		self:UpdateConfig(actionBar)
	end
end

function AB:ActionButton_AddMasque(actionButton)
	if CO.db.char.actionbar.useMasque == true and self.Masque and not actionButton.HasMasque then
		
		local buttonData = {
			Icon = actionButton.icon,
			Cooldown = actionButton.cooldown,
			Normal = actionButton:GetNormalTexture(),
			Pushed  = actionButton:GetPushedTexture(),
			Border = actionButton.Border
		}
		
		actionButton.HasMasque = true
	
		self.MasqueGroup:AddButton(actionButton, buttonData)
	elseif not CO.db.char.actionbar.useMasque == true and actionButton.HasMasque then
		actionButton.HasMasque = nil
		
		if self.MasqueGroup then
			--self.MasqueGroup:RemoveButton(actionButton)
		end
	end
end

function AB:ActionButton_GetBindings()
	local BindButton = self:GetParent().bindButtons
	local ButtonIndex = self.index
	
	local keys = ''
	
	for i = 1, select('#', GetBindingKey(BindButton .. ButtonIndex)) do
		
		local hotKey = select(i, GetBindingKey(BindButton .. ButtonIndex))
		if keys ~= '' then
			keys = keys .. ', '
		end
		keys = keys .. GetBindingText(hotKey, 'KEY_')
	end
	
	return keys
end

function AB:ActionButton_SetKey(key)
	local BindButton = self:GetParent().bindButtons
	local ButtonIndex = self.index
	
	SetBinding(key, BindButton .. ButtonIndex)
	
	AB:ReassignBindings()
end

function AB:ActionButton_ClearBindings()
	local Key = GetBindingKey(format('%s%s', self:GetParent().bindButtons, self.index))
	if not Key then return end
	
	SetBinding(Key, nil)
	
	AB:ReassignBindings()
end

function AB:ActionButton_OnEnter()
	E.TooltipOwnedByActionButton = true
	TT:UpdateStyle(true)
end

function AB:ActionButton_OnLeave()
	E.TooltipOwnedByActionButton = nil
end

function AB:ClearBindings()
	if InCombatLockdown() then return end

	for _, Bar in pairs(self.ActionBars) do
		if Bar then
			ClearOverrideBindings(Bar)
		end
	end
end

function AB:UpdateConfig(Bar)
	local Button, Binding, ButtonBinding, BarNum, BarProfile
	
	if self.keyRebind then return end
	
	if Bar.buttons and not Bar.BlizzBar then
		
		_, BarNum = E:ExtractDigits(Bar:GetName())
		BarProfile = CO.db.profile.actionbar[format('bar%s', BarNum)]
		
		if not Bar.buttonConfig then Bar.buttonConfig = {} end
		
		Bar.buttonConfig.outOfRangeColoring = 'button'
		Bar.buttonConfig.tooltip = BarProfile.showTooltip
		Bar.buttonConfig.showGrid = BarProfile.showGrid
		Bar.buttonConfig.colors = { range = { 0.8, 0.1, 0.1 }, mana = { 0.5, 0.5, 1.0 } }
		Bar.buttonConfig.hideElements = { macro = false, hotkey = false, equipped = false }
		-- Bar.buttonConfig.keyBoundTarget = false
		Bar.buttonConfig.clickOnDown = BarProfile.clickOnDown
		Bar.buttonConfig.flyoutDirection = BarProfile.flyoutDirection
		
		ClearOverrideBindings(Bar)
		
		for i=1, NUM_ACTIONBAR_BUTTONS do
			Button = Bar.buttons[i]
			ButtonBinding = format('%s%s', Bar.bindButtons, i)
			Binding = GetBindingKey(ButtonBinding)
			
			if Binding then
				SetOverrideBindingClick(Bar, false, Binding, Button:GetName())
			end
			
			Bar.buttonConfig.keyBoundTarget = Binding
			
			Button:UpdateConfig(Bar.buttonConfig)
		end
	end
end

function AB:OnSetCooldown(start, duration)
	if start > 0 and duration > 0.5 then
		-- self:Show()
		self.nextUpdate = 0 -- Force Immediate Update
		
		self.start = start
		self.duration = duration
		self:SetScript('OnUpdate', AB.ActionButton_UpdateCooldownText)
	else
		-- self:Hide()
		self:SetScript('OnUpdate', nil)
	end
end

function AB:OnClearCooldown()
	self.cooldownText:SetText('')
end

function AB:ActionButton_UpdateCooldownText(elapsed)
	
	self.update = self.update + elapsed
	if self.update >= (self.nextUpdate or 0) then
		self.durationRemaining = self.duration + (self.start - GetTime())
		
		if self.durationRemaining <= 0 then
			self.cooldownText:SetText('')
			
			return
		end
		
		self.nextUpdate = 0.5
		
		if self.duration > 1.5 then
			if not self.Parent.cooldownFormat then
				self.timeRemaining = 0
				if self.durationRemaining > 10 then
					if self.durationRemaining > 60 then
						if self.durationRemaining > 300 then
							if self.durationRemaining > 3600 then
								if self.durationRemaining > (3600 * 24) then
									self.timeRemaining = format('%dd', (self.durationRemaining / (3600 * 24)))
								else
									self.timeRemaining = format('%dh', (self.durationRemaining / 3600))
								end
							else
								self.timeRemaining = format('%dm', (self.durationRemaining / 60))
							end
						else
							self.timeRemaining = format('%d:%02d', self.durationRemaining / 60, self.durationRemaining % 60)
						end
					else
						self.timeRemaining = E:FormatTime(self.durationRemaining)
					end
				else
					self.timeRemaining, self.nextUpdate = E:FormatTime(self.durationRemaining, 1)
					self.nextUpdate = E:GetNumberFormatNextUpdate(self.nextUpdate)
				end
				self.cooldownText:SetText(self.timeRemaining)
			else
				self.nextUpdate = E:WriteNumberFormat(self.cooldownText, self.Parent.cooldownFormat, self.durationRemaining)
			end
		else
			self.cooldownText:SetText('')
		end
		
		self.update = 0
	end
end

function AB:CreateCooldownText(self)
	local Cooldown = self.Cooldown or self.cooldown
	
	if not Cooldown.cooldownText then
		Cooldown.cooldownText = Cooldown:CreateFontString(nil, 'OVERLAY')
		E:InitializeFontFrame(Cooldown.cooldownText, 'OVERLAY', 'FRIZQT__.TTF', 12, {0.933, 0.886, 0.125}, 1, {0,0}, '', 100, 20, Cooldown, 'CENTER', {1,1})
		--Cooldown.cooldownText:SetParent(Cooldown)
		E:SetFontInfo(Cooldown.cooldownText, nil, 'THICKOUTLINE', nil, nil)
		E:UpdateFont(Cooldown.cooldownText)
		
		-- SetAllPoints sometimes causes the text to be cutoff
		Cooldown.cooldownText:SetPoint('CENTER', self.Overlay, 'CENTER')
		Cooldown.cooldownText:SetJustifyH('CENTER')
		Cooldown.cooldownText:SetDrawLayer('ARTWORK', 5) -- Above Border
		
		Cooldown.update = 0
	end
end

local function InitActionBarChange()
	--[[
		Here, it is ESSENTIAL to do the THREE following things within the attribute function:
			- self:SetAttribute('state', PAGENUM)
			- self:ChildUpdate('state', PAGENUM)
			- self:GetFrameRef('MainMenuBarArtFrame'):SetAttribute('actionpage', PAGENUM)
			
		If we are missing just ONE of those things, the bar will NOT respond to the change (correctly)
		TL;DR: This was a pain in the beep to find out
	]]
	
	AB.ActionBars['bar1']:SetAttribute('_onstate-overrideBar', [[
		local changeTo = GetActionBarPage()
		
		-- print('newstate:', newstate)
		if newstate == 12 or newstate == 13 then
				changeTo = 12
		elseif newstate == 14 then
			if HasOverrideActionBar() then
				changeTo = GetOverrideBarIndex()
			end
		else
			if (newstate == 'noform' or newstate == 'shapeshift') then
				if HasTempShapeshiftActionBar() then
					changeTo = GetTempShapeshiftBarIndex()
				elseif HasOverrideActionBar() then
					changeTo = GetOverrideBarIndex()
				else
					changeTo = GetActionBarPage()
				end
            else
				if HasBonusActionBar() then
					changeTo = GetBonusBarIndex()
				else
					changeTo = GetActionBarPage()
				end
			end
		end
		
		-- print('changeTo:', changeTo)
		self:SetAttribute('state', changeTo)
		self:ChildUpdate('state', changeTo)
		self:GetFrameRef('MainMenuBarArtFrame'):SetAttribute('actionpage', changeTo)
	]]);
	
	
	-- /dump HasTempShapeshiftActionBar(), GetTempShapeshiftBarIndex(), HasOverrideActionBar(), GetOverrideBarIndex()
	-- local condition = '[form:0] noform; [form:0] noform; [form:1] 1; [form:2] 2; [form:4] 4; noform'
	-- SecureCmdOptionParse(condition)
	-- /run print(SecureCmdOptionParse('[form:0] noform; [form:0] noform; [form:1] 1; [form:2] 2; [form:4] 4; noform'))
	-- /dump SecureCmdOptionParse('[overridebar] 14;[vehicleui] 12;[possessbar] 12;[form:1] 1; [form:2] 2; [form:4] 4; noform;')
	-- /dump SecureCmdOptionParse('[shapeshift] shape; noform;')
	-- /dump SecureCmdOptionParse('[bonusbar:1] 1;[bonusbar:2] 2; [bonusbar:3] 3; [bonusbar:4] 4; [bonusbar:5] 5; [bonusbar:6] 6; noform;')
	-- /dump CUI:LoadModule('Actionbars').ActionBars['CUI_ActionBar1']:GetAttribute('state')
	-- /dump MainMenuBarArtFrame:GetAttribute('actionpage')
end

function AB:GetShortHotkey(key)
	key = gsub(key, 'SHIFT%-', 'S-');
	key = gsub(key, 'ALT%-', 'A-');
	key = gsub(key, 'CTRL%-', 'C-');
	key = gsub(key, 'BUTTON', 'MB');
	key = gsub(key, 'MOUSEWHEELUP', 'MWU');
	key = gsub(key, 'MOUSEWHEELDOWN', 'MWD');
	key = gsub(key, 'NUMPAD', 'NP');
	key = gsub(key, 'PAGEUP', 'PgUp');
	key = gsub(key, 'PAGEDOWN', 'PgDown');
	key = gsub(key, 'SPACE', 'Space');
	key = gsub(key, 'INSERT', 'Ins');
	key = gsub(key, 'HOME', 'Home');
	key = gsub(key, 'DELETE', 'Del');
	key = gsub(key, 'NMULTIPLY', '*');
	key = gsub(key, 'NMINUS', 'N-');
	key = gsub(key, 'NPLUS', 'N+');
	key = gsub(key, 'NEQUALS', 'N=');

	return key or LibKeyBound:ToShortKey(key)
end

function AB:GetHotkey()
	local name, key
	name = "CLICK "..self:GetName() or self.config.keyBoundTarget ..":LeftButton"
	key = GetBindingKey(self.config.keyBoundTarget or name)
	
	if not key and self.config.keyBoundTarget and name then
		key = GetBindingKey(name, 1)
		
		if not key then
			key = self.config.keyBoundTarget
		end
	end
	
	if key then
		return AB:GetShortHotkey(key)
	end
end

function AB:SetKeybinder(state)
	if state == true then
		AB:RegisterEvent('REGEN_DISABLED')
		AB:SetScript('OnEvent', function(self) AB:SetKeybinder(false) end)
		
		self.keyRebind = true
		CO:ShowNotification('KEYREBIND_ACTIVE')
		
		for _, button in pairs(AB.ActionButtons) do
			button:HookScript('OnEnter', function(self) AB:BindUpdate(self); end);
		end
	else
		AB:UnregisterEvent('REGEN_DISABLED')
		self.keyRebind = false
	end
end

function AB:UpdateExtraBar()
	if not InCombatLockdown() then
		UnregisterStateDriver(AB.ActionBars['bar1'], 'overrideBar')
		RegisterStateDriver(AB.ActionBars['bar1'], 'overrideBar', '[overridebar] 14;[vehicleui] 12;[possessbar] 12;[form:1] 1; [form:2] 2; [form:3] 3; [form:4] 4;[shapeshift] shapeshift; noform;')
	end
end

function AB:SetupActionbars()
	self:CreateActionBars()
	self:CreatePetActionBar()
	InitActionBarChange()
	self:UpdateExtraBar()
	self:InitStanceBar()
	self:InitExtraActionButton()
	self:InitZoneActionButton()
	self:InitExtraAbility()
	self:InitMicroMenu()
	
	self:InitBarFader()
	
	self:LoadConfig()
end

function AB:Init()
	CO = E:LoadModules('Config')
	
	if not CO.db.char.actionbar.enable then return end
	
	self.Masque = E.Masque
	self:UpdateMasque()
	
	self:SetupActionbars()
	
	self.PageWarningShown = nil
	
	self:SetScript('OnEvent', function(self, event, ...)
		if event == 'UPDATE_BONUS_ACTIONBAR' 
			or event == 'UPDATE_VEHICLE_ACTIONBAR' 
			or event == 'UPDATE_OVERRIDE_ACTIONBAR'
			or event == 'ACTIONBAR_PAGE_CHANGED' then
				if event == 'ACTIONBAR_PAGE_CHANGED' and not AB.PageWarningShown then
					E:print('Warning: You changed the actionbar page. If this was not intentional, it is advised to remove the binding for this action.')
					AB.PageWarningShown = true
				end
			AB:UpdateExtraBar()
		end
		-- Fix for some random bug that appeared first in 8.0.1.
		-- /dump CUI[1]:LoadModule('Actionbars'):InitStanceBar()
		if event == 'PLAYER_SPECIALIZATION_CHANGED' then
			AB:InitStanceBar()
			AB:UpdateExtraBar()
		end
		if event == 'UPDATE_BINDINGS' or event == "PLAYER_ENTERING_WORLD" or event == "PET_BATTLE_CLOSE" then
			AB:ReassignBindings()
		end
		if event == 'CVAR_UPDATE' and select(1, ...) == 'LOCK_ACTIONBAR_TEXT' then
			local val = GetCVarBool('lockActionBars')
			for _, button in pairs(AB.ActionButtons) do
				button:SetAttribute('buttonlock', val)
			end
		end
		if event == 'PET_BATTLE_OPENING_DONE' then
			AB:ClearBindings()
		end
	end)
	
	if C_PetBattles_IsInBattle() then
		AB:ClearBindings()
	end
	
	-- Prevent micromenu from being repositioned in petbattles
	PetBattleFrame.BottomFrame.MicroButtonFrame:SetScript('OnShow', nil)
	
	--hooksecurefunc('MicroButtonAlert_OnShow', function(...) AB:MainMenuMicroButton_RepositionAlerts() end)
	local Blizzard = E:LoadModule('Blizzard')
	Blizzard:RemoveActionbars()
end

E:AddModule('Actionbars', AB)