local E, L = unpack(select(2, ...)) -- Engine, Locale
local A, CO = E:LoadModules("Armory", "Config")
A.Autoload = true

A.Modules = {}
A.SlotInfo = {}
A.SlotInfoFrames = {}
A.SlotInfoFontData = {
	["Head"] 		= {1, "RIGHT"},
	["Neck"] 		= {2, "RIGHT"},
	["Shoulder"] 	= {3, "RIGHT"},
	["Back"] 		= {15, "RIGHT"},
	["Chest"] 		= {5, "RIGHT"},
	["Shirt"] 		= {4, "RIGHT"},
	["Tabard"] 		= {19, "RIGHT"},
	["Wrist"] 		= {9, "RIGHT"},
	["Hands"] 		= {10, "LEFT"},
	["Waist"] 		= {6, "LEFT"},
	["Legs"] 		= {7, "LEFT"},
	["Feet"] 		= {8, "LEFT"},
	["Finger0"] 	= {11, "LEFT"},
	["Finger1"] 	= {12, "LEFT"},
	["Trinket0"] 	= {13, "LEFT"},
	["Trinket1"] 	= {14, "LEFT"},
	["MainHand"] 	= {16, "LEFT"},
	["SecondaryHand"] = {17, "RIGHT"},
}
-- Possible Inventory Types
-- This basically maps the Inventory Types to Slots
-- Type => Slot
A.ItemInvTypeToSlot = {
	["INVTYPE_HEAD"] = {1},
	["INVTYPE_NECK"] = {2},
	["INVTYPE_SHOULDER"] = {3},
	["INVTYPE_BODY"] = {4},
	["INVTYPE_CHEST"] = {5},
	["INVTYPE_ROBE"] = {5},
	["INVTYPE_WAIST"] = {6},
	["INVTYPE_LEGS"] = {7},
	["INVTYPE_FEET"] = {8},
	["INVTYPE_WRIST"] = {9},
	["INVTYPE_HAND"] = {10},
	["INVTYPE_FINGER"] = {11, 12},
	["INVTYPE_TRINKET"] = {13, 14},
	["INVTYPE_CLOAK"] = {15},
	["INVTYPE_WEAPON"] = {16, 17},
	["INVTYPE_SHIELD"] = {17},
	["INVTYPE_2HWEAPON"] = {16},
	["INVTYPE_WEAPONMAINHAND"] = {16},
	["INVTYPE_WEAPONOFFHAND"] = {17},
	["INVTYPE_HOLDABLE"] = {17},
	["INVTYPE_RANGED"] = {18},
	["INVTYPE_THROWN"] = {18},
	["INVTYPE_RANGEDRIGHT"] = {18},
	["INVTYPE_RELIC"] = {18},
	["INVTYPE_TABARD"] = {19},
}

function A:UpdateOverallItemlevelText(Type, Unit)
	if not CO.db.global.customArmory.showItemlevel then
		return
	end
	
	Type = Type or "Character"
	local TargetFrame = _G[string.format("%sModelFrame", Type)]
	
	-- If Slot does not exist
	if TargetFrame and not TargetFrame.OverallItemlevel then
		TargetFrame.OverallItemlevel = CreateFrame("Frame", nil, TargetFrame)
		TargetFrame.OverallItemlevel:SetSize(200, 15)
		TargetFrame.OverallItemlevel:SetPoint('TOP', TargetFrame, 'TOP', 0, -2)
		
		local Font = TargetFrame.OverallItemlevel:CreateFontString(nil, "ARTWORK")
		
		E:InitializeFontFrame(Font, "ARTWORK", nil, 10, {0.09,0.52,0.82}, 1, nil, "", 0, 0, TargetFrame.OverallItemlevel, 'CENTER', {0,0}, "OUTLINE")
		Font:SetJustifyH('CENTER')
		Font:SetSize(200, 15)
		
		TargetFrame.OverallItemlevel.Text = Font
	end
	
	if UnitExists(Unit) then
		TargetFrame.OverallItemlevel.Text:SetTextColor(unpack(E:GetUnitClassColor(Unit)))
		TargetFrame.OverallItemlevel.Text:SetText(select(2, A.Modules["OverallItemlevel"]:GetInfo(Unit)))
	else
		TargetFrame.OverallItemlevel.Text:SetText('')
	end
end

-- Needed: Ilvl, Enchant
function A:CreateSlotInfo(Type)
	Type = Type or "Character"
	
	local Slot, SlotInfoName
	
	for name, data in pairs(self.SlotInfoFontData) do
		Slot = _G[string.format("%s%sSlot", Type, name)]
		SlotInfoName = Type .. name
		
		-- If Slot exists
		if Slot and not self.SlotInfo[SlotInfoName] then
			self.SlotInfo[SlotInfoName] = CreateFrame("Frame", nil, Slot)
			self.SlotInfo[SlotInfoName]:SetSize(64, 40)
			self.SlotInfo[SlotInfoName]:SetPoint(E:InversePosition(data[2]), Slot, data[2], (data[2] == "RIGHT") and 3 or -3, 0)
			
			self.SlotInfo[SlotInfoName].Name = SlotInfoName
			self.SlotInfo[SlotInfoName].Slot = data[1]
			self.SlotInfo[SlotInfoName].Align = data[2]
			
			self:CreateSlotInfoFonts(self.SlotInfo[SlotInfoName])
			self:CreateBackground(self.SlotInfo[SlotInfoName])
		end
	end
end

local function CreateGemSlots(InfoFrame, CurrentName)
	
	A.SlotInfoFrames[CurrentName].Gems = {}
	A.SlotInfoFrames[CurrentName].Gems.Parent = InfoFrame
	
	for Index=1, MAX_NUM_SOCKETS do
		local GemSlot = CreateFrame("Button", ("CUI_SlotGemInfo%s_%s"):format(CurrentName, Index), InfoFrame)
		A.SlotInfoFrames[CurrentName].Gems[Index] = GemSlot
		
		GemSlot:EnableMouse(true)
		GemSlot:SetScript("OnEnter", A.Modules["Gems"].Slot_OnEnter)
		GemSlot:SetScript("OnLeave", A.Modules["Gems"].Slot_OnLeave)
		GemSlot:SetScript("OnClick", A.Modules["Gems"].Slot_OnClick)
		
		GemSlot.Parent = InfoFrame
		GemSlot:SetSize(12, 12)
		GemSlot.Background = E:CreateBackground(GemSlot)
		GemSlot.Border = E:CreateBorder(GemSlot)
		
		local Position = "BOTTOM" .. E:InversePosition(InfoFrame.Align)
		local Prefix = (InfoFrame.Align == "RIGHT") and 1 or -1
		GemSlot:SetPoint(Position, InfoFrame, Position, ((12 * Index) + (2 * (Index - 1))) * Prefix, 2)
		
		GemSlot.Tex = GemSlot:CreateTexture(nil, "OVERLAY")
		GemSlot.Tex:SetAllPoints(GemSlot)
		
		GemSlot:Hide()
	end
end

local ArmoryFonts = {"ilvl", "enchant"}
function A:CreateSlotInfoFonts(InfoFrame)
	
	local CurrentName = InfoFrame.Name
	local FontSize = 10
	local CurrentOffset = 0
	local Font
	
	if not A.SlotInfoFrames[CurrentName] then A.SlotInfoFrames[CurrentName] = {} end
	
	for index, name in pairs(ArmoryFonts) do
		if (name == "enchant" and CO.db.global.customArmory.showEnchants) or (name == "ilvl" and CO.db.global.customArmory.showItemlevel) then
			if name == "ilvl" then
				CurrentOffset = FontSize
			else
				CurrentOffset = 0
			end
			
			Font = InfoFrame:CreateFontString(nil, "ARTWORK")
			A.SlotInfoFrames[CurrentName][name] = Font
			
			E:InitializeFontFrame(Font, "ARTWORK", nil, FontSize, {0.8,0.8,0.8}, 1, {(InfoFrame.Align == "RIGHT") and 12 or -12, CurrentOffset}, "", 0, 0, InfoFrame, E:InversePosition(InfoFrame.Align), {0,0}, "OUTLINE")
			Font:SetJustifyH(E:InversePosition(InfoFrame.Align))
			Font:SetSize(80, 15)
			
			Font.Parent = InfoFrame
		end
	end
	
	if CO.db.global.customArmory.showGems then		
		-- Gems
		CreateGemSlots(InfoFrame, CurrentName)
	end
end

function A:CreateBackground(F)
	E:CreateTextureObject(F, "Background", "BACKGROUND")
	F.Background:SetTexture([[Interface\AddOns\CUI\Textures\layout\InspectInfoBackground]])
	
	F.Background:SetAlpha(0.45)
	--F.Background:SetVertexColor(0.09, 0.51, 0.81)
	F.Background:SetVertexColor(0.450, 0.580, 0.807)
	
	if F.Align == "LEFT" then
		F.Background:SetTexCoord(1, 0, 0, 1)
	else
		F.Background:SetTexCoord(0, 1, 0, 1)
	end
end

function A:UpdateBackground(Background, State)
	if State then
		Background:Show()
	else
		Background:Hide()
	end
end

function A:UpdateData(Unit, Type)
	
	if not CO.db.global.customArmory.enabled then return end
	
	Type = Type or "Character"
	
	-- Contains: SlotID, Alignment
	local Slot, ItemLink, ItemLevel, ItemRarity, Enchant, RarityColor, RarityColorHex,
	AllFontsEmpty, LastFont
	
	for slotInfoName, fonts in pairs(A.SlotInfoFrames) do
		
		if (slotInfoName):find(Type) then
			
			AllFontsEmpty = true
			
			for name, frame in pairs(fonts) do
				
				Slot = frame.Parent.Slot
				
				ItemLink = GetInventoryItemLink(Unit or "player", Slot)
				
				if name ~= "Gems" then
					
					-- Post-set shadow color, since it's just a pain
					frame:SetShadowColor(0, 0, 0)
					
					if ItemLink ~= nil and ItemLink ~= "" then
						AllFontsEmpty = false
					end
					if frame.ItemLink ~= ItemLink then
						frame.ItemLink = ItemLink
						
						if name == "ilvl" then
							local Text, Color = A.Modules["Itemlevel"]:GetInfo(ItemLink)
							
							if Text ~= "" then AllFontsEmpty = false end
							frame:SetText(Text)
						elseif name == "enchant" then
							local Text, IsEnchanted = A.Modules["Enchant"]:GetInfo(ItemLink)
							
							if Text ~= "" then AllFontsEmpty = false end
							frame:SetText(Text)
						end
						
					end
					
					-- To access the parent
					LastFont = frame
				else
					if ItemLink then
						local Gems = A.Modules["Gems"]:GetInfo(ItemLink)
						local GemSlot, GemSlotFrame
						
						-- GEMS
						for i = 1, MAX_NUM_SOCKETS do
							GemData = Gems[i]
							GemSlotFrame = frame[i]
							
							if GemData.isEmpty ~= nil then
								-- GEM SLOT EXISTS
								GemSlotFrame.Tex:SetTexture(GemData.Texture)
								GemSlotFrame.GemLink 	= GemData.GemLink
								GemSlotFrame.GemQuality = GemData.GemQuality
								
								if GemSlotFrame.GemQuality then
									GemSlotFrame.Border:SetBackdropBorderColor(GetItemQualityColor(GemSlotFrame.GemQuality))
									GemSlotFrame.Border:Show()
								else
									GemSlotFrame.Border:Hide()
								end
								
								GemSlotFrame:Show()
							else
								GemSlotFrame.GemLink = nil
								
								GemSlotFrame:Hide()
							end
						end
					else
						for i = 1, MAX_NUM_SOCKETS do
							frame[i]:Hide()
						end
					end
				end
			end
			
		end
		
		if LastFont then
			if AllFontsEmpty then
				A:UpdateBackground(LastFont.Parent.Background, false)
			else
				A:UpdateBackground(LastFont.Parent.Background, true)
			end
		end
	end
end

function A:UpdateInfo()
	if not CO.db.global.customArmory.enabled then return end
	
	if _G["CharacterModelFrame"]:IsVisible() then
		-- Those values are used to RESIZE the panel
		_G["CharacterFrame"]:SetWidth(610)
		_G["CharacterFrameInset"]:SetPoint("BOTTOMRIGHT", CharacterFrame, "BOTTOMLEFT", 400, 5)
		
		local ModelFrame = _G["CharacterModelFrame"]
		
		if not A.AlteredModelFrame then
			
			local function OnMouseWheel(self, delta, maxZoom, minZoom)				
				maxZoom = maxZoom or self.maxZoom;
				minZoom = minZoom or self.minZoom;
				local zoomLevel = self.zoomLevel or minZoom;
				zoomLevel = zoomLevel + delta * MODELFRAME_ZOOM_STEP;
				zoomLevel = min(zoomLevel, maxZoom);
				zoomLevel = max(zoomLevel, minZoom);
				
				if not self.interpolateZoom then
					self:SetPortraitZoom(zoomLevel);
					self.zoomLevel = zoomLevel
				else
					self.zoomElapsed = 0
					self.startZoomLevel = self.zoomLevel
					self.targetZoomLevel = zoomLevel
					self.zoomFinished = nil
				end
				
			end
			
			local InterpolationTargetTime = 0.125 -- 0.1 Second animation
			local function OnUpdate(self, elapsed)				
				-- Interpolate Zoom
				if self.interpolateZoom then
					if self.zoomFinished then return end
					
					self.zoomElapsed = (self.zoomElapsed or 0) + elapsed
					local NewZoom = self.startZoomLevel + ((self.targetZoomLevel - self.startZoomLevel) / (InterpolationTargetTime)) * (self.zoomElapsed)
					
					if NewZoom > 0 and NewZoom < 1 and (abs(NewZoom - self.targetZoomLevel) > 0) then
						self.zoomLevel = NewZoom
						self:SetPortraitZoom(NewZoom)
					end
					
					if self.zoomElapsed >= InterpolationTargetTime then
						self.zoomElapsed = 0
						self.zoomFinished = true
					end
					
				end
			end
			
			ModelFrame.interpolateZoom = true
			
			ModelFrame.OnMouseWheel = OnMouseWheel
			ModelFrame:SetScript('OnMouseWheel', OnMouseWheel)
			ModelFrame.OnUpdate = OnUpdate
			ModelFrame:HookScript('OnUpdate', OnUpdate)
			
			A.AlteredModelFrame = true
		end
		
		ModelFrame:SetPortraitZoom(0)
		ModelFrame.zoomElapsed = 0
		ModelFrame.zoomLevel = 0
		ModelFrame.startZoomLevel = ModelFrame.zoomLevel
		ModelFrame.targetZoomLevel = ModelFrame.zoomLevel
		
		A:UpdateData()
	else
		-- Those values are used to RESET the panel
		_G["CharacterFrame"]:SetWidth(338) -- Default: 338
		_G["CharacterFrameInset"]:SetPoint("BOTTOMRIGHT", CharacterFrame, "BOTTOMLEFT", 333, 4) -- Default: 333, 4
	end
end

-- self = PaperDollFrame
function A:CreateInfo()	
	
	if CO.db.global.customArmory.enabled then
		
		_G["CharacterMainHandSlot"]:ClearAllPoints()
		_G["CharacterMainHandSlot"]:SetPoint('BOTTOM', _G["CharacterFrameInset"], 'BOTTOM', -(_G["CharacterMainHandSlot"]:GetWidth() / 2), 10)

		_G["CharacterModelFrame"]:ClearAllPoints()
		_G["CharacterModelFrame"]:SetPoint('TOPLEFT', _G["CharacterFrameInset"], "TOPLEFT", 32, -5)
		_G["CharacterModelFrame"]:SetPoint('BOTTOMRIGHT', _G["CharacterFrameInset"], "BOTTOMRIGHT", -32, 28)
		
		A:CreateSlotInfo()
		A:UpdateInfo()
	end
	
	A:OverridePanelBackground(_G["CharacterModelFrame"], CO.db.global.customArmory.overrideBackground, CO.db.global.customArmory.useCustomBackground, CO.db.global.customArmory.customBackgroundPath, CO.db.global.customArmory.classBackground)
end

function A:OverridePanelBackground(Frame, State, CustomTexture, CustomTexturePath, TextureUseClass)
	-- This overrides the default armory background based on player character class
	if State then
		Frame.BackgroundTopLeft:Hide()
		Frame.BackgroundTopRight:Hide()
		Frame.BackgroundBotLeft:Hide()
		Frame.BackgroundBotRight:Hide()
		Frame.BackgroundOverlay:Hide()
		
		if not Frame.ModelBackground then
			Frame.ModelBackground = Frame:CreateTexture("ModelBackground", "BACKGROUND")
			Frame.ModelBackground:SetAllPoints(Frame)
		end
		
		if CustomTexture then
			Frame.ModelBackground:SetTexture(CustomTexturePath)
		else
			if TextureUseClass == "PLAYER_CLASS" then
				TextureUseClass = E.PlayerClass
			end
			
			Frame.ModelBackground:SetAtlas("dressingroom-background-" .. E:stringToLower(TextureUseClass))
		end
	end
end

function A:LoadConfig()
	
	self:SetScript("OnEvent", nil)
	
	if CO.db.global.customArmory.enabled then
		self:SetScript("OnEvent", self.UpdateInfo)
	end
	
	for k,v in pairs(self.SlotInfo) do
		if CO.db.global.customArmory.enabled then
			v:Show()
		else
			v:Hide()
		end
	end
	
	A:OverridePanelBackground(_G["CharacterModelFrame"], CO.db.global.customArmory.overrideBackground, CO.db.global.customArmory.useCustomBackground, CO.db.global.customArmory.customBackgroundPath, CO.db.global.customArmory.classBackground)
end

function A:__Construct()
	--self.EquipmentSlots = { PaperDollItemsFrame:GetChildren() }
	local EventFrames = {
		["CharacterFrame"] = {"OnShow", A.UpdateInfo, "OnHide", A.UpdateInfo},
		["CharacterStatsPane"] = {"OnShow", A.CreateInfo, "OnHide", A.UpdateInfo},
		["PaperDollTitlesPane"] = {"OnShow", A.UpdateInfo, "OnHide", A.UpdateInfo},
		["PaperDollEquipmentManagerPane"] = {"OnShow", A.UpdateInfo, "OnHide", A.UpdateInfo},	
	}
	
	for k, v in pairs(EventFrames) do
		_G[k]:HookScript(v[1], v[2])
		_G[k]:HookScript(v[3], v[4])
	end
	
	hooksecurefunc("CharacterFrameTab_OnClick", A.UpdateInfo)
	
	self:RegisterEvent("SOCKET_INFO_SUCCESS")
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	self:RegisterEvent("ITEM_UPGRADE_MASTER_UPDATE")
	self:RegisterEvent("TRANSMOGRIFY_SUCCESS")
	self:RegisterEvent("UPDATE_INVENTORY_DURABILITY")
	self:RegisterEvent("LOADING_SCREEN_DISABLED")
	
	self:LoadConfig()
end

function A:Init()
	self:__Construct()
end

E:AddModule("Armory", A)