local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, DT = E:LoadModules("Config", "Datatexts")
DT.Autoload = true
--------------------------------------------------------

local _

--[[
	[Name] = {
		[Events] = {},
		[OnUpdate] = Func,
		[OnEvent] = Func,
		[AdditionalData] = {},
		[DBUpdate] = Func,
		[PostUpdate] = Func,
		[OnDisable] = Func,
	}
	
	-- Functions also are injected into the font object
	-- For DB Settings, maybe avoid single use entries and instead make one for each Datatext
]]
local ColorizedPing = {[0] = "FF2ded4d", [75] = "FFe3b034", [150] = "FFea2020"}
local ColorizedFPS = {[0] = "FFea2020", [25] = "FFe3b034", [45] = "FF2ded4d"}
local Data = {
	['FPS'] = {
		['OnUpdate'] = function(self, elapsed)
			local Value = GetFramerate()
			local ColorizedText
			for k,v in pairs(self.AdditionalData['ColorizedFPS']) do
				if k <= Value then
					ColorizedText = format("|c%s%s|r", v, E:Round(Value, 1))
				end
			end
			
			self:SetText(ColorizedText)
		end,
		['OnEvent'] = nil,
		['Events'] = nil,
		['AdditionalData'] = {["ColorizedFPS"] = ColorizedFPS},
		['DBUpdate'] = function(self)
			self.UpdateInterval = CO.db.profile.system.layoutUpdateFrequency
		end,
		['PostUpdate'] = function(self)  end,
		['OnDisable'] = nil,
	},
}

function DT:DisableDatatext(Object)
	Object:SetScript('OnUpdate', nil)
	Object:SetScript('OnEvent', nil)
	
	Object:OnDisable()
end

function DT:EnableDatatext(Object)
	local Meta = Data[Object.Type]
	
	if Meta.OnUpdate then
		Object:SetScript('OnUpdate', Object.OnUpdate)
	end
	if Meta.OnEvent then
		Object:SetScript('OnEvent', Object.OnEvent)
	end
	
	Object:DBUpdate()
end

-- Enables AddOn Devs to manually add new types
function DT:AddNewDatatextType(Type, Tbl)	
	if not Data[Type] then
		Data[Type] = {}
	end
	
	for k,v in pairs(Tbl) do
		Data[Type][k] = v
	end
end

function DT:RegisterDatatext(Type)
	local Object = E:NewFontObject(nil, nil, self, 12)
	local Meta = Data[Type]
	
	Object.Type = Type
	
	-- Add Everything
	for k,v in pairs(Meta) do
		Object[k] = v
	end
	
	for _, event in pairs(Meta.Events) do
		Object:RegisterEvent(event)
	end
end

--------------------------------------------------------
function DT:Init()
	
end
--E:AddModule("Datatexts", DT)