local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, FI = E:LoadModules("Config", "Filters")

---------------------------------------------------
local pairs 					= pairs
local select 					= select
local GetSpellInfo 				= GetSpellInfo
---------------------------------------------------

FI.Data = {}
E.AuraFilters = FI.Data

FI.Types = {['Whitelist'] = 1, ['Blacklist'] = 2}
FI.TypesByID = {
	[1] = 'Whitelist',
	[2] = 'Blacklist',
}

-- Reference table to search for filter names
local RecursiveIdentifier = {}
function FI:UpdateRecursiveTable()
	wipe(RecursiveIdentifier)
	
	for index, data in pairs(self.Data) do
		RecursiveIdentifier[data.name] = index
	end
end

local function GetFilterTableByName(name)
	local Index = RecursiveIdentifier[name]
	if Index then
		return FI.Data[Index]
	end
end

---------------------------------------------
	
	function FI:IsProtected(filterIdentifier)
		local tbl = self:GetFilterData(filterIdentifier)
		if not tbl then return nil end
		
		return (tbl.isProtected) and true or false
	end
	
	-- FilterIdentifier HAS to be either the filter index or the name
	function FI:Add(filterIdentifier, id, data)
		if not filterIdentifier or not id or not data then return end
		
		local tbl = self:GetFilterData(filterIdentifier)
		if not tbl then return end
		
		if not tbl[id] then
			tbl[id] = data
		end
	end
	
	-- Method to safely remove filters
	-- @return: Success (bool)
	function FI:Remove(ID)
		if not ID then return false end
		
		local tbl = self.Data[ID]
		if not tbl then return false end
		
		if self:IsProtected(ID) then return false end
		
		-- Search for dependencies first to cleanly remove the filter entry
		for _, data in pairs(CO.db.profile.unitframe.units) do
			if data.buffs.filterType == tbl.name then
				data.buffs.filterType = -1
			end
			if data.debuffs.filterType == tbl.name then
				data.debuffs.filterType = -1
			end
		end
		
		for _, data in pairs(CO.db.profile.auras.units) do
			if data.aurabars then
				if data.aurabars.filterType == tbl.name then
					data.aurabars.filterType = -1
				end
			end
		end
		
		
		self.Data[ID] = nil
		self:UpdateRecursiveTable()
		
		return true
	end
	
	-- Use this for select values
	function FI:GetAvailableFilters()
		local Filters = {}
		
		for k,v in pairs(self.Data) do
			Filters[k] = v.name
		end
		
		Filters[-1] = "None"
		
		return Filters
	end
	
	function FI:GetFilterData(filterIdentifier)
		return GetFilterTableByName(filterIdentifier) or self.FilterDB[filterIdentifier]
	end
	
	function FI:GetFilterType(filterIdentifier)
		local Data = self:GetFilterData(filterIdentifier)
		return Data and self.TypesByID[Data.type] or nil
	end
	
	function FI:CheckAuraID(Filter, id)
		if not Filter then return false end
		if Filter.entries[id] and Filter.entries[id].enabled then
			if Filter.type == 1 then
				return false
			else
				return true
			end
		end
		
		-- Default Filter behaviour
		if Filter.type == 1 then
			return true
		else
			return false
		end
	end
	
	-- To get the correct spell id for the entry
	local function GetCorrectSpellID(id)
		-- First check if the Spell exists in the Spellbook
		
		-- Resolve to Name
		local Name = GetSpellInfo(id)
		-- If we get anything non-nil out of this, there is an correct spell id
		local SpellbookID = Name and select(7, GetSpellInfo(Name)) or nil
		
		return SpellbookID or id
	end
	
	-- Checks if the aura ID should be filtered (not shown)
	function FI:IsFiltered(filterIdentifier, id)
		if not filterIdentifier or (filterIdentifier and filterIdentifier < 1) then return false end
		
		local Filter = self:GetFilterData(filterIdentifier)
		return self:CheckAuraID(Filter, GetCorrectSpellID(id)), self:GetFilterType(filterIdentifier)
	end

---------------------------------------------

function FI:Init()
	self.FilterDB = CO.db.global.filters.auras
	
	self.Data = self.FilterDB
	self:UpdateRecursiveTable()
end

E:AddModule('Filters', FI)