local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, Module = E:LoadModules("Config", "ItemDB")
Module.Autoload = true

-----------------------------------------------------------------------------
local _
local pairs					= pairs
local select				= select
local match					= string.match
local GetItemInfo			= GetItemInfo
local GetItemCount			= GetItemCount
local GetContainerItemLink	= GetContainerItemLink
local GetContainerNumSlots	= GetContainerNumSlots
local GetItemInfoInstant	= GetItemInfoInstant
local GetVoidItemInfo		= GetVoidItemInfo
local GetInventoryItemLink	= GetInventoryItemLink
local C_CurrencyInfo_GetCurrencyInfo		= C_CurrencyInfo.GetCurrencyInfo or GetCurrencyInfo
local C_CurrencyInfo_GetCurrencyListInfo	= C_CurrencyInfo.GetCurrencyListInfo or GetCurrencyListInfo
local C_CurrencyInfo_ExpandCurrencyList		= C_CurrencyInfo.ExpandCurrencyList or ExpandCurrencyList
local C_CurrencyInfo_GetCurrencyListLink	= C_CurrencyInfo.GetCurrencyListLink or GetCurrencyListLink



-- Yo Blizz, why are those locals?
local VOID_STORAGE_PAGES = 2
local VOID_STORAGE_MAX = 80

-----------------------------------------------------------------------------

--['itemDB'] = {		
--	['data'] = {
		--[[
		[CharKey] = {
			[ItemID] = {NumInBags, NumInBank, NumInVoid},
		},
		]]--
--	},
--}


-- Also use realm in key, so we never hit any duplicate chars
local RealmKey = GetRealmName()
local CharKey = UnitName("player") .. " - " .. RealmKey
-- Used for return data limiting
local CurrentFaction = UnitFactionGroup("player")
-- The second value in a players GUID is the Realm ID, which is shared between realms in the same realmpool.
-- We can use that to limit return data to the same pool!
local CurrentRealmID = select(2, strsplit("-", UnitGUID("player") or ""))

function Module:AddItemToCurrentChar(ItemID, IsVoid, IsEquipped)
	if not ItemID then return end -- You're not supposed to be here
	
	if not self.db.data[CharKey].Items[ItemID] then
		self.db.data[CharKey].Items[ItemID] = {}
	end
	
	if not IsVoid and not IsEquipped then
		local TotalNum = GetItemCount(ItemID, true)
		if TotalNum <= 0 then 
			self.db.data[CharKey].Items[ItemID] = nil
			
			return
		end 
		
		local InBags = GetItemCount(ItemID)
		local InBank = TotalNum - InBags
		
		-- Subtract equipped num from the ItemCount, as it appears to already count equipped gear.
		-- This would falsify our real count
		self.db.data[CharKey].Items[ItemID][1] = InBags - (self.db.data[CharKey].Items[ItemID][4] or 0)
		self.db.data[CharKey].Items[ItemID][2] = InBank
	elseif IsVoid then
		self.db.data[CharKey].Items[ItemID][3] = (self.db.data[CharKey].Items[ItemID][3] or 0) + 1
	elseif IsEquipped then
		self.db.data[CharKey].Items[ItemID][4] = (self.db.data[CharKey].Items[ItemID][4] or 0) + 1
	end
end

local NameToDBIndex = {
	['bags'] 	  = 1,
	['bank'] 	  = 2,
	['void'] 	  = 3,
	['equipment'] = 4,
}
-- Without this, we're just adding to the db, but never remove anything
function Module:ResetForCurrentChar(StorageType)
	local DBIndex = NameToDBIndex[StorageType]
	local Target = self.db.data[CharKey].Items
	
	for k, itemData in pairs(Target) do
		itemData[DBIndex] = nil
	end
end
function Module:IsBankInitialized()
	return self.db.data[CharKey].isBankInitialized
end
function Module:SetBankInitialized()
	self.db.data[CharKey].isBankInitialized = true
end

function Module:ScanBag(BagID)
	local ItemID
	for slot=0,GetContainerNumSlots(BagID) do
		local Link = GetContainerItemLink(BagID, slot)
		if Link then
			ItemID = GetItemInfoInstant(Link)
			
			self:AddItemToCurrentChar(ItemID)
		end
	end
end
function Module:ScanVoidStorage()
	self:ResetForCurrentChar('void')
	
	local ItemID
	for tab=1, VOID_STORAGE_PAGES do
		for slot=1, VOID_STORAGE_MAX do
			ItemID = GetVoidItemInfo(tab, slot)
			
			self:AddItemToCurrentChar(ItemID, true)
		end
	end
end
function Module:ScanEquipment()
	self:ResetForCurrentChar('equipment')
	
	local Link, ItemID
	for i=1, 19 do
		Link = GetInventoryItemLink('player', i)
		if Link then
			ItemID = GetItemInfoInstant(Link)
			
			self:AddItemToCurrentChar(ItemID, false, true)
		end
	end
end
--[[
	CURRENCY:
	We can use C_CurrencyInfo_GetCurrencyListInfo(index) to retrieve data about all currencies.
	When a header is collapsed, we can call ExpandCurrencyList(id, expanded[0 or 1]) to actually get the currency data
	Update event: CURRENCY_DISPLAY_UPDATE
	GetCurrencyListSize()
--]]

function Module:UpdateCurrency()
	
	wipe(self.db.data[CharKey].Currency)
	
	local name, isHeader, isExpanded, count, currencyID, link, data
	local index = 1
	
	while true do
		if not C_CurrencyInfo.GetCurrencyListInfo then
			name, isHeader, isExpanded, _, _, count = C_CurrencyInfo_GetCurrencyListInfo(index)
		else
			data = C_CurrencyInfo_GetCurrencyListInfo(index)
			if not data then break end
			
			name, isHeader, isExpanded, count = data.name, data.isHeader, data.isHeaderExpanded, data.quantity
		end
		if not name then break end
		
		if isHeader and not isExpanded then
			C_CurrencyInfo_ExpandCurrencyList(index, 1)
		elseif not isHeader then
			link = C_CurrencyInfo_GetCurrencyListLink(index)
			
			if link then
				currencyID = tonumber(match(C_CurrencyInfo_GetCurrencyListLink(index),"currency:(%d+)"))
				if currencyID then
					self.db.data[CharKey].Currency[currencyID] = count
				end
			end
		end
		
		index = index + 1
	end
	
	-- Money
	self.db.data[CharKey].Currency['Money'] = GetMoney()
end

function Module:OnEvent(event, arg1, ...)
	
	if event == 'PLAYER_ENTERING_WORLD' or event == 'BAG_UPDATE' or event == 'PLAYER_EQUIPMENT_CHANGED' or event == 'CURRENCY_DISPLAY_UPDATE' or event == 'LoadBank' then
		
		self:UpdateCurrency()
		if event == 'CURRENCY_DISPLAY_UPDATE' then return end
		
		self:ScanEquipment()
		-- We scanned equipment, let's leave
		if event == 'PLAYER_EQUIPMENT_CHANGED' then	return end
		
		if event ~= 'LoadBank' then
			self:ResetForCurrentChar('bags')
			for bag = 0,4 do
				self:ScanBag(bag)
			end
			
			if event == 'PLAYER_ENTERING_WORLD' then
				self:ScanVoidStorage()
			end
		end
		
		if self.IsBankOpen then			
			self:ResetForCurrentChar('bank')
			
			for bag = NUM_BAG_SLOTS+1, NUM_BAG_SLOTS+NUM_BANKBAGSLOTS do
				self:ScanBag(bag)
			end
			self:ScanBag(BANK_CONTAINER)
			self:ScanBag(REAGENTBANK_CONTAINER)
		end
	elseif event == 'BANKFRAME_OPENED' or event == 'BANKFRAME_CLOSED' then
		self.IsBankOpen = event == 'BANKFRAME_OPENED'
		
		if event == 'BANKFRAME_OPENED' then
			self:OnEvent('LoadBank')
		end
	elseif event == 'VOID_TRANSFER_DONE' then
		self:ScanVoidStorage()
	end
end

-- We can use this to actually retrieve the DB information in a convenient way
function Module:GetItemInfo(ItemID)
	local Data = self.db.data
	if not Data then return nil end
	
	local Info = {}
		Info.Chars = {}
	local Total = 0
	
	-- Scans all entries
	for CharName, v in pairs(Data) do
		-- Limit returned data to current realmpool AND faction
		if v.RealmID == CurrentRealmID and v.Faction == CurrentFaction then
			if v.Items[ItemID] and ((v.Items[ItemID][1] or 0) > 0 or (v.Items[ItemID][2] or 0) > 0 or (v.Items[ItemID][3] or 0) > 0 or (v.Items[ItemID][4] or 0) > 0) then
				Info.Chars[CharName] = {
					['bags'] = v.Items[ItemID][1] or 0,
					['bank'] = v.Items[ItemID][2] or 0,
					['void'] = v.Items[ItemID][3] or 0,
					['equipped'] = v.Items[ItemID][4] or 0,
					['Class'] = v.Class,
				}
				
				Total = Total + ((v.Items[ItemID][1] or 0) + (v.Items[ItemID][2] or 0) + (v.Items[ItemID][3] or 0) + (v.Items[ItemID][4] or 0))
			end
		end
	end
	
	Info.Total = Total
	
	return Info
end

function Module:GetCurrencyInfo(ItemID)
	local Data = self.db.data
	if not Data then return nil end
	
	local Info = {}
		Info.Chars = {}
	local Total = 0
	
	-- Scans all entries
	for CharName, v in pairs(Data) do
		-- Limit returned data to current realmpool AND faction
		if v.RealmID == CurrentRealmID and v.Faction == CurrentFaction then
			if v.Currency[ItemID] then
				Info.Chars[CharName] = {
					['current'] = v.Currency[ItemID] or 0,
					['Class'] = v.Class,
				}
				
				Total = Total + (v.Currency[ItemID] or 0)
			end
		end
	end
	
	Info.Total = Total
	
	return Info
end

-- Use this for testing
-- /dump CUI[1]:LoadModule("ItemDB"):GetItemInfo(172191)
function Module:LoadConfig()
	self.db = CO.db.global.itemDB
	
	if self.db.enable then
		self:SetScript("OnEvent", self.OnEvent)
	else
		self:SetScript("OnEvent", nil)
	end
end

function Module:Construct()	
	-- Prepare Char Table
	if not self.db.data[CharKey] then	
		self.db.data[CharKey] = {}
		self.db.data[CharKey].Items = {}
		self.db.data[CharKey].Currency = {}
		-- Just another place where we store character data.. Gotta find another permanent place for that
		self.db.data[CharKey].Class = select(2, UnitClass("player"))
		self.db.data[CharKey].RealmID = CurrentRealmID
		self.db.data[CharKey].Faction = CurrentFaction
	end
	
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("BANKFRAME_CLOSED")
	self:RegisterEvent("VOID_TRANSFER_DONE")
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function Module:Init()
	self.db = CO.db.global.itemDB
	
	self:Construct()
	self:LoadConfig()
end

E:AddModule("ItemDB", Module)