local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, TT, ItemDB = E:LoadModules("Config", "Tooltip", "ItemDB")

TT.Hook = LibStub("AceHook-3.0")

--[[-------------------------------------------------------------------------

	We are caching globals, since making those functions local,
	results in a slight performance boost and therefore
	in less CPU time. Exactly what we are aiming for.

-------------------------------------------------------------------------]]--
local _
local _G							= _G
local select						= select
local format						= string.format
local match							= string.match
local GameTooltip_SetDefaultAnchor 	= GameTooltip_SetDefaultAnchor
local GetCreatureDifficultyColor 	= GetCreatureDifficultyColor
local UnitFactionGroup 				= UnitFactionGroup
local UnitName 						= UnitName
local UnitAura 						= UnitAura
local UnitLevel 					= UnitLevel
local UnitClass 					= UnitClass
local UnitRace 						= UnitRace
local GetGuildInfo 					= GetGuildInfo
local UnitHealth 					= UnitHealth
local SOURCE						= SOURCE
local C_CurrencyInfo_GetCurrencyInfo		= C_CurrencyInfo.GetCurrencyInfo or GetCurrencyInfo
local C_CurrencyInfo_GetCurrencyListInfo	= C_CurrencyInfo.GetCurrencyListInfo or GetCurrencyListInfo
local C_CurrencyInfo_ExpandCurrencyList		= C_CurrencyInfo.ExpandCurrencyList or ExpandCurrencyList
local C_CurrencyInfo_GetCurrencyListLink	= C_CurrencyInfo.GetCurrencyListLink or GetCurrencyListLink
-----------------------------------------------------------------------------

TT.CurrentTooltipAnchor = nil
TT.FactionColors = {
	ALLIANCE 	= {100, 149, 237},
	HORDE 		= {178, 34, 34}
}
TT.HexFactionColors = {
	ALLIANCE 	= E:RgbToHex(TT.FactionColors.ALLIANCE),
	HORDE 		= E:RgbToHex(TT.FactionColors.HORDE)
}

TT.BackdropTemplate = {
	  bgFile = "", 
	  edgeFile = [[Interface\AddOns\CUI\Textures\borders\WHITE8X8]],
	  tile = true,
	  tileSize = 16,
	  edgeSize = 1, 
	  insets = { left = 0, right = 0, top = 0, bottom = 0 }
}

local StyleOnlyTooltips = {
	ItemRefTooltip,
	ItemRefShoppingTooltip1,
	ItemRefShoppingTooltip2,
	AutoCompleteBox,
	FriendsTooltip,
	NamePlateTooltip,
	ShoppingTooltip1,
	ShoppingTooltip2,
	----WorldMapTooltip,
	StoryTooltip,
	EmbeddedItemTooltip,
	QuestScrollFrame.WarCampaignTooltip,
}

function TT:IsOwnedByActionBar(Tooltip)
	local Owner = Tooltip:GetOwner()
	
	if Owner and Owner:GetName() then
		return Owner:GetName():find("ActionBar")
	end
end

-- This is an easy approach to style tooltips. Just query the style table,
-- change values according to the tooltip shown and push the update via
-- TT:UpdateStyle(modPosition, styleData)
function TT:GetStyleData(type)
	-- Fallback
	if not self.TooltipStyles[type] then type = "Default"; E:debugprint("Tooltip-Style fallback to Default. Queried style not found!") end
	
	for k, v in pairs(self.TooltipStyles[type]) do
		self.CurrentTooltipStyle[type][k] = v
	end
	
	self.CurrentTooltipStyle[type].Type = type
	
	return self.CurrentTooltipStyle[type]
end

function TT:FindTextInLine(Tooltip, Text, Offset)
	local TooltipName = Tooltip:GetName()
	local Line
	
	for i=(Offset or 1), Tooltip:NumLines() do
		Line = _G[TooltipName .. "TextLeft"..i]
		if(Line and Line:GetText() and Line:GetText():find(Text)) then
			return Line
		end
	end
end

function TT:GetLevelLine(tooltip, offset)
	return self:FindTextInLine(tooltip, LEVEL, offset)
end

function TT:HasIDText(tooltip)
	return self:FindTextInLine(tooltip, ID .. ":")
end

-- self = GameTooltip
-- This happens through the hook
function TT:UpdateAuraTooltip(Unit, Index, Filter)
	if not TT.Enabled then return end
	-- When there is no owner name, the tooltip is most likely supposed to be positioned at a fixed position. We shouldn't mess with that.
		--if not self or not self:GetOwner():GetName() then return end
	
	-- Failsafes
	if Unit and Index and Filter then
		_, _, _, _, _, _, self.AuraSource, _, _, self.CurrentAuraID = UnitAura(Unit, Index, Filter)
		
		if self.AuraSource and self.AuraSource ~= "" and self.CurrentAuraID and self.CurrentAuraID ~= "" then
			self:AddLine(" ")
			self:AddDoubleLine(format("%s: %s" , SOURCE, E:GetColorizedUnitName(self.AuraSource)), format("ID: %s", self.CurrentAuraID))
		elseif self.AuraSource and self.AuraSource ~= "" then
			self:AddLine(" ")
			self:AddLine(format("%s: %s" , SOURCE, E:GetColorizedUnitName(self.AuraSource)))
		elseif self.CurrentAuraID and self.CurrentAuraID ~= "" then
			self:AddLine(" ")
			self:AddLine(format("%s: %d" , ID,  self.CurrentAuraID))
		end
		
		self:Show() -- Fix for height issues due to new line (This seems to just happen on auras somehow)
	end
	
	if Unit then
		self.CurrentStyle = TT:GetStyleData("Aura")
		
		self.CurrentColor = E:GetUnitReactionColor(Unit)
		if self.CurrentColor then
			self.CurrentStyle.BorderR = self.CurrentColor.r
			self.CurrentStyle.BorderG = self.CurrentColor.g
			self.CurrentStyle.BorderB = self.CurrentColor.b
		end
	end
	
	TT:UpdateStyle(nil, self.CurrentStyle)
end

function TT:UpdatePetTooltip(Unit, Index, Filter)
	if not TT.Enabled then return end
	
	self.CurrentStyle = TT:GetStyleData("Spell")
	self.CurrentStyle.BorderR = 0.7
	self.CurrentStyle.BorderG = 0.7
	self.CurrentStyle.BorderB = 0.7
	self.CurrentStyle.BorderA = 0.35
	
	TT:UpdateStyle(true, self.CurrentStyle)
end

function TT:UpdateSpellTooltip(Tooltip, ReturnOnly)
	if not TT.Enabled or Tooltip:IsForbidden() then return end
	
	self.CurrentStyle = TT:GetStyleData("Spell")
	self.CurrentStyle.BorderR = 0.7
	self.CurrentStyle.BorderG = 0.7
	self.CurrentStyle.BorderB = 0.7
	self.CurrentStyle.BorderA = 0.35
	
	if not ReturnOnly then
		self:UpdateStyle(true, self.CurrentStyle)
	else
		return self.CurrentStyle
	end

	_, self.CurrentSpellID = Tooltip:GetSpell()
	if not self.CurrentSpellID then
		return
	else
		if not self:HasIDText(Tooltip) then
			Tooltip:AddLine(format("%s: %s\n" , ID, self.CurrentSpellID))
		end
	end
end

function TT:UpdateUnitTooltip(object, ReturnOnly)
	if object:IsForbidden() or not TT.Enabled then return end
	
	_, self.unit = object:GetUnit()
	if not self.unit then return end
	
	if not ReturnOnly then
		self.targetUnit = self.unit .. "target"
		self.CurrentName, self.realm = UnitName(self.unit)
		
		self.TooltipUnitReactionColor  = E:GetUnitReactionColor(self.unit, false)
		self.TooltipUnitReactionHex = E:RgbToHex(self.TooltipUnitReactionColor, true)
		if not self.realm then self.realm = "" end
		if UnitIsUnit(self.unit, "player") then
			-- if UnitIsPlayer(self.unit) then self.realm = select(2, UnitFullName(self.unit)) or "" end
		end
		
		if self.realm and self.realm ~= "" then self.realm = format(" - %s", self.realm) end
		
		if not self.db then return end
		
		-- Prevent random Lua-errors
		if self.CurrentName and self.realm then
			GameTooltipTextLeft1:SetFormattedText("|c%s%s%s|r", self.TooltipUnitReactionHex, self.CurrentName, self.realm)
		end
		
		-- Colorize Race and Class
		if UnitIsPlayer(self.unit) then
			
			self.localeClass, self.class = UnitClass(self.unit)
			self.guildName, self.guildRankName, _, self.guildRealm = GetGuildInfo(self.unit)
			self.race, self.englishRace = UnitRace(self.unit)
			self.level = UnitLevel(self.unit)
			_, self.factionGroup = UnitFactionGroup(self.unit)
			
			self.classColor = CUSTOM_CLASS_COLORS and CUSTOM_CLASS_COLORS[self.class] or RAID_CLASS_COLORS[self.class]
			
			self.lineOffset = 2
			if self.guildName then
				self.guildLine = GameTooltipTextLeft2
				
				if self.guildRealm then
					self.guildLine:SetFormattedText("|c%s<%s - %s>|r [%s]", TT.HexFactionColors.ALLIANCE, self.guildName, self.guildRealm, self.guildRankName)
				else
					self.guildLine:SetFormattedText("|c%s<%s>|r [%s]", TT.HexFactionColors.ALLIANCE, self.guildName, self.guildRankName)
				end
				
				self.lineOffset = 3
			end
			
			
			self.levelLine = self:GetLevelLine(object, self.lineOffset)
			if(self.levelLine) then
				self.diffColor = GetCreatureDifficultyColor(UnitLevel(self.unit))
				if(self.factionGroup and englishRace == "Pandaren") then
					self.race = self.factionGroup.." "..self.race
				end
				self.levelLine:SetFormattedText("%s |cff%02x%02x%02x%s|r - |c%s%s, %s|r (%s)", LEVEL, self.diffColor.r * 255, self.diffColor.g * 255, self.diffColor.b * 255, self.level > 0 and self.level or "??", self.classColor.colorStr, self.race or '', self.localeClass, PLAYER)
				
				-- Color faction
				self.factionLine = _G["GameTooltipTextLeft" .. self.lineOffset + 1]
				-- Add faction
				if self.factionLine:GetText() == FACTION_ALLIANCE or self.factionLine:GetText() == FACTION_HORDE then
					if self.factionGroup == FACTION_HORDE then
						self.factionColor = self.HexFactionColors.HORDE
						self.factionLine:SetFormattedText("|c%s%s|r", self.factionColor, FACTION_HORDE)
					elseif self.factionGroup == FACTION_ALLIANCE then
						self.factionColor = self.HexFactionColors.ALLIANCE
						self.factionLine:SetFormattedText("|c%s%s|r", self.factionColor, FACTION_ALLIANCE)
					end
				else
					if self.factionGroup == FACTION_HORDE then
						self.factionColor = self.HexFactionColors.HORDE
						object:AddLine(string.format("|c%s%s|r", self.factionColor, FACTION_HORDE))
					elseif self.factionGroup == FACTION_ALLIANCE then
						self.factionColor = self.HexFactionColors.ALLIANCE
						object:AddLine(string.format("|c%s%s|r", self.factionColor, FACTION_ALLIANCE))
					end
				end
			end
		end
		
		self:SetTooltipTarget(object)
	end
	
	self.classColor = E:GetUnitReactionColor(self.unit)
	
	self.CurrentStyle = self:GetStyleData("Unit")
	
	self.CurrentStyle.BorderR = self.classColor.r
	self.CurrentStyle.BorderG = self.classColor.g
	self.CurrentStyle.BorderB = self.classColor.b
	self.CurrentStyle.BorderA = 0.35
	self.CurrentStyle.BackgroundA = self.db.background.rgba[4]
	
	
	if not ReturnOnly then
		self:UpdateStyle(false, self.CurrentStyle)
	
		GameTooltipStatusBar:SetStatusBarTexture(E.Media:Fetch("statusbar", CO.db.profile.unitframe.units.all.barTexture))
		GameTooltipStatusBar.text:SetText(E:readableNumber(UnitHealth(self.unit), 2) .. " / " .. E:readableNumber(UnitHealthMax(self.unit), 2))
	else
		return self.CurrentStyle
	end
end

function TT:SetTooltipTarget(Tooltip)
	if not TT.Enabled then return end
	
	self.GroupTargetText = ""
	self.GroupTargetTextEntries = 0
	self.GroupTargetTextNewLine = true -- Control variable for new lines

	if UnitExists(self.unit .. "target") then
		Tooltip:AddLine(format("<%s: %s>", TARGET, E:GetColorizedUnitName(self.targetUnit)))
	end
	if not IsInRaid() then
		if IsInGroup() then
			
			for i=1,4 do
				if UnitExists(format("party%starget", i)) and UnitIsUnit(self.unit, format("party%starget", i)) then
					if self.GroupTargetText == "" then
						self.GroupTargetText = E:GetColorizedUnitName(format("party%s", i))
					else
						if not self.GroupTargetTextNewLine then
							self.GroupTargetText = format("%s, %s", self.GroupTargetText, E:GetColorizedUnitName(format("party%s", i)))
						else
							self.GroupTargetText = format("%s%s", self.GroupTargetText, E:GetColorizedUnitName(format("party%s", i)))
						end
					end
					
					self.GroupTargetTextEntries = self.GroupTargetTextEntries + 1
					
					if self.GroupTargetTextEntries % 4 < 1 then
						self.GroupTargetText = self.GroupTargetText .. "\n"
						self.GroupTargetTextNewLine = true
					else
						self.GroupTargetTextNewLine = nil
					end
				end
			end
		end
	else
		for i=1,40 do
			if UnitExists(format("raid%starget", i)) and UnitIsUnit(self.unit, format("raid%starget", i)) then
				if self.GroupTargetText == "" then
					self.GroupTargetText = E:GetColorizedUnitName(format("raid%s", i))
				else
					--self.GroupTargetText = format("%s, %s", self.GroupTargetText, UnitName(format("raid%s", i)))
					if not self.GroupTargetTextNewLine then
						self.GroupTargetText = format("%s, %s", self.GroupTargetText, E:GetColorizedUnitName(format("raid%s", i)))
					else
						self.GroupTargetText = format("%s%s", self.GroupTargetText, E:GetColorizedUnitName(format("raid%s", i)))
					end
				end
				
				self.GroupTargetTextEntries = self.GroupTargetTextEntries + 1
				
				if self.GroupTargetTextEntries % 4 < 1 then
					self.GroupTargetText = self.GroupTargetText .. "\n"
					self.GroupTargetTextNewLine = true
				else
					self.GroupTargetTextNewLine = nil
				end
			end
		end
	end
	
	if self.GroupTargetText ~= "" then
		Tooltip:AddLine(format("<%s %s> %s", TARGET, L["of"], self.GroupTargetText))
	end
end

function TT:AddItemCounts(Tooltip, ItemID)
	if not self.db.showItemCounts then return end
	if (ArkInventory and ArkInventory.db.option.tooltip.itemcount.enable) or (Bagnon and Bagnon.sets.tipCount) then return end
	
	local Data = ItemDB:GetItemInfo(ItemID)
	if not Data or Data.Total < 1 then return end
	
	local CountText, CountBase, FormattedName, Color, CountTypes = nil, nil, nil, nil, {}
	
	for name, charData in pairs(Data.Chars) do
		CountText, CountBase, FormattedName = "", "", ""
		wipe(CountTypes)
		
		if charData.bags > 0 then
			CountBase = CountBase .. BACKPACK_TOOLTIP .. ": %s, "
			tinsert(CountTypes, charData.bags)
		end
		if charData.bank > 0 then
			CountBase = CountBase .. BANK .. ": %s, "
			tinsert(CountTypes, charData.bank)
		end
		if charData.void > 0 then
			CountBase = CountBase .. VOID_STORAGE .. ": %s, "
			tinsert(CountTypes, charData.void)
		end
		if charData.equipped > 0 then
			CountBase = CountBase .. L["Equipped"] .. ": %s, "
			tinsert(CountTypes, charData.equipped)
		end
		
		CountBase = CountBase:sub(1, -3)
		CountText = (CountBase):format(unpack(CountTypes))
		
		Color = RAID_CLASS_COLORS[charData.Class or "PRIEST"]
		Tooltip:AddDoubleLine(name, CountText, Color.r, Color.g, Color.b, Color.r, Color.g, Color.b)
	end
	
	Tooltip:AddLine(HONOR_LIFETIME .. ": " .. Data.Total)
end

function TT:UpdateCurrencyTooltip(index)
	local link = C_CurrencyInfo_GetCurrencyListLink(index)
	if not link then return end
	local id = tonumber(match(link,"currency:(%d+)"))	
	
	--if not self.db.showItemCounts then return end
	--if (ArkInventory and ArkInventory.db.option.tooltip.itemcount.enable) or (Bagnon and Bagnon.sets.tipCount) then return end
	
	local Data = ItemDB:GetCurrencyInfo(id)
	if not Data or Data.Total < 1 then return end
	
	local Color = {}
	
	self:AddLine("\n")
	self:AddLine(ID .. ": " .. id)
	
	for name, charData in pairs(Data.Chars) do
		if charData.current > 0 then
			Color = RAID_CLASS_COLORS[charData.Class or "PRIEST"]
			self:AddDoubleLine(name, BreakUpLargeNumbers(charData.current), Color.r, Color.g, Color.b, Color.r, Color.g, Color.b)
		end
	end
	
	self:AddLine(HONOR_LIFETIME .. ": " .. BreakUpLargeNumbers(Data.Total))
	self:Show()
end

function TT:UpdateItemTooltip(Tooltip, ReturnOnly)
	if not TT.Enabled then return end
	
	self.CurrentName, self.CurrentLink = Tooltip:GetItem()
	if not self.CurrentLink then return end
	
	local ItemID = GetItemInfoInstant(self.CurrentLink)
	if ItemID then
		if not self:HasIDText(Tooltip) then
			Tooltip:AddLine(format("%s: %s\n" , ID, ItemID))
			Tooltip:Show()
			
			self:AddItemCounts(Tooltip, ItemID)
		end
	end
	
	self.CurrentStyle = self:GetStyleData("Item")
	
	self.CurrentColor = ITEM_QUALITY_COLORS[E:GetItemLinkInfo(self.CurrentLink).itemRarity] or ITEM_QUALITY_COLORS[1]
	
	-- PawnCommon.ColorTooltipBorder
	
	if self.CurrentLink and (C_AzeriteEmpoweredItem.IsAzeriteEmpoweredItemByID(self.CurrentLink) or C_AzeriteItem.IsAzeriteItemByID(self.CurrentLink)) then		
		self.CurrentStyle.BackgroundR = nil
		self.CurrentStyle.BackgroundG = nil
		self.CurrentStyle.BackgroundB = nil
		self.CurrentStyle.BackgroundA = nil
		
		self.CurrentStyle.BorderR = 1
		self.CurrentStyle.BorderG = 0.85
		self.CurrentStyle.BorderB = 0
		self.CurrentStyle.BorderA = 1
	else
	
		self.CurrentStyle.BackgroundA = self.db.background.rgba[4]
		
		-- Fix for Pawn
		if not PawnCommon or (PawnCommon and not PawnCommon.ColorTooltipBorder) then
			self.CurrentStyle.BorderR = self.CurrentColor.r
			self.CurrentStyle.BorderG = self.CurrentColor.g
			self.CurrentStyle.BorderB = self.CurrentColor.b
		else
			self.CurrentStyle.OverrideBorder = false
		end
	end
	
	if not ReturnOnly then
		self:UpdateStyle(false, self.CurrentStyle)
	else
		return self.CurrentStyle
	end
end

-- I have no idea what fires this
function TT:UpdateQuestTooltip()
	if not TT.Enabled then return end
	
	self:UpdateStyle(nil, self:GetStyleData("Quest"))
end

function TT:UpdateFonts()
	if self.Enabled then		
		GameTooltipHeaderText:SetFont(E.Media:Fetch("font", self.db.header["fontType"]), self.db.header["fontSize"], self.db.header["fontFlags"])
		GameTooltipText:SetFont(E.Media:Fetch("font", self.db.body["fontType"]), self.db.body["fontSize"], self.db.body["fontFlags"])
		GameTooltipStatusBar.text:SetFont(E.Media:Fetch("font", self.db.statusbar["fontType"]), self.db.statusbar["fontSize"], self.db.statusbar["fontFlags"])
	else
		GameTooltipHeaderText:SetFont(unpack(GameTooltipHeaderText.Font))
		GameTooltipText:SetFont(unpack(GameTooltipText.Font))
		GameTooltipStatusBar.text:SetFont(unpack(GameTooltipStatusBar.text.Font))
	end
end

local function UpdateBackdrop(Tooltip, elapsed)
	if not TT.Enabled then return end
	--TT.BackdropElapsed = (TT.BackdropElapsed or 0) + elapsed
	
	--if TT.BackdropElapsed > 0 then
		if not Tooltip or Tooltip:IsForbidden() or not TT.db.background then return end
		local r,g,b,a = unpack(TT.db.background.rgba)
		Tooltip:SetBackdropColor(r,g,b,a, true)
		
		--TT.BackdropElapsed = 0
	--end
end

function TT:GetUpdatedStyle(Tooltip)
	if not Tooltip then return end
	
	if Tooltip:GetSpell() then
		return self:UpdateSpellTooltip(Tooltip, true)
	elseif Tooltip:GetItem() then
		return self:UpdateItemTooltip(Tooltip, true)
	elseif Tooltip:GetUnit() then
		return self:UpdateUnitTooltip(Tooltip, true)
	end
end

-- Style update based on the provided style table
-- Note: This can be used by plugins to easily change the tooltips style!
function TT:UpdateStyle(modPosition, styleData, applyStyle)
	if not TT.CurrentTooltip or not TT.Enabled or TT.CurrentTooltip:IsForbidden() or TT.CurrentTooltip.IsEmbedded then return end
	
	local Tooltip = TT.CurrentTooltip
	
	-- Fix for Bug#002: Check if object has the GetOwner method, since autocomplete frames also are handled by CUI and are not the same as tooltips
	if Tooltip.GetOwner then
		local AnchorType = Tooltip:GetAnchorType()
		
		if not (AnchorType ~= "ANCHOR_NONE" or Tooltip == NamePlateTooltip) then
			local Parent 	 = select(2, Tooltip:GetPoint())
			local IsUIParent = Parent == UIParent or Parent == E.Parent
			
			-- Wether the tooltip position should be modified
			if not E:tableContainsValue(StyleOnlyTooltips, Tooltip) or Tooltip == GameTooltip then
				if modPosition and (AnchorType ~= "ANCHOR_CURSOR" and IsUIParent) then
					Tooltip:ClearAllPoints()
					Tooltip:SetPoint("BOTTOMRIGHT", self.CurrentTooltipAnchor, "BOTTOMRIGHT", 0, TT.StatusBarIsShown and 10 or 0)
				end
			end
		end
	end
	
	-- Stop if there's no style and the tooltip shows an spell/item/unit to prevent unecessary border overrides (also looks weird)
	if not styleData and (Tooltip.GetSpell) and (Tooltip:GetSpell() or Tooltip:GetItem() or Tooltip:GetUnit()) then
			styleData = TT:GetUpdatedStyle(Tooltip)
				
			if not styleData then return end
		end
	
	if applyStyle ~= false then
		-- Fallback
		if not styleData then styleData = self:GetStyleData("Default") end
		
		-- Get missing data
		if styleData.OverrideBorder and (not styleData.BorderR or not styleData.BorderG or not styleData.BorderB) then
			styleData.BorderR, styleData.BorderG, styleData.BorderB = 1, 1, 1
		end
		
		-- Set border size
		self.BackdropTemplate.edgeSize = styleData.BorderSize
		Tooltip:SetBackdrop(self.BackdropTemplate)
		-- For now, we let the user control a global RGBA for the background
		UpdateBackdrop(Tooltip)
		
		if styleData.OverrideBorder then
			Tooltip:SetBackdropBorderColor(styleData.BorderR, styleData.BorderG, styleData.BorderB, styleData.BorderA)
		end
	end
end

function TT:OverrideStyle()
	if not self or not TT.Enabled then return end
	
	TT.CurrentTooltip = self
	TT:UpdateStyle(false)
end

function TT:UpdateTooltip(Parent)
	if not self or not TT.Enabled then return end
	
	TT.CurrentTooltip = self
	TT:UpdateStyle(true)
end

function TT:UpdateStatusBar()
	_, self.Max = self:GetMinMaxValues()
	self.Current = self:GetValue()
	if self.Max then
		self.text:SetText(format("%s / %s", E:readableNumber(self.Current, 2), E:readableNumber(self.Max, 2)))
	end
end

function TT:SetupAnchor()
	
	self.CurrentTooltipAnchor = E:NewFrame("Frame", "TooltipAnchor", "MEDIUM", 100, 50, _, E.Parent)
	
	GameTooltip_SetDefaultAnchor(GameTooltip, self.CurrentTooltipAnchor)
	E:CreateMover(self.CurrentTooltipAnchor, L["tooltipAnchor"], nil, nil, nil, nil, "misc")
end

local function UpdateStyleOnly(self)
	TT.CurrentTooltip = self
	
	TT:UpdateStyle(false)
end

local function UpdatePositionOnly(self)
	TT.CurrentTooltip = self
	
	TT:UpdateStyle(true, nil, false)
end

local function SetToyByItemID(Tooltip, id)
	if Tooltip:IsForbidden() or not TT.Enabled then return end
	
	Tooltip:AddLine(format("%s: %s\n" , ID, id))
	Tooltip:Show()
end
local function OnTooltipSetItem(Tooltip)
	TT:UpdateItemTooltip(Tooltip)
end
local function OnTooltipSetUnit(Tooltip)
	TT:UpdateUnitTooltip(Tooltip)
end
local function OnTooltipSetQuest(Tooltip)
	TT:UpdateQuestTooltip(Tooltip)
end
local function OnTooltipSetSpell(Tooltip)
	TT:UpdateSpellTooltip(Tooltip)
end

function TT:SetupHandlers()
	self.Hook:SecureHookScript(GameTooltip, "OnTooltipSetItem", OnTooltipSetItem)
	self.Hook:SecureHookScript(GameTooltip, "OnTooltipSetUnit", OnTooltipSetUnit)
	self.Hook:SecureHookScript(GameTooltip, "OnTooltipSetQuest", OnTooltipSetQuest)
	self.Hook:SecureHookScript(GameTooltip, "OnTooltipSetSpell", OnTooltipSetSpell)
	
	self.Hook:SecureHook(GameTooltip, "SetToyByItemID", SetToyByItemID)
	
	hooksecurefunc(GameTooltip, "SetUnitAura", TT.UpdateAuraTooltip)
	hooksecurefunc(GameTooltip, "SetUnitBuff", TT.UpdateAuraTooltip)
	hooksecurefunc(GameTooltip, "SetUnitDebuff", TT.UpdateAuraTooltip)
	hooksecurefunc(GameTooltip, "SetPetAction", TT.UpdatePetTooltip)
	hooksecurefunc(GameTooltip, "SetCurrencyToken", TT.UpdateCurrencyTooltip)
	
	if GameTooltip_SetBackdropStyle then
		hooksecurefunc("GameTooltip_SetBackdropStyle", UpdateStyleOnly)
	elseif SharedTooltip_SetBackdropStyle then
		hooksecurefunc("SharedTooltip_SetBackdropStyle", UpdateStyleOnly)
	end
	
	GameTooltip:HookScript("OnShow", UpdateStyleOnly)
	
	hooksecurefunc("GameTooltip_SetDefaultAnchor", UpdatePositionOnly)
	--hooksecurefunc("UnitPowerBarAlt_OnEnter", TT.UpdateTooltip)

	for _, Tooltip in pairs(StyleOnlyTooltips) do
		if Tooltip then
			TT.Hook:SecureHookScript(Tooltip, "OnShow", UpdateStyleOnly)
			UpdateStyleOnly(Tooltip)
		end
	end
	
	-- Initial update, because otherwise the first show-up wouldn't be styled
	UpdateStyleOnly(GameTooltip)
	
	-- Despite all hooks and events, World objects that show tooltips simply aren't always affected by the styling and revert back to a default backdrop somehow
	-- This has an extremely low cpu usage, so we basically can neglect it in that regards
	TT.Hook:SecureHookScript(GameTooltip, "OnUpdate", UpdateBackdrop)
	
	TT.Hook:SecureHookScript(GameTooltipStatusBar, 'OnValueChanged', TT.UpdateStatusBar)
	GameTooltipStatusBar:SetScript("OnShow",function(self)
		-- Fix for random bug
		if GameTooltip and GameTooltip:GetUnit() then
			TT.StatusBarIsShown = true
			TT:UpdateStyle(true, nil, false)
			
			if TT.Enabled then
				self.text:Show()
			else
				self.text:Hide()
			end
		else
			self:Hide()
		end
	end)
	GameTooltipStatusBar:SetScript("OnHide",function(self)
		TT.StatusBarIsShown = nil
		TT:UpdateStyle(true, nil, false)
	end)
end

function TT:Enable()
	if not self.Initalized then		
		--self.BackdropTemplate.bgFile = self.db.background["backgroundFile"]
		self.BackdropTemplate.bgFile = [[Interface\AddOns\CUI\Textures\borders\WHITE8X8]]
		
		self:UpdateFonts()
		
		if IsAddOnLoaded("Pawn") and PawnCommon and PawnCommon.ColorTooltipBorder then
			E:print("Pawn detected. Its option to colorize tooltip borders is enabled. CUI colorization disabled.")
		end
		
		self.Initalized = true
	end
	
	self.Enabled = true
	self.CurrentTooltipAnchor.ForceMoverEnabled = nil
end

function TT:Disable()
	self.Enabled = nil
	self.CurrentTooltipAnchor.ForceMoverEnabled = false
	
	GameTooltipStatusBar:SetStatusBarTexture(GameTooltipStatusBar.BarTexture)
end

function TT:LoadConfig()
	if self.db.enable then
		self:Enable()
	else
		self:Disable()
	end
	
	self:UpdateFonts()
end
function TT:UpdateDB()
	self.db = CO.db.profile.tooltip
end
function TT:Init()
	self:UpdateDB()
	
	GameTooltipStatusBar.text = GameTooltipStatusBar:CreateFontString(nil, "OVERLAY")
	E:InitializeFontFrame(GameTooltipStatusBar.text, "OVERLAY", "FRIZQT__.TTF", 12, {1,1,1}, 1, {0,0}, "10101", 200, 100, GameTooltipStatusBar, "CENTER", {0,0})
	
	-- Cache default font sizes
	GameTooltipHeaderText.Font = {GameTooltipHeaderText:GetFont()}
	GameTooltipText.Font = {GameTooltipText:GetFont()}
	GameTooltipStatusBar.text.Font = {GameTooltipStatusBar.text:GetFont()}
	
	GameTooltipStatusBar.BarTexture = GameTooltipStatusBar:GetStatusBarTexture():GetTexture()
	
	E.Libs.LibSmooth:SmoothBar(GameTooltipStatusBar)
	
	self:SetupAnchor()
	self:SetupHandlers()
	
	self:LoadConfig()
end

E:AddModule("Tooltip", TT)