local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, UF = E:LoadModules("Config", "Unitframes")

----------------------------------------------------------
	local UnitGetTotalAbsorbs 		= UnitGetTotalAbsorbs
	local UnitGetTotalHealAbsorbs 	= UnitGetTotalHealAbsorbs
----------------------------------------------------------

local AbsorbTypes = {"damage", "healing"}
local Module = {}

-----------------------------------------

Module.Frames = {}

--[[------------------------------------------------
	
	This is the CUI library to display absorb
	on a units healthbar.
	
	The profile loader is being handled
	in the Unitframes module.
	
	This is simply an extension.
	
------------------------------------------------]]--

function Module:LoadConfig()
	local Config, HealthConfig, Element, ElementConfig
	local AllDisabled
	
	for _, F in pairs(self.Frames) do
		Config = CO.db.profile.unitframe.units[F.ConfigKey]
		HealthConfig = Config.health
		
		AllDisabled = true
		
		for _, Type in pairs(AbsorbTypes) do
			Element = F.Health.Absorb[Type]
			ElementConfig = HealthConfig.absorbs[Type]
			
			if ElementConfig.enable then
				AllDisabled = false
				
				-- As absorb is basically a statusbar, we can actually handle it like the healthbar					
				F.Health:SetSubBar(Element, true, not HealthConfig.barInverseFill, HealthConfig.barOrientation, not ElementConfig.onHealthbar)
				
				Element:SetAlpha(ElementConfig.alpha)
				
				-- Let user toggle between stripes or full cover
				if ElementConfig.useStripes then
					Element.Border.Background:SetTexture(E.Media:Fetch("statusbar", "CUI Absorb Stripes"), "REPEAT", "REPEAT")
				else
					Element.Border.Background:SetTexture([[Interface\AddOns\CUI\Textures\borders\WHITE8X8]])
				end
				Element.Border:SetBackdropBorderColor(ElementConfig.borderColor[1], ElementConfig.borderColor[2], ElementConfig.borderColor[3], ElementConfig.borderColor[4])
				Element.Border.Background:SetVertexColor(ElementConfig.textureColor[1], ElementConfig.textureColor[2], ElementConfig.textureColor[3], ElementConfig.textureColor[4])
				
				Element.TextureSizeMultiplier = ElementConfig.textureSizeMultiplier or 7
				
				Element.Disabled = false
			else
				Element.Disabled = true
				Element:Hide()
			end
		end
		
		if not AllDisabled then
			self:InitEvents(F.Health.Absorb)
			self:Update(F.Health.Absorb)
		else
			self:RemoveEvents(F.Health.Absorb)
		end
	end
end

function Module:InitEvents(AbsorbFrame)
	self:RemoveEvents(AbsorbFrame)
	
	if not E:IsRunningShadowlands() then
		AbsorbFrame:RegisterUnitEvent("UNIT_HEALTH_FREQUENT", AbsorbFrame.Owner.Unit)
	end
	AbsorbFrame:RegisterUnitEvent("UNIT_MAXHEALTH", AbsorbFrame.Owner.Unit)
	AbsorbFrame:RegisterUnitEvent("UNIT_ABSORB_AMOUNT_CHANGED", AbsorbFrame.Owner.Unit)
	AbsorbFrame:RegisterUnitEvent("UNIT_HEAL_ABSORB_AMOUNT_CHANGED", AbsorbFrame.Owner.Unit)
	
	AbsorbFrame:SetScript("OnEvent", self.BarFrame_OnEvent)
end
function Module:RemoveEvents(AbsorbFrame)
	AbsorbFrame:UnregisterAllEvents()
	AbsorbFrame:SetScript("OnEvent", nil)
end
-- /dump UnitGetTotalAbsorbs("player")
function Module:ForceUpdate()
	 Module.BarFrame_OnEvent(self, 'ForceUpdate')
end

function Module:BarFrame_OnEvent(event, ...)
	Module:Update(self)
end

local function UpdateTexCoord(Element, Value, Unit)
	local Coord  = (Element.TextureSizeMultiplier or 7) * (Value / UnitHealthMax(Unit))
				
	-- Clamp to 0 and 50 to avoid "out of range"
	if Coord < 0 then Coord = 0; elseif Coord > 50 then Coord = 50; end 
	
	Element.Border.Background:SetTexCoord(0, Coord, 0, Coord)
end

function Module:Update(BarFrame)
	local Unit = BarFrame.Owner.Unit
	local Value, Element
	
	for Index, Type in pairs(AbsorbTypes) do
		
		Element = BarFrame[Type]
		
		if not Element.Disabled then
			if Index == 1 then
				Value = UnitGetTotalAbsorbs(Unit) or 0				
			else
				Value = UnitGetTotalHealAbsorbs(Unit) or 0
			end
			
			-- Testing
			-- Value = 10000
			
			-- Just less than 1, because Buffs like Blessing of the Kings cause the unit to always have 1 absorb somehow
			if Value <= 1 then
				Element:Hide()
			else
				Element:SetMinMaxValues(0, UnitHealth(Unit))
				Element:SetValue(Value)
				
				Element:Show()
				UpdateTexCoord(Element, Value, Unit)
			end
		end
	end
	
	
	
end

function Module:InitBar(Bar)
	Bar:Hide()
	
	Bar:SetStatusBarTexture(E.Media:Fetch("statusbar", "CUI Absorb Stripes"))
	Bar:GetStatusBarTexture():SetVertexColor(0, 0, 0, 0) -- Make texture invisible, since we don't need it anyways
	Bar:SetValue(0)
	
	Bar:SetAlpha(0.5)
	
	Bar.Border = E:CreateBackdropFrame("Frame", nil, Bar)
	
	-- Basically scale every corner X pixels inwards to make some space for the healthbar border
	Bar.Border:SetPoint("TOPLEFT", Bar:GetStatusBarTexture(), "TOPLEFT", 1, -1)
	Bar.Border:SetPoint("BOTTOMRIGHT", Bar:GetStatusBarTexture(), "BOTTOMRIGHT", -1, 1)
	
	Bar.Border:SetBackdrop({
        bgFile = [[Interface\AddOns\CUI\Textures\borders\WHITE8X8]], 
        edgeFile = [[Interface\AddOns\CUI\Textures\borders\WHITE8X8]], 
        edgeSize = 3,
    })
	Bar.Border.Background = Bar.Border:CreateTexture(nil, "OVERLAY")
	Bar.Border.Background:SetAllPoints(Bar.Border)
	
    Bar.Border.Background:SetTexCoord(0,3,0,3)
	Bar.Border:SetBackdropColor(0, 0, 0, 0.3)
	
	Bar:SetFrameLevel(Bar.Owner.Owner:GetFrameLevel() + 2)
	Bar:SetReverseFill(true)
	E.Libs.LibSmooth:SmoothBar(Bar)
end

function Module:Create(Frame)
	local Bar = Frame.Health
	if not Bar then return end
	
	-- Handles Events for this Frame
	Bar.Absorb = CreateFrame("Frame", nil, Bar)
	Bar.Absorb:SetAllPoints(Bar)
	Bar.Absorb.Owner = Frame
	Bar.Absorb.ForceUpdate = self.ForceUpdate
	
	for _, Type in pairs(AbsorbTypes) do
		Bar.Absorb[Type] = E:CreateBackdropFrame("Statusbar", nil, Bar)
		Bar.Absorb[Type].Owner = Bar.Absorb
		self:InitBar(Bar.Absorb[Type])
	end
	
	tinsert(self.Frames, Frame)
end

---------- Add Module
UF:RegisterModule('HealthAbsorb', Module)