local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, UF, AUR, B = E:LoadModules("Config", "Unitframes", "Auras", "Blizzard")

local _
AUR.E = CreateFrame("Frame")

------------------------------------
local format		= string.format
local GetTime		= GetTime
------------------------------------


-- AurasNamespace = AUR

AUR.AURA_SIZE 				= 32 -- X and Y size
AUR.AURA_MARGIN 			= 5 -- Margin between auras in px
AUR.BUFF_PER_ROW 			= 8
AUR.BUFF_NUM_PLAYER 		= 32
AUR.DEBUFF_NUM_PLAYER 		= 16
AUR.AURA_TYPES 				= {"Buff", "Debuff"}
local HarmfulColor			= {0.85, 0, 0}
local Masque = E.Libs.Masque
local MasqueGroup_Buffs = Masque and Masque:Group("CUI", format("%s %s", L["player"],  L["Buffs"]))
local MasqueGroup_Debuffs = Masque and Masque:Group("CUI", format("%s %s", L["player"],  L["Debuffs"]))

function AUR:LoadConfig()
	
	self.db = CO.db.profile.unitframe
	
	local Config
	for k, header in pairs(self.Headers) do
		if header:GetName() == "CUIPlayerBuffs" then Config = self.db.buffs else Config = self.db.debuffs end
		
		header.SortMethod 		= Config.sortMethod
		header.SortDirection 	= Config.sortDirection
		header.MaxWraps 		= Config.maxWraps
		header.MaxPerRow 		= Config.maxPerRow
		header.GrowthDirectionX = Config.growthDirectionX
		header.GrowthDirectionY = Config.growthDirectionY
		header.Size 			= Config.size
		header.GapX 			= Config.gapX
		header.GapY 			= Config.gapY
		
		header.Point = ""
		
		header.WrapY = header.Size + header.GapY
		if header.GrowthDirectionY == "DOWN" then
			header.WrapY = header.WrapY * (-1) -- Reverse
			header.Point = "TOP"
		else
			header.Point = "BOTTOM"
		end
		
		header.xOffset = header.Size + header.GapX
		if header.GrowthDirectionX == "LEFT" then
			header.xOffset = header.xOffset * (-1) -- Reverse
			header.Point = header.Point .. "RIGHT"
		else
			header.Point = header.Point .. "LEFT"
		end
		
		header.useClassColor = Config.borderUseClassColor or false
		
		header.Width = ((header.Size + header.GapX) * header.MaxPerRow)
		header.Height = ((header.Size + header.GapY) * header.MaxWraps)
		
		header:SetAttribute("headerWidth", header.Width)
		header:SetAttribute("headerHeight", header.Height)
		header:SetSize(header.Width, header.Height)
		E:UpdateMoverDimensions(header)
		
		self:UpdateHeader(header)
	end
end

function AUR:ColorizeAll()
	for k, header in pairs(self.Headers) do
		for k, button in pairs({ header:GetChildren() }) do
			if button.filter == "HARMFUL" then
				E:ColorizeAuraButton(button, button.AuraDType, button.unit, button.filter, button.AuraName, button.AuraSpellID, nil, button.debuffColor)
			else
				E:ColorizeAuraButton(button, button.AuraDType, button.unit, button.filter, button.AuraName, button.AuraSpellID, header.useClassColor)
			end
		end
	end
end


local QualityColors = {}
do
	for i=0,7 do
		QualityColors[i] = {GetItemQualityColor(i)}
	end
end

function AUR:UpdateEnchant(self, index)
	
	local InfoOffset = (strmatch(self:GetName(), '2$') and 6) or 2
	local duration, remaining = 600, 0
	local expiration = select(InfoOffset, GetWeaponEnchantInfo())
	local charges = select(InfoOffset+1, GetWeaponEnchantInfo())
	
	self.HasTempEnchant = true
	if not self.header then
		self.header = self:GetParent()
	end
	
	if charges and charges > 1 then
		self.count:SetText(charges)
	else
		self.count:SetText('')
	end
	
	self.Tex:SetTexture(GetInventoryItemTexture('player', index))
	
	if expiration then
		
		local quality = GetInventoryItemQuality('player', index)
		local color
		if quality and quality > 1 then
			color = QualityColors[quality]
			
			E:ColorizeAuraButton(self, nil, nil, nil, nil, nil, nil, color)
		else
			E:ColorizeAuraButton(self, nil, nil, nil, nil, nil, self.header.useClassColor)
		end
		
		remaining = expiration / 1000
		if remaining <= 3600 and remaining > 1800 then
			duration = 3600
		elseif remaining <= 1800 and remaining > 600 then
			duration = 1800
		end
		
		self.AuraExpirationTime, self.AuraDuration = remaining, duration
		
		if duration <= 0.05 then
			self.time:SetText('')
			
			self:SetScript("OnUpdate", nil)
			self.Cooldown:Hide()
		else
			self:SetScript("OnUpdate", AUR.Button_OnUpdate)
			
			--CooldownFrame_SetDisplayAsPercentage(self.Cooldown, (duration / remaining) - 1)
		end
	else
		self.time:SetText('')
		self:SetScript("OnUpdate", nil)
		self.Cooldown:Hide()
	end
end

local function AuraAttributeChanged(self, attribute, index)
	if attribute == "target-slot" then AUR:UpdateEnchant(self, index); return end
	if attribute == "index" then
		
		-- Refresh those values all the time, since it seems to cause problems when we don't
		if not self.header then
			self.header = self:GetParent()
		end
		
		self.HasTempEnchant = nil
		
		self.index = index
		self.unit = self.header:GetAttribute("unit")
		self.filter = self.header:GetAttribute("filter")
		self.AuraName, self.AuraTexture, self.AuraCount, self.AuraDType, self.AuraDuration, self.AuraExpirationTime, _, _, _, self.AuraSpellID = UnitAura(self.unit, index, self.filter)
		if self.AuraName then
			if self.filter == "HARMFUL" then
				if self.AuraDType then
					self.debuffColor = DebuffTypeColor[self.AuraDType]
				else
					self.debuffColor = DebuffTypeColor["none"]
				end
				
				E:ColorizeAuraButton(self, self.AuraDType, self.unit, self.filter, self.AuraName, self.AuraSpellID, nil, self.debuffColor)
			else
				E:ColorizeAuraButton(self, self.AuraDType, self.unit, self.filter, self.AuraName, self.AuraSpellID, self.header.useClassColor)
				--Module:ColorizeAura(Slot, DType, Unit, UnitAuraClass, AuraName, SpellID)
				--E:SkinButtonIcon(self.texture, E:ParseDBColor(self.header.useClassColor))
			end
			
			if self.AuraCount and self.AuraCount > 1 then
				self.count:SetText(self.AuraCount)
			else
				self.count:SetText(E.STR.EMPTY)
			end
			
			self.Tex:SetTexture(self.AuraTexture)
			
			if self.AuraDuration <= 0 then
				self.time:SetText(E.STR.EMPTY)
				
				self:SetScript("OnUpdate", nil)
				self.Cooldown:Hide()
			else
				self:SetScript("OnUpdate", AUR.Button_OnUpdate)
				self.Cooldown:Show()
				
				self.Cooldown:SetCooldown(self.AuraExpirationTime - self.AuraDuration, self.AuraDuration)
			end
		end
	end
end

local function AuraButton_OnEnter(self)
	self.IsHovered = true
end
local function AuraButton_OnLeave(self)
	self.IsHovered = nil
	GameTooltip:Hide()
end

function AUR:SetupCooldown(Button)
	Button.Cooldown = CreateFrame("Cooldown", nil, Button, "CooldownFrameTemplate")
	Button.Cooldown:SetParent(Button)
	Button.Cooldown:SetAllPoints(Button)
	Button.Cooldown:SetReverse(true)
	Button.Cooldown:Show()
end

local CreateIconFont = "FRIZQT__.TTF"
function AUR:CreateIcon(button)

	-- This method gets called when a new button was created automatically by the "SecureAuraHeaderTemplate" attribute "template".
	-- The template is simply an XML construct with several initial data for the frame.
	-- Right here, we can do all sort of "post-processing" for that frame(s).
	-- @TODO
	-- 		Make the method more dynamic and allow aura bars to be created through it.
	--		Maybe a simple bool?
	
	button.Tex = button:CreateTexture(nil, "BORDER")
	button.Tex:SetTexCoord(0,1,0,1)
	--E:SkinButtonIcon(button.texture, E:GetUnitClassColor("player"))	
	
	button.Tex:SetAllPoints(button)
	button.border:Hide()
	
	AUR:SetupCooldown(button)
	
	button.FontHolder = CreateFrame("Frame", nil, button)
	button.FontHolder:SetAllPoints(button.Cooldown)
	
	button.Cooldown:SetHideCountdownNumbers(true)

	button.count = button.FontHolder:CreateFontString(nil, "ARTWORK")
	E:InitializeFontFrame(button.count, "ARTWORK", CreateIconFont, 14, {1,1,1}, 1, {0,0}, "", 0, 0, button.FontHolder, "CENTER", {1,1})
	button.count:ClearAllPoints()
	button.count:SetParent(button.FontHolder)

	button.time = button.FontHolder:CreateFontString(nil, "ARTWORK")
	E:InitializeFontFrame(button.time, "ARTWORK", CreateIconFont, 11, {1,0.96,0.41}, 1, {0,0}, "", 0, 0, button.FontHolder, "CENTER", {1,1})
	button.time:SetParent(button.FontHolder)
	
	if button:GetParent():GetAttribute("filter") == "HARMFUL" then
		E:RegisterPathFont(button.count, "CO.db.profile.unitframe.debuffs.count")
		E:RegisterPathFont(button.time, "CO.db.profile.unitframe.debuffs.time")
		
		E:UpdatePathFont("CO.db.profile.unitframe.debuffs.count")
		E:UpdatePathFont("CO.db.profile.unitframe.debuffs.time")
	else
		E:RegisterPathFont(button.count, "CO.db.profile.unitframe.buffs.count")
		E:RegisterPathFont(button.time, "CO.db.profile.unitframe.buffs.time")
		
		E:UpdatePathFont("CO.db.profile.unitframe.buffs.count")
		E:UpdatePathFont("CO.db.profile.unitframe.buffs.time")
	end
		
	button.elapsed = 0
	button:HookScript("OnEnter", AuraButton_OnEnter)
	button:HookScript("OnLeave", AuraButton_OnLeave)

	-- This Script gets called every time a aura changed/was added or removed.
	-- We can use this to update the whole thing and its children.
	button:SetScript("OnAttributeChanged", AuraAttributeChanged)
	
	if not MasqueGroup_Buffs or not MasqueGroup_Debuffs then return end
	
	button.Highlight = E:CreateHighlight(button)
	button.Highlight:SetColorTexture(1,1,0, 0.15)
	
	local ButtonData = {
		FloatingBG = nil,
		Icon = button.Tex,
		Cooldown = button.Cooldown,
		Flash = nil,
		Pushed = nil,
		Normal = nil,
		Disabled = nil,
		Checked = nil,
		Border = nil,
		AutoCastable = nil,
		Highlight = button.Highlight,
		HotKey = nil,
		Count = false,
		Name = nil,
		Duration = false,
		AutoCast = nil,
	}
	
	local Target
	if CO.db.char.unitframe.buffs.useMasque and button:GetParent():GetAttribute("filter") == "HELPFUL" then
		Target = MasqueGroup_Buffs
	elseif CO.db.char.unitframe.debuffs.useMasque and button:GetParent():GetAttribute("filter") == "HARMFUL" then
		Target = MasqueGroup_Debuffs
	end
	if Target then
		Target:AddButton(button, ButtonData)
		-- Don't ReSkin here, as it will: Impact performance, due to rapid creation of buttons and cause flickering, since the whole group is being iterated
		--Target:ReSkin()
		
		-- Not needed anymore, but let's keep this
		if button.__MSQ_BaseFrame then
			button.__MSQ_BaseFrame:SetFrameLevel(2) --Lower the framelevel to fix issue with buttons created during combat
		end
	end
end

function AUR:Button_OnUpdate(elapsed)
	self.elapsed = self.elapsed - elapsed;

	if ( self.elapsed <= 0 ) then
		--------------------------------------------------------------
		-- OnUpdate Code BEGIN
		
		--if self.AuraDuration ~= 0 and self.AuraExpirationTime ~= nil then
			
			
			if not self.HasTempEnchant then
				self.timeLeftBase = E:makePositive(E:Round(GetTime() - self.AuraExpirationTime, 2))
				
			else
				self.timeLeftBase = self.AuraExpirationTime
			end
			
			if self.timeLeftBase > 10 then self.timeLeft = E:FormatTime(self.timeLeftBase, 0); else self.timeLeft = E:FormatTime(self.timeLeftBase, 1); end
			self.time:SetText(self.timeLeft)
		--end
		
		if self.IsHovered and not self.HasTempEnchant then
			if not GameTooltip:IsOwned(self) then
				GameTooltip:SetOwner(self)
			end
			GameTooltip:SetUnitAura(self.unit, self.index, self.filter)
			
			GameTooltip:Show()
		end
		

		-- OnUpdate Code END
		--------------------------------------------------------------
		self.elapsed = 0.07;
	end
end

function AUR:UpdateHeader(header)
	-- By setting the template, everything starts.
	-- The buttons are being created by the secure aura header
	-- To be exact, it reads the set filter and calculates the number of total auras to display.
	-- Based on that, AUR:CreateIcon() gets called by the OnLoad handler of the template.
	-- This way, we can modify the auras however we want to.
	-- We simply need different headers for multiple aura bars.
	-- That said, a frame with the "SecureAuraHeaderTemplate" simply needs to be created. Followed by the SetAttribute("template") or SetAttribute("weaponTemplate")
	-- Maybe XML proves to be not that bad.	
	
	header:SetAttribute("consolidateTo", 0)
	header:SetAttribute('weaponTemplate', ("CUIAuraTemplate%d"):format(header.Size))
	
	header:SetAttribute("separateOwn", 0)
	header:SetAttribute("sortMethod", header.SortMethod)
	header:SetAttribute("sortDirection", header.SortDirection)
	
	header:SetAttribute("maxWraps", header.MaxWraps)
	header:SetAttribute("wrapAfter", header.MaxPerRow)

	header:SetAttribute("point", header.Point)
	
		header:SetAttribute("minWidth", header:GetAttribute("headerWidth"))
		header:SetAttribute("minHeight", header:GetAttribute("headerHeight"))
		header:SetAttribute("xOffset", header.xOffset)
		header:SetAttribute("yOffset", 0)
		header:SetAttribute("wrapXOffset", 0)
		header:SetAttribute("wrapYOffset", header.WrapY)

	header:SetAttribute("template", ("CUIAuraTemplate%d"):format(header.Size))
	
	-- Post-fix of values we have to update manually
	local index = 1
	for k, child in pairs({ header:GetChildren() }) do
		-- Set new size
		child:SetSize(header.Size, header.Size)
		
		if (index > (header.MaxWraps * header.MaxPerRow)) and child:IsShown() then
			child:Hide()
		end
		
		index = index + 1
	end
	
	-- To actually apply the size
	if MasqueGroup_Buffs and CO.db.char.unitframe.buffs.useMasque then
		MasqueGroup_Buffs:ReSkin()
	elseif MasqueGroup_Debuffs and CO.db.char.unitframe.debuffs.useMasque then
		MasqueGroup_Debuffs:ReSkin()
	end
	
	self:ColorizeAll()
end

local function CreatePlayerAuraHeader(filter)
	local name = "CUIPlayerDebuffs"
	if filter == "HELPFUL" then name = "CUIPlayerBuffs" end
	
	local header = CreateFrame("Frame", name, E.Parent, "SecureAuraHeaderTemplate")
	header:SetClampedToScreen(true)
	header:SetAttribute("unit", "player")
	header:SetAttribute("filter", filter)
	
	RegisterStateDriver(header, "visibility", "[petbattle] hide; show")
	RegisterAttributeDriver(header, "unit", "[vehicleui] vehicle; player")

	if filter == "HELPFUL" then
		header:SetAttribute('consolidateDuration', nil)
		header:SetAttribute("includeWeapons", 1)
	end
	
	header:SetAttribute("headerWidth", 300)
	header:SetAttribute("headerHeight", 90)
	
	header:SetSize(300, 90)
	header:Show()

	return header
end

function AUR:InitializeAuras()
	self.BuffFrame = CreatePlayerAuraHeader("HELPFUL")
	E:CreateMover(self.BuffFrame, L["buffs"], "TOPRIGHT")

	self.DebuffFrame = CreatePlayerAuraHeader("HARMFUL")	
	E:CreateMover(self.DebuffFrame, L["debuffs"], "BOTTOMRIGHT")
	
	self.Headers = {self.BuffFrame, self.DebuffFrame}
	
	B:RemovePlayerAuras()
end

function AUR:Init()	
	self.db = CO.db.profile.unitframe.auras
	
	self:InitializeAuras()
	self:LoadConfig()
end

E:AddModule("Auras", AUR)