local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, BC, UF, TT = E:LoadModules("Config", "Castbar", "Unitframes", "Tooltip")

local _
	BC.Bars = {}
	BC.DBColors = {}
	
local pairs							= pairs
local select						= select
local format						= string.format
local GetNetStats					= GetNetStats
local GetTime						= GetTime
local CastingBarFrame_ApplyAlpha	= CastingBarFrame_ApplyAlpha
local INTERRUPTED					= INTERRUPTED

-- string.format usage:
-- string.format("CUI_%sCastbar%s", unit, index)
-- %s can be used an unlimited amount of times. Also do additional args
local BarBaseName 			= "CUI_%sCastbar"
local IconSize 				= 23

function BC:LoadConfig(limit)
	local GlobalConfig, Config = CO.db.profile.unitframe.units.all.castbar
	
	self.DBColors.Success = CO.db.profile.colors.castbar.success
	self.DBColors.Failed =  CO.db.profile.colors.castbar.failed
	self.DBColors.Interruptible =  CO.db.profile.colors.castbar.interruptible
	self.DBColors.NotInterruptible =  CO.db.profile.colors.castbar.notInterruptible
	
	for Name, Bar in pairs(BC.Bars) do
		--if limit then Bar = limit end
		
		Config = CO.db.profile.unitframe.units[Bar:GetParent().ConfigKey].castbar
		
		if not Config.enable then
			Bar:Hide()
			Bar.MoverEnabled = false
			Bar.ForceMoverEnabled = nil
			BC:RemoveEventHandler(Bar)
		else			
			Bar.MoverEnabled = true
			BC:AddEventHandler(Bar)
			
			if Bar.ForceToggle == true then
				Bar:Show()
			end
		
		-- Bar
			Bar:SetSize(Config.width, Config.height)
			Bar.LagBar:SetSize(Config.width, Config.height) -- Set both as we just override one of them			
			
			if Config.attachToUnitframe then
				local BarSmartPosition = E:InversePosition(Config.barPosition)
				local Frame = UF:GetUnitframe(Bar.Owner.RealUnit or Bar.Owner.Unit)
				
				print(Bar.Owner.RealUnit, Bar.Owner.Unit, Frame:GetName())
				Bar:ClearAllPoints()
				Bar:SetParent(UF:GetUnitframe(Bar.Owner.RealUnit or Bar.Owner.Unit))
				Bar:SetPoint(BarSmartPosition, UF:GetUnitframe(Bar.Owner.RealUnit or Bar.Owner.Unit), Config.barPosition, Config.barOffsetX, Config.barOffsetY)
			end
			
			Bar.Overlay:SetReverseFill(Config.barInverseFill)
			Bar.Overlay:SetOrientation(Config.barOrientation)
			
			Bar.Reverse = Config.barInverseFill
			Bar.Vertical = (Config.barOrientation == "VERTICAL")
			
			Bar.ShowCastingTarget = Config.showCastingTarget
			Bar.CastingTarget = nil
			
			Bar.LagBar:ClearAllPoints()
			Bar.Spark:ClearAllPoints()
			
			local NewPosition
			
			if Bar.Vertical then
				
				Bar.LagBar:SetHeight(3)
				
				if Bar.Reverse then NewPosition = "BOTTOM" else NewPosition = "TOP" end
				
				Bar.Spark:SetRotation(1.5708)
				Bar.Spark:SetSize(Config.sparkHeight, Config.sparkWidth) -- Also rotates the axis due to SetRotation
			else
				
				Bar.LagBar:SetWidth(3)
				
				if Bar.Reverse then NewPosition = "LEFT" else NewPosition = "RIGHT" end
				
				Bar.Spark:SetRotation(0)
				Bar.Spark:SetSize(Config.sparkWidth, Config.sparkHeight) -- Also rotates the axis due to SetRotation
			end
			
			Bar.LagBar:SetPoint(NewPosition, Bar.Overlay, NewPosition)
			Bar.Spark:SetPoint("CENTER", Bar.Overlay:GetStatusBarTexture(), NewPosition, 0, 0)
			
			Bar:SetBorderColor(unpack(Config.barBorderColor))
			Bar:SetBorderSize(Config.barBorderSize)
		
		-- Texture
			if not Config.enableIcon then
				Bar.Icon:Hide()
			else
				Bar.Icon:SetSize(Config.iconSize, Config.iconSize)
				
				local SmartPosition = E:InversePosition(Config.iconPosition)
				Bar.Icon:ClearAllPoints()
				Bar.Icon:SetPoint(SmartPosition, Bar, Config.iconPosition, Config.iconOffsetX, Config.iconOffsetY)
				
				Bar.Icon:Show()
			end
			
		-- Flash
			if Bar.Flash then
				Bar.Flash:Size(GlobalConfig.flashSize)
			end
			
		-- Fonts
			-- Those are being handled by the PathFonts system now
			
			
			E:UpdateMoverDimensions(Bar)
		end
		
		--if limit then break end
	end
end

function BC:Toggle(Unit)
	
	if type(self) == "string" then Unit = self end
	
	local Bar = BC.Bars[Unit]
	if not Bar then return end
	
	if Bar:IsVisible() then
		Bar.ForceMoverEnabled = nil
		
		Bar:Hide()
		Bar.ForceToggle = false
	else
		Bar.ForceMoverEnabled = true
		Bar.Icon.Tex:SetTexture(136235)
		Bar:SetMinMaxValues(0, 100)
		Bar:SetValue(50)
		Bar.Name:SetText("Long Long Long Long Long Spell Name")
		Bar.Time:SetText("3.4")
		
		Bar:Show()
		Bar:SetAlpha(1)
		Bar.ForceToggle = true
	end
end

function BC:Get(BarName)
	return self.Bars[BarName]
end

function BC:GetIndex(unit)
	local i = 1
	for k, v in pairs(self.Bars) do
		if v.unit == unit then
			i = i + 1
		end
	end
	
	return i
end

local function Flash_SetSize(self, size)
	if not self then return false end
	
	self:SetPoint('TOPLEFT', self.BarParent, 'TOPLEFT', -size, size)
	self:SetPoint('BOTTOMRIGHT', self.BarParent, 'BOTTOMRIGHT', size, -size)
end

function BC:AddFlash(b)
	b.Flash = b:CreateTexture(nil, 'OVERLAY')
	local Flash = b.Flash
	
	Flash.Size = Flash_SetSize
	Flash.BarParent = b
	
	Flash:Size(6)
	Flash:SetTexture([[Interface\AddOns\CUI\Textures\castingbar\flash]])
	Flash:SetBlendMode('ADD')
	Flash:SetAlpha(0)
end

function BC:AddText(b, n, a, x, y)
	b[n] = b.Overlay:CreateFontString(nil, "ARTWORK")
	local f = b[n]
	
	E:InitializeFontFrame(b[n], "ARTWORK", E.Media:Fetch("font", "FRIZQT__.TTF"), 11, {0.8,0.8,0.8}, 1, {0,0}, "", 0, 0, b.Overlay, "RIGHT", {1,1})
	f:SetFont(E.Media:Fetch("font", "FRIZQT__.TTF"), 11, "")
	f:ClearAllPoints()
	f:SetPoint(a, b, a)
	f:SetJustifyH(a)
	f:SetJustifyV("MIDDLE")
	
	if x and y then E:PushFrame(f, x, y) end
	
	return f
end

function BC:ToggleMovers(state)
	for k, v in pairs(self.Bars) do
		self.Bars[k].ForceMoverEnabled = state
		
		if state == true then
			self.Bars[k]:Show()
			self.Bars[k]:SetAlpha(1)
		else
			self.Bars[k]:Hide()
		end
	end
end

function BC:AddLagBar(o)
	o.LagBar = CreateFrame("Frame", nil, o.Overlay)
	o.LagBar:SetSize(o.Overlay:GetWidth(), o.Overlay:GetHeight()) -- Set both as we just override one of them
	o.LagBar:SetPoint("RIGHT", o.Overlay, "RIGHT")
	o.LagBarTex = o.LagBar:CreateTexture(nil)
	o.LagBarTex:SetAllPoints(o.LagBar)
	o.LagBarTex:SetColorTexture(0.65, 0, 0, 0.5)
	
	self:UpdateLagBar(o, false)
end

function BC:UpdateLagBar(o, s)
	if s == false then
		o.LagBar:Hide()
		return
	else
		o.LagBar:Show()
	end
	
	local min, max = o:GetMinMaxValues()
	-- We always assume the min max values are timings
	-- We use the delta value to determine the required dimensions of the LagBar
	local delta = max - min
	
	local timePerPixel = (o.Vertical and o.Overlay:GetHeight() or o.Overlay:GetWidth()) / delta
	local lagBarSize = (select(4, GetNetStats()) / 1000) * timePerPixel
	
	if not o.Vertical then
		o.LagBar:SetWidth(lagBarSize)
	else
		o.LagBar:SetHeight(lagBarSize)
	end
end

function BC:AddSpark(o)
	o.Spark = o.Overlay:CreateTexture(nil, "OVERLAY")
	o.Spark:SetTexture([[Interface\CastingBar\UI-CastingBar-Spark]])
	o.Spark:SetBlendMode("ADD")
end

function BC:SetSparkPosition(o, isChanneling)
	if not o.Vertical then
		if isChanneling then
			o.sparkPositionX = ((o.value / o.maxValue) * o.Overlay:GetWidth()) + 1
		else
			o.sparkPositionX = ((o.value / o.endTime) * o.Overlay:GetWidth()) - 1
		end
		
		if not o.Reverse then
			if select(3, o.Spark:GetPoint(3)) == "RIGHT" then
				o.Spark:ClearAllPoints()
			end
			o.Spark:SetPoint("CENTER", o, "LEFT", o.sparkPositionX, 0)
		else
			if select(3, o.Spark:GetPoint(3)) == "LEFT" then
				o.Spark:ClearAllPoints()
			end
			o.Spark:SetPoint("CENTER", o, "RIGHT", -o.sparkPositionX, 0)
		end
	else
		if isChanneling then
			o.sparkPositionY = ((o.value / o.maxValue) * o.Overlay:GetHeight()) + 1
		else
			o.sparkPositionY = ((o.value / o.endTime) * o.Overlay:GetHeight()) - 1
		end
		
		if not o.Reverse then
			if select(3, o.Spark:GetPoint(3)) == "TOP" then
				o.Spark:ClearAllPoints()
			end
			o.Spark:SetPoint("CENTER", o, "BOTTOM", 0, o.sparkPositionY)
		else
			if select(3, o.Spark:GetPoint(3)) == "BOTTOM" then
				o.Spark:ClearAllPoints()
			end
			o.Spark:SetPoint("CENTER", o, "TOP", 0, -o.sparkPositionY)
		end
	end
end

function BC:SetInterruptible(bar, notInterruptible)
	if bar.casting or bar.channeling then
		if notInterruptible then
			bar:SetOverlayColor(unpack(BC.DBColors.NotInterruptible))
			E:SkinButtonIcon(bar.Icon, BC.DBColors.NotInterruptible)
		else
			bar:SetOverlayColor(unpack(BC.DBColors.Interruptible))
			E:SkinButtonIcon(bar.Icon, BC.DBColors.Interruptible)
		end
	end
end

local function Castingbar_ResetFlash(self)
	if self.Flash then
		self.Flash:SetAlpha(0.0)
		self.Flash:Show()
	end
end

local function Castingbar_FinishCast(self, state, holdTime)
	
	local Color = (state == true or state == nil) and BC.DBColors.Success or BC.DBColors.Failed
	
	self.Time:SetText('')
	self:SetValue(select(2, self.Overlay:GetMinMaxValues()))
	self:SetOverlayColor(unpack(Color))
	
	Castingbar_ResetFlash(self)	
	if self.Flash then
		self.Flash:SetVertexColor(unpack(Color))
	end
	
	if self.Spark then
		self.Spark:Hide()
	end
	
	self.flash = true
	self.fadeOut = true
	self.casting = nil
	self.channeling = nil
	self.holdTime = holdTime or 0
end

function BC:RemoveEventHandler(bar)
	if bar.active then
		bar:UnregisterAllEvents()
		bar:SetScript("OnEvent", nil)
		bar:SetScript("OnUpdate", nil)
	end
	
	bar.active = nil
end

local function Custom_UnitIsUnit(unit1, unit2)
	if not unit1 then return false end
	if not unit2 then return false end
	if unit1 == 'party1' then unit1 = 'player' end
	if unit2 == 'party1' then unit2 = 'player' end
	
	return UnitIsUnit(unit1, unit2)
end
local function Custom_UnitExists(unit)
	if unit == 'party1' then unit = 'player' end
	
	return UnitExists(unit)
end

local InterruptStr = "%s [%s]"
local function Bar_OnEvent(self, event, ...)
	-- Interruptor handler START
	----------------------------
	if event == "COMBAT_LOG_EVENT_UNFILTERED" then
		if not self:IsVisible() then return end
		-- Probably the most efficient way we can go
		
		_, self.combatLogInfoType, _, _, self.combatLogInfoName, _, _, self.combatLogInfoGUID = CombatLogGetCurrentEventInfo()
		
		if self.combatLogInfoType == "SPELL_INTERRUPT" and self.combatLogInfoGUID == UnitGUID(self.Owner.Unit) then
			-- Check if the interruptor name is valid. Environmental effects like quaking leave this at nil, a.e.
			if self.combatLogInfoName then
				self.Name:SetText(InterruptStr:format(INTERRUPTED, self.combatLogInfoName))
			end
		end
		
		-- End call directly, since we do not want the script to iterate through everything else. This does get fired REALLY rapidly in combat.
		return
	end
	-- Interruptor handler END
	----------------------------
	
	-- print(event)
	
	self.eventUnit = ...
	
	-- Hide when there is no such unit
	if not Custom_UnitExists(self.Owner.Unit) then self:Hide(); return; end
	if not Custom_UnitIsUnit(self.eventUnit, self.Owner.Unit) and event ~= "PLAYER_TARGET_CHANGED" then return end
	
	if (event == "PLAYER_TARGET_CHANGED" or event == "PLAYER_FOCUS_CHANGED") and not (UnitCastingInfo(self.Owner.Unit) or UnitChannelInfo(self.Owner.Unit)) then
		self.casting = nil
		self.channeling = nil
		self.fadeOut = true
		self.holdTime = 0
		
		self:Hide()
	end
	
	if event == "UNIT_SPELLCAST_SENT" and self.ShowCastingTarget then
		
		local target = select(2, ...)
		self.CastingTarget = (target and target ~= "") and target or nil
		self.CastingTargetGUID = select(3, ...)
		
		return
	end
	
	if event == "UNIT_SPELLCAST_START" or ((event == "PLAYER_TARGET_CHANGED" or event == "PLAYER_FOCUS_CHANGED") and UnitCastingInfo(self.Owner.Unit)) then
		self.SpellName, _, self.SpellTexture, self.startTime, self.endTime, self.SpellIsTradeSkill, self.castID, self.notInterruptible  = UnitCastingInfo(self.Owner.Unit)
		if self.SpellName then
			CastingBarFrame_ApplyAlpha(self, 1.0)
			self.holdTime = 0
			self.casting = true
			self.channeling = nil
			self.fadeOut = nil
			
			self.endTime = (self.endTime - self.startTime) / 1000
			self.value = (GetTime() - (self.startTime / 1000))
			self:SetMinMaxValues(0, self.endTime)
			self:SetValue(self.value)
			
			if ( self.Spark ) then
				self.Spark:Show()
			end
			self.Name:SetText((self.CastingTarget and self.CastingTargetGUID == self.castID) and (self.SpellName .. " -> " .. self.CastingTarget) or self.SpellName)
			self.Icon.Tex:SetTexture(self.SpellTexture)
			
			
			BC:SetInterruptible(self, self.notInterruptible)
			
			-- Keep lagbar enabled for non-player units, when the cast is interruptible to help with interrupting it on high latency
			if self.Owner.Unit ~= "player" and self.notInterruptible then BC:UpdateLagBar(self, false) else BC:UpdateLagBar(self, true) end
			
			self:Show()
			self:SetAlpha(1)
			
			-- self.holdTime = GetTime() + CASTING_BAR_HOLD_TIME
		else
			self.casting = nil
			self.channeling = nil
			self.fadeOut = true
			self.holdTime = 0
			
			self:Hide()
		end
		
	
		
	elseif event == "UNIT_SPELLCAST_CHANNEL_STOP" then
		-- If still casting [Fix for passive auras that also trigger this event]
		if not self.channeling or UnitChannelInfo(self.Owner.Unit) then return end
		if not self:IsVisible() then
			self:Hide()
		end
		
		Castingbar_FinishCast(self, true)
		
	elseif event == "UNIT_SPELLCAST_FAILED" or event == "UNIT_SPELLCAST_INTERRUPTED" then
		if ( self:IsShown() and
		     (self.casting and select(2, ...) == self.castID) and not self.fadeOut ) then
			if self.Name then
				if event == "UNIT_SPELLCAST_FAILED" then
					self.Name:SetText(FAILED);
				else
					self.Name:SetText(INTERRUPTED);
				end
			end
			
			Castingbar_FinishCast(self, false, GetTime() + CASTING_BAR_HOLD_TIME)
			
		end
		
	elseif event == "UNIT_SPELLCAST_INTERRUPTIBLE" or event == "UNIT_SPELLCAST_NOT_INTERRUPTIBLE" then
		BC:SetInterruptible(self, event == "UNIT_SPELLCAST_NOT_INTERRUPTIBLE")
	elseif event == "UNIT_SPELLCAST_DELAYED" then
		if ( self:IsShown() ) then
			self.SpellName, _, _, self.startTime, self.endTime, _, _, _  = UnitCastingInfo(self.Owner.Unit)
			if not self.SpellName then
				-- if there is no name, there is no bar
				self:Hide();
				return;
			end
			
			self.flash = nil
			
			self.endTime = (self.endTime - self.startTime) / 1000
			self.value = (GetTime() - (self.startTime / 1000))
			self:SetMinMaxValues(0, self.endTime)
		end
		
	-- Cast succeeded
	elseif event == "UNIT_SPELLCAST_STOP" then
		if not self:IsVisible() then
			self:Hide()
		end
		if ( (self.casting and event == "UNIT_SPELLCAST_STOP" and select(2, ...) == self.castID) or
		     (self.channeling and event == "UNIT_SPELLCAST_CHANNEL_STOP") ) then
			Castingbar_FinishCast(self, true)		
		end
		
	
	elseif event == "UNIT_SPELLCAST_CHANNEL_START" or ((event == "PLAYER_TARGET_CHANGED" or event == "PLAYER_FOCUS_CHANGED") and UnitChannelInfo(self.Owner.Unit)) then
		self.SpellName, _, self.SpellTexture, self.startTime, self.endTime, self.SpellIsTradeSkill, self.notInterruptible = UnitChannelInfo(self.Owner.Unit)
		
		if not self.SpellName then
			-- if there is no name, there is no bar
			self:Hide()
			return
		end
			
		self:Show()
		self:SetAlpha(1)

		self.Name:SetText(self.CastingTarget and (self.SpellName .. " -> " .. self.CastingTarget) or self.SpellName)
		self.Icon.Tex:SetTexture(self.SpellTexture)
		self.maxValue = (self.endTime - self.startTime) / 1000
		self.value = (self.endTime / 1000) - GetTime()
		self.minValue = 0
		self:SetMinMaxValues(0, self.maxValue)
		self:SetValue(self.value)
		self.casting = nil
		self.channeling = true
		
		
		BC:SetInterruptible(self, self.notInterruptible)
		
		if ( self.Spark ) then
			self.Spark:Show()
		end
		
		BC:UpdateLagBar(self, false)
		
		self.holdTime = GetTime() + CASTING_BAR_HOLD_TIME
		
	-- Channel delay
	elseif event == "UNIT_SPELLCAST_CHANNEL_UPDATE" then
		if ( self:IsShown() ) then
			self.SpellName, _, self.SpellTexture, self.startTime, self.endTime, self.SpellIsTradeSkill, _, _ = UnitChannelInfo(self.Owner.Unit)
			if not self.SpellName then
				-- if there is no name, there is no bar
				self:Hide();
				return;
			end

			self.value = ((self.endTime / 1000) - GetTime());
			self.maxValue = (self.endTime - self.startTime) / 1000;
			self:SetMinMaxValues(0, self.maxValue);
			self:SetValue(self.value);
		end
		
	end
end

function Bar_OnShow(self)
	if self.casting then
		local _, _, _, startTime = UnitCastingInfo(self.Owner.Unit);
		if startTime then
			self.value = (GetTime() - (startTime / 1000));
		end
	else
		local _, _, _, _, endTime = UnitChannelInfo(self.Owner.Unit);
		if endTime then
			self.value = ((endTime / 1000) - GetTime());
		end
	end
end

function BC:AddEventHandler(bar)
	
	if bar.active then return end
	
	bar.casting = nil
	bar.channeling = nil
	bar.fadeOut = false
	bar.holdTime = 0
	
	-- Register a bunch (all) of spellcast events (all we need)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_START", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_STOP", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_FAILED", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_INTERRUPTED", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_INTERRUPTIBLE", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_DELAYED", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_START", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_STOP", bar.RealUnit)
	bar:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_UPDATE", bar.RealUnit)
	bar:RegisterEvent("UNIT_SPELLCAST_SENT")
	bar:RegisterEvent("UNIT_TARGET")
	
	if bar.RealUnit == "target" or bar.RealUnit == "targettarget" then
		bar:RegisterEvent("PLAYER_TARGET_CHANGED")
		if bar.RealUnit == "targettarget" then
			bar:RegisterUnitEvent("UNIT_TARGET", "target")
		end
	elseif bar.RealUnit == "focus" or bar.RealUnit == "focustarget" then
		bar:RegisterEvent("PLAYER_FOCUS_CHANGED")
		if bar.RealUnit == "focustarget" then
			bar:RegisterUnitEvent("UNIT_TARGET", "focus")
		end
	end
	
	-- Interruptor
	bar:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	
	bar:SetScript("OnShow", Bar_OnShow)
	bar:SetScript("OnEvent", Bar_OnEvent)
	
	bar.OnUpdate = self.OnUpdate
	bar:SetScript("OnUpdate", bar.OnUpdate)
	
	bar.active = true
end

function BC:OnUpdate(elapsed)
	if self.ForceMoverEnabled then return end
	
	if ( self.casting ) then
		
		self.value = self.value + elapsed;
		if ( self.value >= self.endTime ) then
			self:SetValue(self.endTime);
			Castingbar_FinishCast(self)
			return;
		end
		self:SetValue(self.value)
		self.Time:SetText(E:Round(self.endTime - self.value, 1))
		if ( self.Flash ) then
			self.Flash:Hide();
		end
		
	elseif ( self.channeling ) then
		self.value = self.value - elapsed;
		if ( self.value <= self.minValue ) then
			Castingbar_FinishCast(self)
			self.channeling = nil;
			self.fadeOut = true;
			return;
		end
		
		self:SetValue(self.value);
		self.Time:SetText(E:Round(self.value, 1))
		if ( self.Flash ) then
			self.Flash:Hide();
		end
	elseif ( GetTime() < self.holdTime ) then
		return;
	elseif ( self.flash ) then
		local alpha = 0;
		if ( self.Flash ) then
			alpha = self.Flash:GetAlpha() + CASTING_BAR_FLASH_STEP;
		end
		if ( alpha < 1 ) then
			if ( self.Flash ) then
				self.Flash:SetAlpha(alpha);
			end
		else
			if ( self.Flash ) then
				self.Flash:SetAlpha(1.0);
			end
			self.flash = nil;
		end
	elseif ( self.fadeOut ) then
		local alpha = self:GetAlpha() - 0.015;
		if ( alpha > 0 ) then
			self:SetAlpha(alpha)
		else
			self.fadeOut = nil;
			self:Hide();
		end
	end
end

function BC:Create(F, unit)
	-- Override as party1 technically doesn't exist
	F = F or UF:GetUnitframe(unit)
	
	unit = unit or F.RealUnit
	if self.Bars[unit] then return end
	
	local i = self:GetIndex(unit) -- Get index
	local name = format("CUI_%sCastbar%s", unit, i)
	
	local Bar = E:CreateBar(name, "LOW", 235, 25, {"CENTER", E.Parent, "CENTER"}, F or E.Parent)
	Bar.Owner = F
	E.Libs.LibSmooth:ResetBar(Bar.Overlay) -- Leaving the smooth anim on somehow causes the bar to not go at a 100%. This results in the LagBar simply being useless and just looks weird
	Bar:SetBackgroundColor(nil, nil, nil, 0.95)
	
	Bar.Unit = unit
	Bar.RealUnit = (F.RealUnit or unit) == 'party1' and 'player' or (F.RealUnit or unit)
	Bar.RawUnit, Bar.UnitNum = E:ExtractDigits(unit)
	
	Bar.Icon = CreateFrame("Frame", "CUI_CastbarIconHolder", Bar)
	Bar.Icon:SetSize(IconSize, IconSize)
	Bar.Icon:SetPoint("LEFT", Bar, "LEFT", -IconSize, 0)
	
	Bar.Icon.Tex = Bar.Icon:CreateTexture(nil, "OVERLAY")
	Bar.Icon.Tex:SetTexCoord(0.06, 0.94, 0.06, 0.94)
	Bar.Icon.Tex:SetParent(Bar.Icon)
	Bar.Icon.Tex:SetAllPoints(Bar.Icon)
	
	E:RegisterPathFont(self:AddText(Bar, "Time", "RIGHT", -10, 0), "db.profile.unitframe.units." .. Bar.RawUnit .. ".castbar.fonts.time")
	E:RegisterPathFont(self:AddText(Bar, "Name", "LEFT", 5, 0), "db.profile.unitframe.units." .. Bar.RawUnit .. ".castbar.fonts.name")
	
	-- Lag Bar
	self:AddLagBar(Bar)
	-- Spark
	self:AddSpark(Bar)
	-- Flash
	self:AddFlash(Bar)
	-- Functionality
	self:AddEventHandler(Bar)
	-- Initial Hide
	Bar:Hide()
	-- Add mover
		-- No mover for clustered units !
	if Bar.UnitNum == "" then
		E:CreateMover(Bar, format("%s - %s", L[Bar.RawUnit], L["castbar"]), nil, nil, nil, nil, "unitframes")
	end
	-- Register castbar to lib
	self.Bars[unit] = Bar
	F.Castbar = Bar
end

function BC:UpdateDB()
	self.moverdb = CO.db.profile.movers
end
function BC:Init()
	if not CO.db.char.unitframe.enable then return end
	
	--local BarUnits = {'player','target','targettarget','focus','focustarget','pet','boss1','boss2','boss3','boss4','boss5','party1','party2','party3','party4','party5','arena1','arena2','arena3','arena4','arena5'}
	local BarUnits = {'player','target','targettarget','focus','focustarget','pet','boss1','boss2','boss3','boss4','boss5'}
	
	for k,v in pairs(BarUnits) do
		self:Create(nil, v)
	end
	
	-- Load castbars
	self:LoadConfig()
end

E:AddModule("Castbar", BC)
---------- Add Module
UF.Modules["Castbar"] = BC