local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, UF = E:LoadModules("Config", "Unitframes")

--[[--------------------
	Unitframe Extension	
--------------------]]--

local _
local tinsert 			= table.insert
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitPower 		= UnitPower
local UnitPowerMax 		= UnitPowerMax
local Module = {}

-----------------------------------------

Module.Frames = {}

local function UpdateVisibility(Element, Value)
	if not Element.HideWhenEmpty then return end
	if Value > 0 then
		if not Element.Visible then
			Element:Show()
			Element.Visible = true
		end
	else
		if Element.Visible then
			Element:Hide()
			Element.Visible = nil
		end
	end
end

local function UpdateElement(Element, Event, Unit)
	
	if Element.Disabled then return end
	if not Unit then Unit = Element.Owner.Unit end -- Required for ForceUpdate
	
	if Event == "UNIT_DISPLAYPOWER" or Event == "ForceUpdate" then
		UF:UpdateBarColor(Element, E:GetUnitPowerColor(Unit))
	end
	
	if Event == "UNIT_MAXPOWER" or Event == "UNIT_DISPLAYPOWER" or Event == "UNIT_POWER_BAR_SHOW" or Event == "UNIT_POWER_BAR_HIDE"
	or Event == "ForceUpdate" then
		Element.MaxValue = UnitPowerMax(Unit)
		
		if Element.MaxValue == 0 then
			if Element.Visible then
				Element:Hide()
				Element.Visible = nil
			end
			
			return
		else
			if not Element.Visible then
				Element:Show()
				Element.Visible = true
			end
			
			Element:SetMinMaxValues(0, Element.MaxValue)
		end
	end
	
	if not UnitIsDeadOrGhost(Unit) then
		local Value = UnitPower(Unit)
		Element:SetValue(Value)
		UpdateVisibility(Element, Value)
	else
		Element:SetValue(0)
		UpdateVisibility(Element, 0)
	end
end

local function ForceUpdate(Element)
	UpdateElement(Element, "ForceUpdate", Element.Owner.Unit)
end

local function OnEvent(Element, event, unit)
	if(not unit or Element.Owner.Unit ~= unit) then return end
	UpdateElement(Element, event, unit)
end

local function UpdateAutoSettings(F)
	Module:LoadSizeConfig(F)
end

----------

function Module:LoadSizeConfig(F)
	local Config = self.db.units[F.ConfigKey].power
	local Element = F.Power
	
	local Width, Height
	
	if not Element then return end
	
	if Config.autoWidth then
		Width = self.db.units[F.ConfigKey].health.width
	else
		Width = Config.barWidth
	end
	if Config.autoHeight then
		Height = self.db.units[F.ConfigKey].health.height
	else
		Height = Config.barHeight
	end
	
	Element:SetSize(Width, Height)
end

function Module:LoadConfig(limit)
	local Config, Element
	
	for _, F in pairs(Module.Frames) do
		if limit then F = limit end
		
		Config = self.db.units[F.ConfigKey].power
		Element = F.Power
		
		if Config.enable then
			Element.Disabled = false
			
			UpdateAutoSettings(F)
			Element:SetReverseFill(Config.barInverseFill)
			Element:SetOrientation(Config.barOrientation)
			if Config.barSmooth then
				E.Libs.LibSmooth:SmoothBar(Element)
			else
				E.Libs.LibSmooth:ResetBar(Element)
			end

			Element:ClearAllPoints()
			Element:SetParent(F.Overlay)
			Element:SetPoint(E:InversePosition(Config.barPosition), F.Health, Config.barPosition, Config.barXOffset, Config.barYOffset)
			
			-- Texture
			if Config.overrideBarTexture then
				Element:SetAttribute("ReceivesGlobalTexture", false)
				Element:SetStatusBarTexture(E.Media:Fetch("statusbar", Config.barTexture or self.db.units.all.barTexture))
			else
				Element:SetAttribute("ReceivesGlobalTexture", true)
				Element:SetStatusBarTexture(E.Media:Fetch("statusbar", self.db.units.all.barTexture))
			end
			
			-- Power Border and Background
			Element.Background:SetColorTexture(unpack(Config.barBackgroundColor))
			E:SetFrameBorder(Element.Border, Config.barBorderSize, unpack(Config.barBorderColor))
			
			Element.HideWhenEmpty = Config.hideWhenEmpty
			
			if not F.Eventless then
				if not Element:IsEventRegistered("UNIT_POWER_UPDATE") then
					Element:RegisterEvent("UNIT_POWER_UPDATE")
				end
				
				if Config.fastUpdate then
					Element:RegisterEvent("UNIT_POWER_FREQUENT")
				else
					Element:UnregisterEvent("UNIT_POWER_FREQUENT")
				end
				
				if not Element:IsEventRegistered("UNIT_MAXPOWER") then
					Element:RegisterEvent("UNIT_MAXPOWER")
					Element:RegisterEvent("UNIT_DISPLAYPOWER")
					Element:RegisterEvent("UNIT_POWER_BAR_SHOW")
					Element:RegisterEvent("UNIT_POWER_BAR_HIDE")
				end
			end
			
			Element:SetScript("OnEvent", OnEvent)
			Element:ForceUpdate()
		else
			Element.Disabled = true
			
			Element:SetScript("OnEvent", nil)
			Element:Hide()
			Element.Visible = nil
		end
		
		if limit then return end
	end
end

function Module:Create(F)
	local Element 	= UF:CreateUFBar()
	F.Power 		= Element
	
	Element.Border 		= E:CreateBorder(Element)
	Element.Background 	= E:CreateBackground(Element)
	
	Element.Visible = true
	
	Element.Owner = F
	Element.ForceUpdate = ForceUpdate
	
	hooksecurefunc(F, "SetSize", UpdateAutoSettings)
	
	Element:SetFrameLevel(F.Overlay:GetFrameLevel() + 5)
	
	tinsert(Module.Frames, F)
end

---------- Add Module
UF:RegisterModule('BarPower', Module)