local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, UF = E:LoadModules("Config", "Unitframes")

--[[--------------------
	Unitframe Extension	
--------------------]]--

local _
local unpack		= unpack
local pairs			= pairs
local tinsert		= table.insert
local IsResting		= IsResting
local Module = {}
Module.IncludeUnits = {'player'}

-----------------------------------------

local EventHandler = CreateFrame("Frame")
local Events = {"PLAYER_ENTERING_WORLD", "PLAYER_UPDATE_RESTING"}

local Texture = [[Interface\CharacterFrame\UI-StateIcon]]
local TexCoord = {0, 0.5, 0, 0.421875}

local function UpdateElement(self)
	if self.Disabled then self:Hide() return end
	
	if IsResting() then
		self:Show()
	else
		self:Hide()
	end
end

do
	-- Handles all event updates for this module
	for k, v in pairs(Events) do
		EventHandler:RegisterEvent(v)
	end
	EventHandler.Handles = {}
	EventHandler:SetScript("OnEvent", function(self, event, ...)
		for _, F in pairs(self.Handles) do
			UpdateElement(F.RestingIndicator)
		end
	end)
end

----------

-- Gets called automatically when the unitframes first are initialized and on config update
function Module:LoadConfig()
	local Config
	
	for _, self in pairs(EventHandler.Handles) do
		Config = CO.db.profile.unitframe.units[self.ConfigKey]
		
		if Config.restingIndicator then
			if not Config.restingIndicator.enable then self.RestingIndicator:Hide(); self.RestingIndicator.Disabled = true; else
				self.RestingIndicator:ClearAllPoints()
				self.RestingIndicator:SetPoint("CENTER", self.Overlay, Config.restingIndicator.position, Config.restingIndicator.offsetX, Config.restingIndicator.offsetY)
				self.RestingIndicator:SetSize(Config.restingIndicator.size, Config.restingIndicator.size)
				self.RestingIndicator:SetFrameLevel(self.Overlay:GetFrameLevel() + 25)
				
				self.RestingIndicator.Disabled = false
			end
		else
			self.RestingIndicator.Disabled = true
		end
		
		self.RestingIndicator:ForceUpdate()
	end
end

function Module:Create(F)
	F.RestingIndicator = E:CreateTextureFrame(nil, F, 20, 20, "ARTWORK")
	F.RestingIndicator.T:SetTexture(Texture)
	F.RestingIndicator.T:SetTexCoord(unpack(TexCoord))
	
	F.RestingIndicator.ForceUpdate = UpdateElement
	
	tinsert(EventHandler.Handles, F)
end

---------- Add Module
UF.Modules["RestingIndicator"] = Module