local E, L = unpack(select(2, ...)) -- Engine, Locale
local CO, UF, TT, BC = E:LoadModules("Config", "Unitframes", "Tooltip", "Castbar")

--[[-------------------------------------------------------------------------

	Caching globals for faster access times

-------------------------------------------------------------------------]]--

local _
local format 							= string.format
local tsort								= table.sort
local tinsert							= table.insert
local unpack 							= unpack
local select 							= select
local pairs 							= pairs
local tonumber 							= tonumber
local type 								= type
local CreateFrame 						= CreateFrame
local RegisterUnitWatch 				= RegisterUnitWatch
local UnitExists 						= UnitExists
local UnitClass 						= UnitClass
local UnitName 							= UnitName
local UnitLevel 						= UnitLevel
local UnitIsConnected 					= UnitIsConnected
local UnitHealth 						= UnitHealth
local UnitHealthMax 					= UnitHealthMax
local UnitPower 						= UnitPower
local UnitPowerMax						= UnitPowerMax
local UnitStagger						= UnitStagger
local UnitInParty 						= UnitInParty
local UnitInRaid 						= UnitInRaid
local UnitIsGroupLeader 				= UnitIsGroupLeader
local UnitIsGroupAssistant 				= UnitIsGroupAssistant
local UnitGroupRolesAssigned 			= UnitGroupRolesAssigned
local WarlockPowerBar_UnitPower			= WarlockPowerBar_UnitPower
local GetRuneCooldown					= GetRuneCooldown
local GetSpecialization 				= GetSpecialization
local GetSpecializationInfoForClassID 	= GetSpecializationInfoForClassID
local UnregisterStateDriver 			= UnregisterStateDriver
local RegisterStateDriver 				= RegisterStateDriver
local InCombatLockdown 					= InCombatLockdown
local DEAD 								= DEAD
local FRIENDS_LIST_OFFLINE 				= FRIENDS_LIST_OFFLINE
-----------------------------------------------------------------------------

UF.Modules = {}

UF.UNITFRAMES_RANGE_UPDATE 					= 0.5 -- Range update frequency in seconds
local UseNewGroupSystem = true
UF.UseNewGroupSystem = UseNewGroupSystem


-- Affected = Source
local Targets = {
	targettarget = {"target"},
	focustarget = {"focus"},
}
local PeriodicUnitUpdate = CreateFrame("Frame")

UF.HolderVisibilityOverride = false

UF.ToCreate = {["arena"] = 5,["party"] = 5,["boss"] = 5,["raid"] = 20,["raid40"] = 40, ["maintank"] = 5}

function UF:UnitExists(unit)
	return unit and (UnitExists(unit) or ShowBossFrameWhenUninteractable(unit))
end

function UF:GetUnitSpecs(Unit)
	local classID, specID, name, description, iconID, role, isRecommended, isAllowed
	classID 	= select(3,UnitClass(Unit))
	local data 	= {}

	for i=1,GetNumSpecializationsForClassID(classID) do
		specID, name, description, iconID, role, isRecommended, isAllowed = GetSpecializationInfoForClassID(classID, i)
		data[i] = {specID, name, description, iconID, role, isRecommended, isAllowed}
	end

	return data
end

function UF:UpdateBarColor(Bar, RGBA, r, g, b, a)
	if not RGBA and not r and not g and not b and not a then return end
	local BarTexture = Bar:GetStatusBarTexture()
	
	if RGBA then
		BarTexture:SetVertexColor(RGBA[1], RGBA[2], RGBA[3], RGBA[4] or select(4, BarTexture:GetVertexColor()) or 1)
		Bar.RGBA = RGBA
	else
		BarTexture:SetVertexColor(r, g, b, a or select(4, BarTexture:GetVertexColor()) or 1)
		Bar.r = r; Bar.g = g; Bar.b = b; Bar.a = a or select(4, BarTexture:GetVertexColor()) or 1
	end
end

local function UnitFrame_OnLeave(self)
	GameTooltip:FadeOut()
end

function UF:SetHoverScript(Frame, State)
	-- For blizz functionality (Blizz does not use unit with an capital U)
	Frame.unit = Frame.Unit
	Frame.hideStatusOnTooltip = nil
	
	if State == true then
		Frame:SetScript('OnEnter', UnitFrame_OnEnter)
		Frame:SetScript('OnLeave', UnitFrame_OnLeave)
		
		if not InCombatLockdown() or not Frame:IsProtected() then
			Frame:EnableMouse(true)
		end
	else
		Frame:SetScript('OnEnter', nil)
		Frame:SetScript('OnLeave', nil)
		
		if not InCombatLockdown() or not Frame:IsProtected() then
			Frame:EnableMouse(false)
		end
	end
end

-------------------------------------------------------------------------------------------------------------------------------------
-- Refactor prototype
-------------------------------------------------------------------------------------------------------------------------------------

	----------------------------------------------------------
	-- Profile handler
	----------------------------------------------------------
	function UF:ApplyFontStrings(self)
		for k,v in pairs(self.Fonts) do
			if v.Format then
				E:RegisterTagFont(v, v.Format, self.Unit)
			end
		end
	end
	local function ApplyFontConfig(self, Config)
		Config = Config or UF.db.units[self.ConfigKey]
		
		local CurrentFont, CurrentFontProfile
		for k,v in pairs(self.Fonts) do
			CurrentFontProfile = Config.fonts[E:stringToLower(k)]
			if CurrentFontProfile then
				v.Enable = CurrentFontProfile.enable
				
				if v.Enable == false then
					v:Hide()
				else
					v:Show()
					
					-- Font Shadow
						if CurrentFontProfile.fontShadowColor then
							v:SetShadowColor(CurrentFontProfile.fontShadowColor[1], CurrentFontProfile.fontShadowColor[2], CurrentFontProfile.fontShadowColor[3], CurrentFontProfile.fontShadowColor[4] or 1)
							v:SetShadowOffset(CurrentFontProfile.xFontShadowOffset or 1, CurrentFontProfile.yFontShadowOffset or 1)
						end
					-- Alignment
						if CurrentFontProfile.horizontalAlign then
							v:SetJustifyH(CurrentFontProfile.horizontalAlign)
						end
						if CurrentFontProfile.verticalAlign then
							v:SetJustifyV(CurrentFontProfile.verticalAlign)
						end
					-- Repositioning
						v:ClearAllPoints()
						v:SetPoint(CurrentFontProfile.position)
						E:MoveFrame(v, CurrentFontProfile.xOffset, CurrentFontProfile.yOffset)
					
					-- Level hide
						if CurrentFontProfile.doNotShowOnMaxLevel ~= nil then
							v.ShowAtMax = CurrentFontProfile.doNotShowOnMaxLevel
						end
					-- Font Color
						if CurrentFontProfile.fontColor then
							v:SetTextColor(CurrentFontProfile.fontColor[1], CurrentFontProfile.fontColor[2], CurrentFontProfile.fontColor[3], CurrentFontProfile.fontColor[4] or 1)
						end
					-- Width
						if CurrentFontProfile.width then
							v:SetWidth(CurrentFontProfile.width)
						end
						if CurrentFontProfile.height then
							v:SetHeight(CurrentFontProfile.height)
						end
						
					
					-- Flags
						if CurrentFontProfile.fontFlags == "None" then v.Flags = E.TBL.EMPTY else v.Flags = CurrentFontProfile.fontFlags end
						
					-- 
						-- (Frame, fontName, fontFlags, fontHeight, fontColor)
						E:SetFontInfo(v, E.Media:Fetch("font", CurrentFontProfile.fontType), v.Flags, CurrentFontProfile.fontHeight, nil)
						E:UpdateFont(v)
						
					-- Text Format
						if CurrentFontProfile.textFormat then
							v.Format = CurrentFontProfile.textFormat
						end
				end
			end
		end
		
		UF:ApplyFontStrings(self)
		self:UpdateFonts()
	end
	
	local function ApplyUFConfig(self, limit, arg2)
		-- Needed when tthis is called through PerformForUnits
		if arg2 then limit = arg2 end
		
		if not self then return end
		local Config = UF.db.units[self.ConfigKey]
		
		limit = limit or "all"
		if limit == "fonts" or limit == "all" then
			ApplyFontConfig(self, Config)
			-- Limit update to this module
			if limit == "fonts" then return end
		end
		
		if not Config.enable then
			-- @TODO: If is NOT clustered
			if not UF.ToCreate[self.ConfigKey] then
				self.ForceMoverEnabled = false
			end
		
			if not InCombatLockdown() then
				UnregisterUnitWatch(self)
				self:Hide()
			end
			
			return
		else
			if not UF.ToCreate[self.ConfigKey] then
				self.ForceMoverEnabled = nil
			end
			
			-- All we had to do to fully fix grouped toggles was to check for forceShow here...
			if not InCombatLockdown() and not self.isForcedShown then
				RegisterUnitWatch(self)
			end
		end

		-- Absorb
			-- if self.Health.Absorb then
				-- if Config.health.enableAbsorb then
					-- -- As absorb is basically a statusbar, we can actually handle it like the healthbar					
					-- self.Health:SetSubBar(self.Health.Absorb, true, not Config.health.barInverseFill, Config.health.barOrientation)
					
					-- -- Let user toggle between stripes or full cover
					-- if Config.health.absorbUseStripes then
						-- self.Health.Absorb.Border.Background:SetTexture(E.Media:Fetch("statusbar", "CUI Absorb Stripes"), "REPEAT", "REPEAT")
					-- else
						-- self.Health.Absorb.Border.Background:SetTexture([[Interface\AddOns\CUI\Textures\borders\WHITE8X8]])
					-- end
					-- self.Health.Absorb.Border:SetBackdropBorderColor(Config.health.absorbBorderColor[1], Config.health.absorbBorderColor[2], Config.health.absorbBorderColor[3], Config.health.absorbBorderColor[4])
					-- self.Health.Absorb.Border.Background:SetVertexColor(Config.health.absorbTextureColor[1], Config.health.absorbTextureColor[2], Config.health.absorbTextureColor[3], Config.health.absorbTextureColor[4])
					
					-- self.Health.Absorb.TextureSizeMultiplier = Config.health.absorbTextureSizeMultiplier or 7
					
					-- UF:InitAbsorbEvents(self.Health.Absorb)
					-- self.Health.Absorb:Update()
				-- else
					-- UF:RemoveAbsorbEvents(self.Health.Absorb)
					-- self.Health.Absorb:Hide()
				-- end
			-- end
			
			if self.Unit == "player" or self.Unit == "target" then
				Config.power.fastUpdate = true
			end
		
		-- Player specific
			if self.ConfigKey == "player" then
				E:LoadModule("Alternate_Power"):LoadConfig()
			end
		
		-- Override range update frequency
			self.RangeUpdateFrequency = 0.25
			
			self.enableRangeIndicator = Config.rangeIndicator
			
		-- Override range indicator
			if not Config.rangeIndicator then
				UF:RemoveRangeIndicator(self)
			else
				UF:AddRangeIndicator(self)
			end
		
		-- Update Mover
			E:UpdateMoverDimensions(self)
	end
	UF.ApplyUFConfig = ApplyUFConfig

	function UF:LoadConfig(limit, onlyInitialize)
		local WriteLoaded
		if limit and type(limit) == 'table' and not limit.LoadedConfigs then
			limit.LoadedConfigs = {}
			WriteLoaded = true
		end
		
		-- Load individual unitframe settings
		if type(limit) == 'table' or not limit then
			for ModuleName, Module in pairs(self.Modules) do
				if ModuleName ~= "Auras" and Module.LoadConfig and (((limit and onlyInitialize and not limit.LoadedConfigs[ModuleName]) or not onlyInitialize) or limit) then
					Module:LoadConfig(limit)
					
					if type(limit) == 'table' then
						limit.LoadedConfigs[ModuleName] = true
					end
				end
			end
		end
		for _, frames in pairs(UF.Frames) do
			for _, frame in pairs(frames) do
				if frame.UpdateConfig then
					-- Somehow, this passes frames that are nil??
					frame:UpdateConfig(limit)
				end
			end
		end
		
		if (limit and (onlyInitialize and not limit.LoadedConfigs['Auras']) or not onlyInitialize) or not limit then
			self.Modules['Auras']:LoadConfig(limit)
		end
	end

	function UF:HasFrameKeyHeader(Key)
		if self.RegisteredHeaderClusters[Key] then
			return true
		end
	end
	function UF:GetUFMover(type)
		if type == "raid" or type == "raid40" or type == "party" or type == "boss" or type == "arena" or type == "maintank" then
			return E:GetMover(self:GetHolder(type))
		else
			return E:GetMover(self.Frames[type][1])
		end
	end
	
	function UF:IsUnitGrouped(Unit)
		if not Unit then return end
		
		for Compare, _ in pairs(UF.ToCreate) do
			if Unit:find(Compare) then
				return true
			end
		end
	end
	
	function UF:GetUnitframe(Unit)
		if self.Frames[Unit] then
			return self.Frames[Unit][Index or 1]
		end
		
		-- Scan frame metatable for this unit
		for ConfigKey, Frames in pairs(self.Frames) do
			for Index, Frame in pairs(Frames) do
				if (ConfigKey .. Index) == Unit then
					return Frame
				end
			end
		end
	end
	
	-- To iterate a function over every unitframe of a specified unit type
	function UF:PerformForUnits(Unit, Function, ...)
		if self.Frames[Unit] then
			local NumFrames = #self.Frames[Unit]
			for Index, Frame in pairs(self.Frames[Unit]) do
				Function(Unit .. (NumFrames > 1 and Index or ""), Frame, ...)
			end
		else
			Function(Unit, self:GetUnitframe(Unit), ...)
		end
			
		-- if Unit == 'boss' then
			-- for i = 1, 5 do
				-- Function(Unit .. i)
			-- end
		-- elseif Unit ~= 'player' and Unit ~= 'target' and Unit ~= 'pet' and Unit ~= 'targettarget' and Unit ~= 'focus' and Unit ~= 'focustarget' then
			-- Function(Unit)
			
			-- return 
		-- end
		
		-- if self.ToCreate[Unit] then
			-- for i = 1, self.ToCreate[Unit] do
				-- Function(Unit .. i)
			-- end
		-- else
			-- Function(Unit)
		-- end
	end
	
	local function LoadConfigForSingle(Unit, ...)
		local Frame = UF:GetUnitframe(Unit)
		if not Frame then return end
		
		if Frame.UpdateConfig then
			Frame.UpdateConfig(...)
		end
	end
	function UF:LoadProfileForUnits(Unit, Limit)
		UF:PerformForUnits(Unit, LoadConfigForSingle, Limit)
	end
	
	----------------------------------------------------------
	-- Update handlers
	----------------------------------------------------------
		
		local function NameFont_PostUpdate(self)
			self:SetTextColor(unpack(E:GetUnitReactionColor(self.Owner.Unit, false)))
		end
		
		local function HealthFont_PostUpdate(self)
			if not UnitIsDeadOrGhost(self.Owner.Unit) then
				if not UnitIsConnected(self.Owner.Unit) then
					self:SetText(FRIENDS_LIST_OFFLINE)
				end
			else
				self:SetText(DEAD)
			end
		end
		
		local function PowerFont_OnEvent(self, event, unit)
			if not event or (event and event == "UNIT_DISPLAYPOWER") then
				self:SetTextColor(unpack(E:GetUnitPowerColor((self.Owner.Unit or unit))))
			end
		end
		
		local function PowerFont_PostUpdate(self)
			if not (UnitPowerMax(self.Owner.Unit) > 0) then
				self:SetText("")
			end
		end
		
		local function LevelFont_PostUpdate(self)
			self.Level = UnitLevel(self.Owner.Unit)
			
			if self.ShowAtMax == true and self.Level == E.UNIT_MAXLEVEL then
				self:SetText(E.STR.EMPTY)
			else
				if self.Level <= -1 then
					--self:SetText(E:ParseString(self.Format or "[level]", self.Unit))
					self:SetText(E.STR.Boss)
				end
			end
		end

		local function Fonts_Update(F)
			-- Fix for Bug that appeared first on 8.2 PTR
			if not UF:UnitExists(F.Unit) then return end
			
			for k,v in pairs(F.Fonts) do
				if v.ForceUpdate then
					v:ForceUpdate()
				elseif v.Update and k ~= "Index" then
					v:Update()
				end
				if v.OnEvent then
					v:OnEvent()
				end
			end
		end

		-- This gets called by the OnEvent handler, which basically fires whenever a frame shows up and is missing data.
		-- The OnUpdate handler handles the periodic update calls for units we do not receive any events for. (targettarget, focustarget etc.)
		-- Every other update is performed by the individual modules
		local function UF_Update(self, event, unit, ...)
			-- Instead of "UnitExists". Bugfix for "No Chambers displayed at Mother (Uldir)"
			if not UF:UnitExists(self.Unit) then return end
				
				-- Base modules that definetely exist
				self.Health:ForceUpdate()
				self.Health.Absorb:ForceUpdate()
				self.Health.HealPrediction:ForceUpdate()
				self.Power:ForceUpdate()
				if self.AltPower then
					self.AltPower:ForceUpdate()
				end
				Fonts_Update(self)
				
				-- Modules we dont want to include in the OnUpdate ticks, as the internal events work just fine for them
				if not event or (event and (event ~= "OnUpdate" and event ~= "UNIT_FACTION")) then
					if self.Portrait then
						self.Portrait:ForceUpdate()
					end
					if self.Auras then
						self.Auras:ForceUpdate()
					end
					if self.RangeIndicator then
						self.RangeIndicator:ForceUpdate()
					end
					if self.LeaderIcon then
						self.LeaderIcon:ForceUpdate()
					end
					if self.Role then
						self.Role:ForceUpdate()
					end
					if self.TargetIcon then
						self.TargetIcon:ForceUpdate()
					end
					if self.TargetHighlight then
						self.TargetHighlight:ForceUpdate()
					end
					
					-- Optional modules that probably exist
					if self.ResurrectIndicator then
						self.ResurrectIndicator:ForceUpdate()
					end
					if self.SummonIndicator then
						self.SummonIndicator:ForceUpdate()
					end
					if self.Threat then
						self.Threat:ForceUpdate()
					end
				end
				
				if UnitIsConnected(self.Unit) then
					UF:AddRangeIndicator(self)
				else
					UF:RemoveRangeIndicator(self)
					self:SetAlpha(0.5)
					UF:UpdateBarColor(self.Health, nil, 0.5, 0.5, 0.5)
				end
		end

		function UF:UpdateAllUF()
			for k, frames in pairs(self.Frames) do
				for _, frame in pairs(frames) do
					-- Header Unitframes don't have this
					if frame.ForceUpdate then
						frame:ForceUpdate()
					end
				end
			end
			
			UF.Modules['AlternatePower']:LoadConfig()
			E:LoadModule("Alternate_Power"):LoadConfig()
		end
		
		function UF:UpdateGroup(group)
			for k, frame in pairs(self.Frames) do
				if frame.ConfigKey == group then
					frame:ForceUpdate()
				end
			end
		end
		
		local function UpdateUnit(self)
			local modUnit, unit = SecureButton_GetModifiedUnit(self.UnitButton), SecureButton_GetUnit(self.UnitButton)
			
			if(unit == 'playerpet') then
				unit = 'pet'
			elseif(unit == 'playertarget') then
				unit = 'target'
			end

			if(modUnit == 'pet' and unit ~= 'pet') then
				modUnit = 'vehicle'
			end
			
			if(not UF:UnitExists(modUnit)) then return end
			
			self.UnitButton.Unit 	= modUnit
			self.UnitButton.unit 	= modUnit
			self.Unit				= modUnit
			
			if self.Castbar then
				if unit == 'player' then
					BC:UpdateBarUnit(self.Castbar, modUnit)
				end
			end
			
			-- Update Fonts
			for _, font in pairs(self.Fonts) do
				if font.UpdateUnit then
					font:UpdateUnit(modUnit)
				end
			end
			
			return true
		end

	----------------------------------------------------------
	-- OnEvent handler
	----------------------------------------------------------
		
		local function UnitFrame_ForceUpdate(self)
			UpdateUnit(self)
			UF_Update(self, "ForceUpdate")
		end
		
		local function UnitFrame_OnPetStateChanged(self, unit)
			unit = unit or self.Unit
			
			local petUnit
			if(unit == 'target') then
				return
			elseif(unit == 'player') then
				petUnit = 'pet'
			else
				-- Convert raid26 -> raidpet26
				petUnit = unit:gsub('^(%a+)(%d+)', '%1pet%2')
			end

			if(self.unit ~= petUnit) then return end
			
			if UpdateUnit(self) then
				return true
			end
		end
		
		local function UnitFrame_OnEvent(self, event, unit, ...)
			-- We cannot check the event unit like this, as UnitIsUnit returns false for (player, vehicle) when exiting a vehicle (obviously)
			--if unit and not UnitIsUnit(unit, self.Unit) then return end
			
			if event == 'UNIT_PET' then
				if not UnitFrame_OnPetStateChanged(self, unit) then
					return
				end
			end
			
			if (event == 'UNIT_ENTERED_VEHICLE' or event == 'UNIT_EXITED_VEHICLE') then
				if not UpdateUnit(self) then
					return
				end
			end
			
			UF_Update(self, event, unit, ...)
		end
		
		local function UnitFrame_OnShow(self)
			if self.RangeIndicator and self.enableRangeIndicator then
				UF:AddRangeIndicator(self)
			end
			UnitFrame_ForceUpdate(self)
		end
		local function UnitFrame_OnHide(self)
			if self.RangeIndicator then
				UF:RemoveRangeIndicator(self)
			end
		end

	----------------------------------------------------------
	-- Methods to create UF modules
	----------------------------------------------------------
		function UF:CreateUFBar(F, Name, HasBorder)
			local B
			if not HasBorder then
				B = CreateFrame("Statusbar", Name or nil)
			else
				B = E:CreateBackdropFrame("Statusbar", Name or nil)
			end
			if F then
				B:SetAllPoints(F)
			else
				B:SetSize(150, 15)
			end
			
			B:SetParent(F or E.Parent)

			B:SetMinMaxValues(0, 100)
			B:SetValue(50)
			B:SetStatusBarTexture(E.Media:Fetch("statusbar", self.db.units.all.barTexture))

			E:RegisterStatusBar(B)

			return B
		end
		
		local function Font_OnEvent(self, event, ...)
			if self.Parent.OnEvent then
				self.Parent:OnEvent(event, ...)
			end
		end

		function UF:CreateFonts(F)

			local Fonts, FontName = {"Health", "Power", "Level", "Name", "Index"}, nil
			
			if not F.Fonts then F.Fonts = {} end
			
			for k,v in pairs(Fonts) do
				-- We have to define this here, since Lua only uses table referencing
				local Font = {}
				FontName = format("%s%sFont", F.Unit, v)
				
				Font = F.Overlay:CreateFontString(nil)
					E:InitializeFontFrame(Font, "OVERLAY", "FRIZQT__.TTF", 17, {0.933, 0.886, 0.125}, 0.9, {0,0}, "", 0, 0, F.Overlay, "CENTER", {1,1})
					Font.Unit = F.Unit
				Font.E = CreateFrame("Frame")
				Font.E.Parent = Font
				Font.Owner = F

				F.Fonts[v] = Font

				if v == "Health" then
					E:RegisterTagFontPostUpdate(Font, HealthFont_PostUpdate)
				end
				if v == "Power" then
					--Font.E:RegisterUnitEvent("UNIT_POWER_UPDATE", F.Unit)
					--Font.E:RegisterUnitEvent("UNIT_POWER_FREQUENT", F.Unit)
					Font.E:RegisterUnitEvent("UNIT_DISPLAYPOWER", F.Unit)
					
					Font.OnEvent = PowerFont_OnEvent
					E:RegisterTagFontPostUpdate(Font, PowerFont_PostUpdate)
				end
				if v == "Level" then
					E:RegisterTagFontPostUpdate(Font, LevelFont_PostUpdate)
				end
				if v == "Name" then
					E:RegisterTagFontPostUpdate(Font, NameFont_PostUpdate)
				end
				if v == "Index" then
					Font.Parent = CreateFrame("Frame", nil, F.Overlay)
					Font.Parent:SetAllPoints(F)
					Font.Parent:SetFrameLevel(999)
					
					Font:SetText(F.IndexInGroup or select(2, E:ExtractDigits(F.Unit)))
					Font:SetPoint("CENTER", Font.Parent, "CENTER")
					Font:SetParent(Font.Parent)
					Font.Update = function(self, NewNum) self:SetText(NewNum) end

					Font:Hide()
				end

				Font.E:SetScript("OnEvent", Font_OnEvent)
				if Font.OnEvent then Font:OnEvent() end
			end
			
			F.UpdateFontConfig 	= ApplyFontConfig
			F.UpdateFonts		= Fonts_Update
		end
		
		function UF:LoadAllUnitframeModules(self)
			if not self.Health then return end
			
			-- Base modules that definetely exist
			self.Health:ForceUpdate()
			self.Health.HealPrediction:ForceUpdate()
			self.Power:ForceUpdate()
			if self.AltPower then
				self.AltPower:ForceUpdate()
			end
			if self.Portrait then
				self.Portrait:ForceUpdate()
			end
			if self.Auras then
				self.Auras:ForceUpdate()
			end
			if self.RangeIndicator then
				self.RangeIndicator:ForceUpdate()
			end
			if self.LeaderIcon then
				self.LeaderIcon:ForceUpdate()
			end
			if self.Role then
				self.Role:ForceUpdate()
			end
			if self.TargetIcon then
				self.TargetIcon:ForceUpdate()
			end
			if self.TargetHighlight then
				self.TargetHighlight:ForceUpdate()
			end
			
			-- Optional modules that probably exist
			if self.ResurrectIndicator then
				self.ResurrectIndicator:ForceUpdate()
			end
			if self.SummonIndicator then
				self.SummonIndicator:ForceUpdate()
			end
			if self.Threat then
				self.Threat:ForceUpdate()
			end
			
			-- @TODO: Pack into UF module
			if self.Health.Absorb then
				self.Health.Absorb:ForceUpdate()
			end
			
			if UnitIsConnected(self.Unit) then
				UF:AddRangeIndicator(self)
			else
				UF:RemoveRangeIndicator(self)
				self:SetAlpha(0.5)
				UF:UpdateBarColor(self.Health, nil, 0.5, 0.5, 0.5)
			end
			
			UF:ApplyFontStrings(self)
			self:UpdateFonts()
		end
		
		function UF:HasUnitframeModule(Unitframe, ModuleName)
			if not Unitframe.LoadedModules then Unitframe.LoadedModules = {} return false end
			return E:tableContainsValue(Unitframe.LoadedModules, ModuleName)
		end
		
		function UF:RegisterModule(Name, Module)
			
			if (Module.IncludeUnits and type(Module.IncludeUnits) ~= 'table') or (Module.ExcludeUnits and type(Module.ExcludeUnits) ~= 'table') then
				error(("Module %s has a variable 'IncludeUnits' or 'ExcludeUnits' of the wrong data type. It MUST be a table!"):format(Name))
				return
			end
			if (Module.IncludeUnits and Module.ExcludeUnits) then
				error(("Module %s has a List of compatible units AND an exclusion list. Only one of them is allowed!"):format(Name))
				return
			end
			
			self.Modules[Name] = Module
			Module.db = self.db
		end
		
		function UF:AddModule(Object, ModuleName)
			local Module = self.Modules[ModuleName]
			if not Module then error(("Module %s does not exist!"):format(ModuleName)); return; end
			
			if not Object.LoadedModules then Object.LoadedModules = {} end
			
			local IsAllowed = false
			
			-------------------------
			
			-- Only allow specific units to own this module
			-- tables 'IncludeUnits' and 'ExcludeUnits' consist of config key values like 'player' or 'raid40' without any individual suffix
			-- If not specified every unit is allowed
			
			if Module.IncludeUnits then
				if E:tableContainsValue(Module.IncludeUnits, Object.ConfigKey) then
					IsAllowed = true
				end
			elseif Module.ExcludeUnits then
				if not E:tableContainsValue(Module.ExcludeUnits, Object.ConfigKey) then
					IsAllowed = true
				end
			end
			if not Module.IncludeUnits and not Module.ExcludeUnits then
				IsAllowed = true
			end
			
			-------------------------
			
			-- Check if this module requires any dependencies
			
			if Module.Dependencies then
				for _, Dependency in pairs(Module.Dependencies) do
					if not self:HasUnitframeModule(Object, Dependency) then
						error(("ERROR: Unitframe Module %s could not have been added to %s, as it's missing the dependency module %s!"):format(ModuleName, Object.Unit, Dependency))
						IsAllowed = false
					end
				end
			end
			
			-------------------------
			
			if IsAllowed then
				Module:Create(Object)
				tinsert(Object.LoadedModules, ModuleName)
			end
		end
		
		-- Return the actual DB key for the specified unit
		function UF:GetConfigKey(Unit, RawUnit, Index)
			RawUnit = RawUnit or E:ExtractDigits(Unit)

			-- Add Unitframe to register
			if Index then
				return RawUnit .. Index
			else
				return RawUnit
			end
		end
		
		-- Returns the unitframe module attachment frame/module and checks if there are any issues
		function UF:GetModuleAttachmentFrame(Module)
			if not assert(Module.Owner, "Module " .. Module:GetName() .. " has no specified owner!") then return E.Parent end
			
			local OtherModule
			if Module.AttachTo then
				-- Attached module exists within the unitframe
				OtherModule = Module.Owner[Module.AttachTo]
				if OtherModule then
					if not OtherModule.AttachTo or (OtherModule.AttachTo and OtherModule.AttachTo ~= Module.AttachTo) then
						return OtherModule
					elseif OtherModule.AttachTo and OtherModule.AttachTo ~= Module.AttachTo then
						E:print("The attachment point of " .. Module:GetName() .. " is creating a parenting loop with " .. OtherModule:GetName() .. "!")
					end
				end
			end
			
			return Module.Owner
		end
		
		function UF:RegisterUFUpdateFunction(Frame)
			Frame.UpdateConfig = ApplyUFConfig
		end
		
		function UF:RegisterUFForClique(Frame)
			if not Frame.RegisterForClicks then return end
			if not _G.ClickCastFrames then
				_G.ClickCastFrames = ClickCastFrames or {}
			end
			
			ClickCastFrames[Frame] = true
		end

	----------------------------------------------------------
	-- Method to create a UnitFrame
	----------------------------------------------------------
	UF.Frames = {}
	UF.RegisteredConfigKeys = {}
	function UF:CreateUF(Unit, Index, Header, UnitButton)
		local RawUnit, UnitNum = E:ExtractDigits(Unit)
		local FrameName = format("CUI_%s", Unit)
		local OverlayFrameName
		local F = CreateFrame("Frame", FrameName, UnitButton or E.Parent)
		F.Fonts = {}
		
		F.ConfigKey = self:GetConfigKey(nil, RawUnit, Index)
		if not E:tableContainsValue(self.RegisteredConfigKeys, F.ConfigKey) then
			tinsert(self.RegisteredConfigKeys, F.ConfigKey)
		end

		-- Add Unitframe to register
		if Index then
			OverlayFrameName = RawUnit .. Index .. "_" .. UnitNum
		else
			OverlayFrameName = Unit
		end
		
		if not self.Frames[F.ConfigKey] then
			self.Frames[F.ConfigKey] = {}
		end
		tinsert(self.Frames[F.ConfigKey], F)		
		
		F.Overlay = CreateFrame("Frame", format("CUI_UF_%s_Overlay", E:firstToUpper(OverlayFrameName)), F)
		F.Overlay:SetFrameLevel(F.Overlay:GetFrameLevel() + 10)
		F.Overlay:SetAllPoints(F)
		
	
		if not Header then
		-- UF Overlay to interact with, since the highlight does not work on the Base frame
			F.UnitButton = CreateFrame("Button", format("CUI_UF_%s", E:firstToUpper(OverlayFrameName)), F.Overlay, "SecureUnitButtonTemplate")
			F.UnitButton:SetAllPoints(F.Overlay)
			F.UnitButton:EnableMouse(true)
			F:SetIgnoreParentAlpha(true)
			
			E:HandleFrameInPetBattles(F)
		else
			if not UnitButton then
				F.UnitButton = CreateFrame("Frame", format("CUI_UF_%s", E:firstToUpper(OverlayFrameName)), Header)
				F.UnitButton:SetAllPoints(Header)
			else
				F.UnitButton = UnitButton
			end
		end
		
		-- For 'unittarget' units, we have to use an onupdate handler, since we don't get any events for them
		if Unit:match('%w+target') then
			F.Eventless = true
			tinsert(PeriodicUnitUpdate, F)
		end
		
		F.UnitButton:SetFrameLevel(10)
		F.Overlay:SetFrameLevel(10)

		-- Set unit
			F.Unit = Unit
			F.RawUnit = RawUnit
			F.UnitNum = tonumber(UnitNum)
			F.UnitButton.Unit = Unit
			F.UnitButton.Owner = F
			F.Overlay.Unit = Unit
			F.BackupUnit = Unit -- In case the dummy mode is enabled, we have to use this one

		-- Add UF Modules
			
			self:AddModule(F, 'BarHealth')			-- F.Health
			self:AddModule(F, 'HealthAbsorb')		-- F.Absorb
			self:AddModule(F, 'HealPrediction')		-- F.Health.HealPrediction
			self:AddModule(F, 'BarPower')			-- F.Power
			self:AddModule(F, 'Portrait')			-- F.Portrait
			self:AddModule(F, 'Auras')				-- F.Auras
			self:AddModule(F, 'RoleIndicator')		-- F.Role
			self:AddModule(F, 'Highlight')			-- F.Highlight
			self:AddModule(F, 'TargetHighlight')	-- F.TargetHighlight
			self:AddModule(F, 'ResurrectIndicator')	-- F.ResurrectIndicator
			self:AddModule(F, 'LeaderIcon')			-- F.LeaderIcon
			self:AddModule(F, 'TargetIcon')			-- F.TargetIcon
			self:AddModule(F, 'Threat')				-- F.Threat
			self:AddModule(F, "SummonIndicator")	-- F.SummonIndicator
			self:AddModule(F, "ReadyCheckIndicator")-- F.ReadyCheckIndicator
			self:AddModule(F, "AlternatePower")		-- F.AltPower
			self:AddModule(F, "RestingIndicator")	-- F.RestingIndicator
			self:AddModule(F, "CombatIndicator")	-- F.CombatIndicator
			
			BC:Create(F, true)
			
			--self:AddHealthAbsorb(F.Health)
			
		-- Add UF Fonts
			self:CreateFonts(F)

		-- Set Required attributes
			F:SetAttribute("unit", Unit)
			if not Header then
				F.UnitButton:SetAttribute("unit", Unit)
			end

		-- Set hover script
			self:SetHoverScript(F.UnitButton, true)

		-- Set interaction attributes
			if not Header or UnitButton then
				F.UnitButton:RegisterForClicks("AnyUp")
			end
			if not Header then
				F.UnitButton:SetAttribute("type1", "target")
				F.UnitButton:SetAttribute("*type2", "togglemenu")
				F.UnitButton:SetAttribute("shift-type1", "focus")
					if Unit == "focus" then F.UnitButton:SetAttribute("shift-type1", "macro"); F.UnitButton:SetAttribute("macrotext", "/focus none"); end
			end

			-- Register Frame to the engine so it will take care of its visibility
				RegisterUnitWatch(F)

			-- Register a bunch of events to handle target changing etc.
			-- Since we register just the neccessary event(s) to each frame, we do not have to care
			-- about which event fired in the OnEvent handler.
			-- Wohoo!

				-- When the unit turns hostile/friendly or whatever
				F:RegisterUnitEvent("UNIT_FACTION", Unit)
				F:RegisterUnitEvent("UNIT_FLAGS", Unit)
				
				-- Events that handle situations in which we have to update the unitframe
				if Unit == "target" or Unit == "targettarget" then
					F:RegisterEvent("PLAYER_TARGET_CHANGED")
				elseif Unit == "focus" or Unit == "focustarget" then
					F:RegisterEvent("PLAYER_FOCUS_CHANGED")
				elseif Unit == "pet" then
					F:RegisterEvent("UNIT_PET")
				elseif RawUnit == "party" or RawUnit == "raid" then
					F:RegisterEvent("GROUP_ROSTER_UPDATE")
					F:RegisterEvent("UPDATE_INSTANCE_INFO")
				elseif RawUnit == "boss" then
					F:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
				end
				if Unit ~= "player" then
					F:RegisterUnitEvent("UNIT_CONNECTION", Unit)
					F:RegisterUnitEvent("UNIT_NAME_UPDATE", Unit) -- Probably the only reliable way to update when every dataset is ready
				end
				
				if not Unit:match('%w+target') then
					F:RegisterUnitEvent('UNIT_ENTERED_VEHICLE', Unit)
					F:RegisterUnitEvent('UNIT_EXITED_VEHICLE', Unit)
					
					if Unit ~= "player" then
						F:RegisterUnitEvent('UNIT_PET', Unit)
					end
					
					F.UnitButton:SetAttribute('toggleForVehicle', true) -- Doesn't work for some reason?
					F.UnitButton:HookScript("OnAttributeChanged", function(self, attrName, value)						
						if attrName == 'unit' then
							UpdateUnit(self.Owner, value)
							self.Owner:ForceUpdate()
						end
					end)
				end
				
				-- We previously scanned for every unit target call
				-- But with this method, we have total control over what happens
				if Targets[Unit] then
					for _, source in pairs(Targets[Unit]) do
						if source then
							F:RegisterUnitEvent("UNIT_TARGET", source)
						end
					end
				end
			-- When a frame event fires, update modules
				F:SetScript("OnEvent", UnitFrame_OnEvent)
				-- To make the update instantaneous when the frame shows up
					F:SetScript("OnShow", UnitFrame_OnShow)
					F:SetScript("OnHide", UnitFrame_OnHide)

		-- Initial Update
			UnitFrame_ForceUpdate(F)
		-- Setup profile methods
			F.ForceUpdate = UnitFrame_ForceUpdate
			UF:RegisterUFUpdateFunction(F)
			UF:RegisterUFForClique(F.UnitButton)

		-- Prevent creation of movers for clustered unitframes
		-- We'll do it for those when the holder is being created
		if not Header then
			if RawUnit ~= "arena" and RawUnit ~= "party" and RawUnit ~= "boss" and RawUnit ~= "raid" then
				E:CreateMover(F, L[Unit .. "Frame"], nil, nil, nil, format("The %s Unitframe", E:firstToUpper(RawUnit)), "unitframes")
			end
		end

		return F
	end

	UF.Holders = {}
	UF.Holders.SortMethod = {
		["boss"] 	= {"TOPLEFT", 0, 15, "+", "-"},
		["arena"] 	= {"TOPLEFT", 0, 15, "+", "-"},
	}

	function UF:CreateUFHolder(Type, SX, SY)
		local Holder = CreateFrame("Frame", format("%sHolder", Type), E.Parent, "SecureHandlerStateTemplate")
		if SX and SY then Holder:SetSize(SX, SY) end
		Holder:SetPoint("CENTER", E.Parent, "CENTER")
		Holder.Type = Type
		Holder.Unitframes = {}

		self.Holders[Type] = Holder

		E:SetVisibilityHandler(Holder)
		E:HandleFrameInPetBattles(Holder)

		E:CreateMover(Holder, L[format("%sFrame", Type)], nil, nil, nil, format("The %s Unitframe Cluster", E:firstToUpper(Type)), "unitframes")

		return Holder
	end

	function UF:GetHolder(Unit)
		return self.Holders[Unit]
	end
	
	function UF:IsHolderVisible(Holder)
		return SecureCmdOptionParse(Holder.visibilityCondition) == "1"
	end
	
	-- @ ForceNormalize: (bool)	- Wether or not to force normal visiblity on force unshow, no matter what the dummy mode is currently set to
	function UF:OverrideSingleHolderVisibility(Holder, state, forceNormalize)
		
		local Holder = type(Holder) == 'string' and self:GetHolder(Holder) or Holder
		
		--if state == true and Holder.ForceMoverEnabled ~= false then
		if state == true then
			UnregisterStateDriver(Holder, "visible")
			RegisterStateDriver(Holder, "visible", "1")
			Holder.ForceShow = true
		else
			Holder.ForceShow = nil
			self:LoadHolderConfig(Holder.Type, forceNormalize)
		end
	end
	function UF:OverrideHolderVisibility(state, limit)
		self.HolderVisibilityOverride = state
		
		for k,v in pairs(self.Holders) do
			if limit then v = limit; k = 'Force' end
			
			if k ~= "SortMethod" and not v.HasHeader then
				self:OverrideSingleHolderVisibility(v, state)
			end
			
			if limit then break end
		end
	end
	
	function UF:LoadAllHolderConfig()
		for k,v in pairs(self.Holders) do
			if k ~= "SortMethod" then
				self:LoadHolderConfig(k)
			end
		end
	end
	
	function UF:GetClusterConfig(Name)
		return self.db.clusters[Name]
	end

	function UF:LoadHolderConfig(Unit, ForceNormalizeVisibility)
		local Holder = self:GetHolder(Unit)
		
		if not Holder then return end
		
		local HolderSortMethod = self.Holders.SortMethod[Unit]
		local Config = self.db.units[Unit]
		
		if not Config.enable then
			-- Straight up disables the mover
			Holder.ForceMoverEnabled = false
			UnregisterStateDriver(Holder, "visible")
			RegisterStateDriver(Holder, "visible", "0")
			
			return
		else
			Holder.ForceMoverEnabled = nil
		end
		
		local ClusterConfig = Config.UFInfo.cluster
		
		-- Check if the user currently wants the holders to stay visible
		Holder.visibilityCondition = ClusterConfig.visibilityCondition
		if (self.HolderVisibilityOverride == false or ForceNormalizeVisibility) and not Holder.ForceShow then
			UnregisterStateDriver(Holder, "visible")
			RegisterStateDriver(Holder, "visible", ClusterConfig.visibilityCondition)
		end
		
		if Unit ~= "boss" and Unit ~= "arena" then return end
			
		-- Override sort config
		HolderSortMethod[1] = ClusterConfig.perRow
		HolderSortMethod[2] = not ClusterConfig.inverseStartX
		HolderSortMethod[3] = not ClusterConfig.inverseStartY
		HolderSortMethod[4] = ClusterConfig.gapX
		HolderSortMethod[5] = ClusterConfig.gapY
		
		-- Apply changes
		self:SortUFHolderContents(Holder)
	end

	function UF:SortUFHolderContents(Holder)

		local PerRow, InverseStartX, InverseStartY, GapX, GapY = unpack(self.Holders.SortMethod[Holder.Type])
		-- Frames, Parent, Width, Height, SizeMult, PerRow, InverseStartX, InverseStartY, GapX, GapY, Ordered
		local totalWidth, totalHeight = E:SortFrames(Holder.Unitframes, Holder, nil, nil, nil, PerRow, InverseStartX, InverseStartY, GapX, GapY, true)

		Holder:SetSize(totalWidth, totalHeight)
		E:UpdateMoverDimensions(Holder)
	end

	function UF:AssignUFHolder(Holder, F)
		F:ClearAllPoints()
		F:SetPoint("CENTER", Holder, "CENTER")
		F:SetParent(Holder)

		tinsert(Holder.Unitframes, F)
	end
	
	local function SortByClass(a, b)
		if (a and b) and (a.SortValue_Class and b.SortValue_Class) then
			if a.SortValue_Class < b.SortValue_Class then
				return true
			elseif a.SortValue_Class > b.SortValue_Class then
				return false
			end
		end
	end
	
	local function SortByGroup(a, b)
		if ( IsInRaid() ) then			
			if ( not a or not b ) then
				return false
			end
			
			-- local subgroup1 = a.SortValue_Subgroup
			-- local subgroup2 = b.SortValue_Subgroup
			
			-- if ( subgroup1 and subgroup2 and subgroup1 ~= subgroup2 ) then
				-- return subgroup1 < subgroup2;
			-- end
			
			if (a.SortValue_Subgroup < b.SortValue_Subgroup) then
			   return true
			elseif (a.SortValue_Subgroup > b.SortValue_Subgroup) then
				return false
			else
				  return a.SortValue_Rank > b.SortValue_Rank
			end
			
			
		end
		
		--Fallthrough: Sort by order in Raid window.
		return a.UnitNum < b.UnitNum
	end
	
	local function SortByDefault(a, b)
		return a.UnitNum < b.UnitNum
	end
	

-------------------------------------------------------------------------------------------------------------------------------------
-- Refactor end
-------------------------------------------------------------------------------------------------------------------------------------
function UF:UpdateDB()
	self.db = CO.db.profile.unitframe
	
	for _, Module in pairs(self.Modules) do
		Module.db = self.db
	end
end
function UF:Init()
	self:UpdateDB()
	
	if not CO.db.char.unitframe.enable then return end

	self:CreateUF("player")
	self:CreateUF("target")
	self:CreateUF("targettarget")
	self:CreateUF("focus")
	self:CreateUF("focustarget")
	self:CreateUF("pet")

	-- @TODO: Create a new type of UF Holder that dynamically can create unitframes on the fly when needed
	-- local HolderSortHandler = CreateFrame("Frame")
	-- HolderSortHandler:RegisterEvent("GROUP_ROSTER_UPDATE")
	-- HolderSortHandler:SetScript("OnEvent", UF.SortRaidHolder)
	
	local ArenaHolder, PartyHolder, BossHolder, RaidHolder, RaidFullHolder, MaintankHolder
	BossHolder 		= self:CreateUFHolder("boss")
	ArenaHolder 	= self:CreateUFHolder("arena")
	PartyHolder 	= self:CreateUFHolder("party")
	RaidHolder 		= self:CreateUFHolder("raid")
	RaidFullHolder 	= self:CreateUFHolder("raid40")
	MaintankHolder 	= self:CreateUFHolder("maintank")
	
	for i=1,5 do
		self:AssignUFHolder(BossHolder, self:CreateUF("boss" .. i))
		self:AssignUFHolder(ArenaHolder, self:CreateUF("arena" .. i))
	end
	
	self.RegisteredHeaderClusters = {
		-- Arena doesn't work with that
		-- ['arena'] = {
			-- ['Holder'] = ArenaHolder,
			-- ['GroupSize'] = 1,
			-- ['LiteralName'] = "arena",
			-- ['Unit'] = "arena",
			-- ['ConfigKey'] = "arena",
			-- ['Attributes'] = {
				-- showParty 		= true,
				-- showRaid 		= false,
				-- showPlayer 		= true,
				-- showSolo 		= false, -- Setting this to true somehow results in a f*ed up header
			-- }
		-- },
		['party'] = {
			['Holder'] = PartyHolder,
			['GroupSize'] = 1,
			['LiteralName'] = "party",
			['Unit'] = "party",
			['ConfigKey'] = "party",
			['Attributes'] = {
				showParty 		= true,
				showRaid 		= false,
				showPlayer 		= false,
				showSolo 		= false, -- Setting this to true somehow results in a f*ed up header
			}
		},
		['raid'] = {
			['Holder'] = RaidHolder,
			['GroupSize'] = 4,
			['LiteralName'] = "raid",
			['Unit'] = "raid",
			['ConfigKey'] = "raid",
			['Attributes'] = {
				showParty 		= false,
				showRaid 		= true,
				showPlayer 		= true,
				showSolo 		= false, -- Setting this to true somehow results in a f*ed up header
			}
		},
		['raid40'] = {
			['Holder'] = RaidFullHolder,
			['GroupSize'] = 8,
			['LiteralName'] = "raid40",
			['Unit'] = "raid",
			['ConfigKey'] = "raid40",
			['Attributes'] = {
				showParty 		= false,
				showRaid 		= true,
				showPlayer 		= true,
				showSolo 		= false, -- Setting this to true somehow results in a f*ed up header
			}
		},
		
		-- Special Frames
		['maintank'] = {
			['Holder'] = MaintankHolder,
			['GroupSize'] = 1,
			['LiteralName'] = "MainTanks",
			['Unit'] = "",
			['ConfigKey'] = "maintank",
			['Attributes'] = {
				showParty 		= false,
				showRaid 		= true,
				showPlayer 		= true,
				showSolo 		= false, -- Setting this to true somehow results in a f*ed up header
				groupFilter		= "MAINTANK",
			}
		},
	}
	local Headers = self.RegisteredHeaderClusters
	
	-- Creates all headers
	for k,v in pairs(Headers) do
		if not v.Frames then
			v.Frames = {}
		end
		local WriteFilter
		for i=1, v.GroupSize do
			
			if not v.Attributes.groupFilter then
				WriteFilter = true
			end
			if WriteFilter then
				v.Attributes.groupFilter = tostring(i)
			end
			v.Attributes.groupingOrder = tostring(i)
			
			v.Frames[i] = self.Headers:Create(v.LiteralName .. "_" .. i, v.ConfigKey, i, v.Holder, v.Unit, v.Attributes)
		end
		
		v.Holder.HasHeader = true
		self.Headers:LoadConfig(v)
	end

	self:LoadConfig()
	self:LoadAllHolderConfig()
	
	-- Handler for eventless units
	PeriodicUnitUpdate:SetScript('OnUpdate', function(self, elapsed)
		self.elapsed = (self.elapsed or 0) + elapsed
		
		if self.elapsed > 0.5 then
			-------------------------
				for _, frame in pairs(self) do
					if type(frame) == 'table' then
						if frame.ForceUpdate and frame:IsVisible() then
							UF_Update(frame, "OnUpdate")
						end
					end
				end
			-------------------------
			
			self.elapsed = 0
		end
	end)
	
	local Blizzard = E:LoadModule('Blizzard')
	Blizzard:RemoveUnitframes()
end

E:AddModule("Unitframes", UF)
