local E, L = unpack(CUI) -- Engine
local CO, CD = E:LoadModules("Config", "Config_Dialog")

local Module = {}

function Module:Disable()
	CD.Options.args.blizzard = nil
end

function Module:Enable()
	CD.Options.args.blizzard = {
		name =  CD:GetNewFeatureString("Blizzard"),
		type = 'group',
		order = CD:GetAutoSortIndex(),
		childGroups = "tab",
		args = {
			talkingHead = {
				order = 1,
				type = 'group',
				name =  "Talking Head",
				get = function(info) return CO.db.profile.blizzard.talkingHead[ info[#info] ] end,
				set = function(info, value) CO.db.profile.blizzard.talkingHead[ info[#info] ] = value; E:LoadModule("Blizzard_TalkingHeadUI"):LoadConfig() end,
				args = {
					positionHeader = {
						type = "header",
						order = 10,
						name = L["Positioning"],
					},
					generalHeader = {
						type = "header",
						order = 20,
						name = L["General"],
					},
					scale = {
						type = "range",
						order = 21,
						name = L["Scale"],
						min = 0.3, max = 3,
					},
				},
			},
			infoFrame = {
				order = 5,
				type = 'group',
				name = "Info Frame",
				get = function(info) return CO.db.profile.blizzard.infoFrame[ info[#info] ] end,
				set = function(info, value) CO.db.profile.blizzard.infoFrame[ info[#info] ] = value; end,
				args = {
					positionHeader = {
						type = "header",
						order = 10,
						name = L["Positioning"],
					},
				},
			},
			uiWidget = {
				order = 10,
				type = 'group',
				name = "UI Widget",
				get = function(info) return CO.db.profile.blizzard.uiWidget[ info[#info] ] end,
				set = function(info, value) CO.db.profile.blizzard.uiWidget[ info[#info] ] = value; end,
				args = {
					positionHeader = {
						type = "header",
						order = 10,
						name = L["Positioning"],
					},
				},
			},
			durabilityFrame = {
				order = 12.5,
				type = 'group',
				name = "Durability Frame",
				get = function(info) return CO.db.profile.blizzard.durabilityFrame[ info[#info] ] end,
				set = function(info, value) CO.db.profile.blizzard.durabilityFrame[ info[#info] ] = value; end,
				args = {
					positionHeader = {
						type = "header",
						order = 10,
						name = L["Positioning"],
					},
				},
			},
			mirrorBar = {
				order = 15,
				type = 'group',
				name = L["MirrorTimer"],
				get = function(info) return CO.db.profile.blizzard.mirrorBar[ info[#info] ] end,
				set = function(info, value) CO.db.profile.blizzard.mirrorBar[ info[#info] ] = value; end,
				args = {
					positionHeader = {
						type = "header",
						order = 10,
						name = L["Positioning"],
					},
				},
			},
			dungeonReadyDialog = {
				order = 20,
				type = 'group',
				name = CD:GetNewFeatureString("Dungeon Popup"),
				get = function(info) return CO.db.profile.blizzard.dungeonReadyDialog[ info[#info] ] end,
				set = function(info, value) CO.db.profile.blizzard.dungeonReadyDialog[ info[#info] ] = value; E:LoadModule("Blizzard_LFGDungeonReadyDialog"):LoadConfig() end,
				args = {
					timerHeader = {
						type = "header",
						order = 5,
						name = "Time Left Bar",
					},
					enable = {
						type = "toggle",
						order = 10,
						name = L["Enable"],
						desc = "Enables a timer bar for the queue popup",
					},
					hideBigWigs = {
						type = "toggle",
						order = 11,
						name = "Hide BigWigs",
						desc = "When enabled, the BigWigs queue timer will be hidden [When installed. There's no internal BigWigs option for that]",
					},
				},
			},
			chatBubbles = {
				order = 30,
				type = 'group',
				name = CD:GetNewFeatureString("Chat Bubbles"),
				get = function(info) return CO.db.profile.blizzard.chatBubbles[ info[#info] ] end,
				set = function(info, value) CO.db.profile.blizzard.chatBubbles[ info[#info] ] = value; E:LoadModule("Blizzard_ChatBubbles"):LoadConfig() end,
				childGroups = "tab",
				args = {
					enable = {
						type = "toggle",
						order = 5,
						name = L['Enable'],
						desc = "Enables a skinned variant of Chat Bubbles",
						get = function(info) return CO.db.char.blizzard.chatBubbles[ info[#info] ] end,
						set = function(info, value) CO.db.char.blizzard.chatBubbles[ info[#info] ] = value; CD:ShowNotification("CHARACTERSETTING_NOTIFICATION") end,
					},
					
					generalGroup = {
						type = "group",
						order = 10,
						name = "General",
						disabled = function() return not CO.db.char.blizzard.chatBubbles.enable end,
						get = function(info) return CO.db.profile.blizzard.chatBubbles[ info[#info] ] end,
						args = {
							borderSize = {
								type = "range",
								order = 10,
								name = "Border Size",
								min = 0, max = 10, step = 0.1,
								set = function(info, value)
									if value == 0 then
										value = 0.1
									end
									CO.db.profile.blizzard.chatBubbles.borderSize = value
									E:LoadModule("Blizzard_ChatBubbles"):LoadConfig()
								end,
							},
							borderColor = {
								name = L["BorderColor"],
								type = "color",
								hasAlpha = true,
								order = 11,
								get = function(info)
									local c = CO.db.profile.blizzard.chatBubbles.borderColor or {0,0,0,1}
									return c[1], c[2], c[3], c[4]
								end,
								set = function(info, r, g, b, a)
									if not CO.db.profile.blizzard.chatBubbles.borderColor then CO.db.profile.blizzard.chatBubbles.borderColor = {0,0,0,1} end
									local c = CO.db.profile.blizzard.chatBubbles.borderColor
									c[1], c[2], c[3], c[4] = r, g, b, a
									E:LoadModule("Blizzard_ChatBubbles"):LoadConfig()
								end,
								hidden = true,
							},
							backgroundColor = {
								name = L["BackgroundColor"],
								type = "color",
								hasAlpha = true,
								order = 12,
								get = function(info)
									local c = CO.db.profile.blizzard.chatBubbles.backgroundColor or {0,0,0,1}
									return c[1], c[2], c[3], c[4]
								end,
								set = function(info, r, g, b, a)
									if not CO.db.profile.blizzard.chatBubbles.backgroundColor then CO.db.profile.blizzard.chatBubbles.backgroundColor = {0,0,0,1} end
									local c = CO.db.profile.blizzard.chatBubbles.backgroundColor
									c[1], c[2], c[3], c[4] = r, g, b, a
									E:LoadModule("Blizzard_ChatBubbles"):LoadConfig()
								end,
							},
						},
					},
					
				},
			},
		},
		
	}

	for k,v in pairs(CD:GetMoverOptions("TalkingHeadFrameMover", 11)) do
		CD.Options.args.blizzard.args.talkingHead.args[k] = v
	end
	for k,v in pairs(CD:GetMoverOptions("UIWidgetTopCenterContainerFrameMover", 11, true)) do
		CD.Options.args.blizzard.args.infoFrame.args[k] = v
	end
	for k,v in pairs(CD:GetMoverOptions("UIWidgetBelowMinimapContainerFrameMover", 11, true)) do
		CD.Options.args.blizzard.args.uiWidget.args[k] = v
	end
	for k,v in pairs(CD:GetMoverOptions("MirrorTimerHolderMover", 11, true)) do
		CD.Options.args.blizzard.args.mirrorBar.args[k] = v
	end
	for k,v in pairs(CD:GetMoverOptions("DurabilityFrameMover", 11, true)) do
		CD.Options.args.blizzard.args.durabilityFrame.args[k] = v
	end
	
	Fonts = {{Path = "db.profile.blizzard.chatBubbles.name", Order = 200, GroupName = L["Name"]}, {Path = "db.profile.blizzard.chatBubbles.text", Order = 300, GroupName = "Text"}}
	for k,v in pairs(CD:GetFontOptions(Fonts)) do
		CD.Options.args.blizzard.args.chatBubbles.args[k] = v
	end
	
	CD.Options.args.blizzard.args.chatBubbles.args[L['Name']].disabled = function()
		return not CO.db.char.blizzard.chatBubbles.enable
	end
	CD.Options.args.blizzard.args.chatBubbles.args["Text"].disabled = function()
		return not CO.db.char.blizzard.chatBubbles.enable
	end
end

CD:RegisterConfigModule(Module, 'Advanced')