local E, L = unpack(CUI) -- Engine
local CO, CD = E:LoadModules("Config", "Config_Dialog")

local Module = {}

function Module:Disable()
	CD.Options.args.layout = nil
end

function Module:Enable()
	
	CD.Options.args.layout = {
	type = "group",
	name = "Layout",
	order = 99999,
	childGroups = "tab",
	get = function(info) return CO.db.profile.system[ info[#info] ] end,
	set = function(info, value) CO.db.profile.system[ info[#info] ] = value; E:LoadModule("Layout"):LoadConfig() end,
	args = {
		layoutFrames = {
			type = "group",
			name = L["LayoutTextures"],
			order = 1,
			args = {
				enableTop = {
					order = 2,
					type = "toggle",
					name = L["TopBar"],
				},
				enableBottom = {
					order = 3,
					type = "toggle",
					name = L["BottomBar"],
				},
				enableBottomLeft = {
					order = 4,
					type = "toggle",
					name = L["BottomLeftCorner"],
				},
				enableBottomRight = {
					order = 5,
					type = "toggle",
					name = L["BottomRightCorner"],
				},
			},
		},
		
		layoutFonts = {
			type = "group",
			name = L["LayoutFonts"],
			order = 2,
			childGroups = "tab",
			args = {
				layoutUpdateFrequency = {
					type = "range",
					order = 11,
					min = 0,
					max = 60,
					softMin = 0,
					softMax = 1,
					name = L["SystemUpdateFrequency"],
					desc = "Controls the amount of time (in seconds) that has to pass between each update of the FPS and Latency.\n0 = Real-Time\nYou can use hard values from 0 to 60\n\nThis feature may impact the games performance when using high update frequencies (Low values)",
					get = function() return CO.db.profile.system.layoutUpdateFrequency end,
					set = function(info, value) CO.db.profile.system.layoutUpdateFrequency = value end,
					disabled = function() return (not CO.db.profile.layout.fps.enable and not CO.db.profile.layout.ping.enable) end,
				},
				coordsUpdateFrequency = {
					type = "range",
					order = 12,
					min = 0,
					max = 60,
					softMin = 0,
					softMax = 1,
					name = L["CoordinatesUpdateFrequency"],
					desc = "Controls the amount of time (in seconds) that has to pass between each update.\n0 = Real-Time\nYou can use hard values from 0 to 60\n\nThis feature may impact the games performance when using high update frequencies (Low values)",
					get = function() return CO.db.profile.system.coordsUpdateFrequency end,
					set = function(info, value) CO.db.profile.system.coordsUpdateFrequency = value end,
					disabled = function() return (not CO.db.profile.layout.coordx.enable and not CO.db.profile.layout.coordy.enable) end,
				},
				layoutHeader = {
					order = 99,
					type = "header",
					name = L["AdvancedOptions"],
				},
			},
		},
	},
}


	local Fonts = {{Path = "db.profile.layout.fps", Order = 100, GroupName = L["FPS"]}, {Path = "db.profile.layout.ping", Order = 200, GroupName = L["Latency"]}, {Path = "db.profile.layout.zone", Order = 300, GroupName = L["Zone"]}, {Path = "db.profile.layout.coordx", Order = 400, GroupName = L["CoordinatesX"]},{Path = "db.profile.layout.coordy", Order = 500, GroupName = L["CoordinatesY"]}}
	for k,v in pairs(CD:GetFontOptions(Fonts)) do
		CD.Options.args.layout.args.layoutFonts.args[k] = v
	end
end

CD:RegisterConfigModule(Module, 'Advanced')